"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const spinnies_1 = tslib_1.__importDefault(require("@trufflesuite/spinnies"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const lodash_1 = require("lodash");
const node_fs_1 = require("node:fs");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const spec_analytics_1 = require("../../analytics/spec-analytics");
const config_default_1 = tslib_1.__importDefault(require("../../constants/config.default"));
const git_providers_1 = require("../../constants/git-providers");
const languages_1 = require("../../constants/languages");
const messages_1 = tslib_1.__importStar(require("../../constants/messages"));
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const helpers_1 = require("../../lib/helpers");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const cmds_json_1 = require("../../logs/cmds.json");
const internal_json_1 = require("../../logs/internal.json");
const docs_schema_1 = require("../../schemas/docs.schema");
const artifact_service_1 = require("../../services/artifact.service");
const build_portal_url_1 = require("../../services/build-portal-url");
const enhanced_api_spec_service_1 = require("../../services/enhanced-api-spec/enhanced-api-spec.service");
const hooks_service_1 = require("../../services/hooks.service");
const liblab_config_service_1 = require("../../services/liblab-config/liblab-config.service");
const pr_service_1 = tslib_1.__importDefault(require("../../services/pr.service"));
const spec_format_service_1 = require("../../services/spec-format.service");
const spec_service_1 = require("../../services/spec/spec.service");
const validate_service_1 = require("../../services/validate.service");
const workflows_service_1 = require("../../services/workflows.service");
const spec_enhancer_1 = tslib_1.__importDefault(require("../../spec-enhancer"));
const authenticate_user_1 = require("../../utils/authenticate-user");
const docs_utils_1 = require("../../utils/docs.utils");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const exec_promise_1 = require("../../utils/exec-promise");
const file_utils_1 = require("../../utils/file.utils");
const liblab_version_1 = require("../../utils/liblab-version");
const normalize_sdk_names_1 = require("../../utils/normalize-sdk-names");
const sleep_1 = require("../../utils/sleep");
const get_docs_response_table_1 = require("../../utils/tables/get-docs-response-table");
const text_utils_1 = require("../../utils/text.utils");
const base_command_1 = require("../base-command");
const outputDir = node_path_1.default.join(process.cwd(), './output');
class Build extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.spinners = new spinnies_1.default();
        this.createPr = false;
        this.hooksService = new hooks_service_1.HooksService();
        this.specService = new spec_service_1.SpecService();
        this.liblabClient = new liblab_client_provider_1.default(config);
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command build' },
            commandType: Build,
        });
        this.workflowService = new workflows_service_1.WorkflowsService(this.liblabClient);
    }
    // TODO: remove when code is split and error handling is managed in the method scope
    async catch(err) {
        var _a, _b;
        const artifactIds = (_b = (_a = this.build) === null || _a === void 0 ? void 0 : _a.artifacts) === null || _b === void 0 ? void 0 : _b.map((artifact) => artifact.id);
        const logEventParams = {
            buildSdkResult: this.build,
            error: err,
            reason: err.message,
            status: 'failed',
            artifact_ids: artifactIds,
        };
        await super.catch(err, logEventParams);
        this.terminateSpinners(err.message);
    }
    async enhanceApiSpec({ build, liblabConfig, spec, api, }) {
        var _a, _b;
        const { enhancedSpec, languages } = await (0, spec_enhancer_1.default)(build, liblabConfig, spec);
        const enrichedSpecService = new enhanced_api_spec_service_1.EnhancedApiSpecService(this.analytics);
        let enhancementStatus = 'SUCCESS';
        let enhancementError;
        try {
            await enrichedSpecService.exportLocalEnhancedApiSpec(liblabConfig, enhancedSpec, languages, this.approveAllOperations);
        }
        catch (error) {
            enhancementStatus = 'FAIL';
            enhancementError = error;
        }
        await this.liblabClient.postEnhancedSpecArtifact(liblabConfig, build.id, {
            status: enhancementStatus,
            error: enhancementError,
        });
        if (enhancementStatus === 'FAIL') {
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_TO_ENHANCE_API_SPEC, enhancementError);
        }
        if (((_a = liblabConfig.integrations) === null || _a === void 0 ? void 0 : _a.enhancedApiSpec) && ((_b = liblabConfig.publishing) === null || _b === void 0 ? void 0 : _b.githubOrg)) {
            const buildUrl = (0, build_portal_url_1.buildPortalUrl)({ apiSlug: api.slug, buildId: build.id });
            await enrichedSpecService.publish(liblabConfig, buildUrl, enhancedSpec, languages);
        }
    }
    async getLiblabConfig() {
        const isPayingCustomer = await this.isPayingCustomer();
        const liblabConfigHelper = new liblab_config_service_1.LiblabConfigService();
        const rawLiblabConfig = await liblabConfigHelper.getRawLiblabConfig(this.remoteConfigUrl);
        const validatedConfig = liblabConfigHelper.validate(rawLiblabConfig);
        const configWithMcp = liblabConfigHelper.handleMcp(validatedConfig, isPayingCustomer);
        const liblabConfig = liblabConfigHelper.modify(configWithMcp, this.languageFlag);
        return { liblabConfig, rawLiblabConfig };
    }
    async initFlagsOnCommand() {
        const { flags } = await this.parseCommand();
        this.skipValidation = flags['skip-validation'];
        this.shouldApproveDocs = flags['approve-docs'];
        this.approveAllOperations = flags.yes;
        this.createPr = flags.pr;
        this.org = flags.org;
        this.remoteConfigUrl = flags['liblab-config'];
        this.languageFlag = flags.language;
        this.platform = flags.platform;
    }
    async monitorBuildProgress(liblabConfig, build) {
        var _a;
        let buildInProgress = { ...build };
        await this.initSpinners(liblabConfig);
        let currentPollAttempt = 0;
        while (((_a = buildInProgress.artifacts) === null || _a === void 0 ? void 0 : _a.some((artifact) => artifact.status === 'IN_PROGRESS')) &&
            buildInProgress.status === 'IN_PROGRESS') {
            await (0, sleep_1.sleep)(3000);
            currentPollAttempt += 1;
            buildInProgress = await this.liblabClient.getBuild(buildInProgress.id, currentPollAttempt);
            if (!buildInProgress.artifacts) {
                throw new Error('Internal server error');
            }
            (0, helpers_1.debugLog)(`commands/build#run.monitorBuildProgress: current pool attempt: ${currentPollAttempt}`, this.build);
            const normalizedArtifacts = this.normalizeArtifacts(buildInProgress);
            this.updateSpinnersAfterBuildResponse(normalizedArtifacts);
        }
        return buildInProgress;
    }
    /**
     * wrapper for testing proposes
     */
    async parseCommand() {
        return this.parse(Build);
    }
    async run() {
        var _a, _b, _c, _d, _e;
        await this.initFlagsOnCommand();
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        this.analytics.updateUser(currentUser);
        const { rawLiblabConfig, liblabConfig } = await this.getLiblabConfig();
        if (liblabConfig.languages.length === 0) {
            this.log(cmds_json_1.BUILD.NO_LANGUAGES_SELECTED);
            return;
        }
        this.analytics.addConfigData(liblabConfig);
        const subscriptionAllowsBuild = await this.verifySubscriptionStatusAllowsBuild(currentUser, liblabConfig);
        if (!subscriptionAllowsBuild) {
            return;
        }
        const eventProperties = {
            ...liblabConfig,
            is_free_build: config_default_1.default.specFilePath === liblabConfig.specFilePath,
        };
        this.analytics.updateEventProperties(eventProperties);
        const rawSpec = await this.specService.getSpecFile(liblabConfig);
        const specFileFormat = (0, file_utils_1.identifyFileContentFormat)(rawSpec, internal_json_1.UNSUPPORTED_API_SPEC_FORMAT);
        const specFormatService = new spec_format_service_1.SpecFormatService(rawSpec, specFileFormat);
        const format = specFormatService.getSpecFormat();
        this.analytics.updateEventProperties({
            ...eventProperties,
            pagination: (0, spec_analytics_1.usesPagination)(liblabConfig),
            streaming: (0, spec_analytics_1.usesStreaming)(liblabConfig),
            spec_format: format,
        });
        const { apiName, apiVersion } = specFormatService.getSpecTitleAndVersion(liblabConfig);
        const modifiedLiblabConfig = {
            ...liblabConfig,
            apiName,
            apiVersion,
        };
        if (format === spec_format_service_1.AllowedSpecFormats.Postman && !this.skipValidation) {
            this.log(cmds_json_1.BUILD.POSTMAN_VALIDATION_NOT_SUPPORTED);
            this.skipValidation = true;
        }
        const spec = await this.specService.customize(rawSpec, modifiedLiblabConfig, specFileFormat, format);
        const hooks = await this.hooksService.getReadStream();
        const api = await this.liblabClient.createApi({
            liblabConfig: modifiedLiblabConfig,
            specFormat: specFileFormat,
            specContent: spec,
        });
        if (!this.skipValidation) {
            const shouldBuildContinue = await this.validateSpec(api.id, liblabConfig);
            if (!shouldBuildContinue) {
                return;
            }
        }
        if (modifiedLiblabConfig.workflows) {
            await this.workflowService.uploadWorkFlows(liblabConfig, api.id);
        }
        const build = await this.liblabClient.postBuild({
            apiId: api.id,
            apiName: modifiedLiblabConfig.apiName,
            ...(hooks && { hooks }),
            liblabConfig: modifiedLiblabConfig,
            rawLiblabConfig,
        });
        this.build = build;
        this.log(messages_1.default.INFO.BUILD_INVITE_TO_PORTAL({
            apiSlug: api.slug,
            buildId: build.id,
        }));
        if (!build.artifacts || build.artifacts.length === 0) {
            const reason = messages_1.default.ERROR.BUILD_NO_SDKS_GENERATED;
            throw new Error(reason);
        }
        this.initSpinners(modifiedLiblabConfig);
        const finalBuildState = await this.monitorBuildProgress(modifiedLiblabConfig, build);
        this.build = finalBuildState;
        if (!finalBuildState.artifacts) {
            (0, helpers_1.debugLog)(`Build artifacts were not generated in buildId: ${finalBuildState.id}`, finalBuildState);
            throw new Error('Internal Liblab server error');
        }
        const artifactHelper = new artifact_service_1.ArtifactsService(finalBuildState.artifacts);
        await artifactHelper.downloadSdks();
        await artifactHelper.exportSdks(this.approveAllOperations, (_b = (_a = modifiedLiblabConfig.languageOptions) === null || _a === void 0 ? void 0 : _a.terraform) === null || _b === void 0 ? void 0 : _b.providerName);
        await this.summarizeBuildStatus(finalBuildState);
        await this.handleTsPostBuild(finalBuildState);
        // We are not setting up the mcp post build yet. This is a placeholder for future implementation if needed.
        // await this.handleMcpPostBuild(finalBuildState);
        if (artifactHelper.successfulSdks.length > 0) {
            artifactHelper.logBuiltSdks();
            (_c = finalBuildState.warnings) === null || _c === void 0 ? void 0 : _c.forEach((warning) => {
                this.log((0, text_utils_1.getColoredText)('yellow', 'Warning: ') + warning);
            });
        }
        if ((0, docs_utils_1.shouldGenerateApiDocs)(modifiedLiblabConfig) && artifactHelper.successfulDoc) {
            await this.publishDocs(artifactHelper.successfulDoc);
        }
        if (this.createPr) {
            const owner = this.org || ((_d = modifiedLiblabConfig.publishing) === null || _d === void 0 ? void 0 : _d.githubOrg);
            if (!owner) {
                throw new Error(cmds_json_1.PR.ERROR.MISSING_ORG);
            }
            const prHelper = new pr_service_1.default(modifiedLiblabConfig, this.liblabClient, owner, this.analytics, this.languageFlag, this.platform);
            await prHelper.run();
        }
        if ((_e = modifiedLiblabConfig.docs) === null || _e === void 0 ? void 0 : _e.includes(docs_schema_1.DocsOptions.EnhancedApiSpec)) {
            await this.enhanceApiSpec({
                spec,
                build: finalBuildState,
                artifactService: artifactHelper,
                liblabConfig: modifiedLiblabConfig,
                api,
            });
        }
        await this.logAfterBuildSubscriptionInformation(currentUser);
    }
    async validateSpec(apiId, liblabConfig) {
        var _a;
        const validator = new validate_service_1.ValidateService(apiId, liblabConfig);
        await validator.validateSpec();
        await validator.exportValidation();
        const diagnostics = validator.getViolations();
        if (this.approveAllOperations || diagnostics.length === 0) {
            return true;
        }
        validator.logViolations();
        const { runAnyway } = await inquirer_1.default.prompt({
            message: cmds_json_1.BUILD.SHOULD_BUILD_CONTINUE_QUESTION,
            name: 'runAnyway',
            type: 'confirm',
        });
        if (!runAnyway) {
            await ((_a = this.analytics) === null || _a === void 0 ? void 0 : _a.logEvent({
                reason: 'User chose not to run build after validation issues prompt',
                status: 'abort',
                validIssues: diagnostics,
            }));
            this.log(cmds_json_1.BUILD.EXIT_BUILD_PROCESS);
            return false;
        }
        this.log(cmds_json_1.BUILD.CONTINUE_BUILD_WITH_WARNINGS);
        return true;
    }
    async verifySubscriptionStatusAllowsBuild(currentUser, liblabConfig) {
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!subscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return false;
        }
        if (subscription.status === 'free') {
            if (subscription.remainingCredits === 0) {
                this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role, 'error'));
                return false;
            }
            const hasEnoughCredits = subscription.remainingCredits >= this.calculateRequestedCredits(liblabConfig);
            if (!hasEnoughCredits) {
                this.log(messages_1.default.ERROR.SUBSCRIPTION_NO_CREDITS(currentUser.role, subscription.remainingCredits, this.calculateRequestedCredits(liblabConfig)));
                return false;
            }
        }
        return true;
    }
    calculateRequestedCredits(liblabConfig) {
        return liblabConfig.languages.length;
    }
    async handleMcpPostBuild(build) {
        var _a;
        const mcpPath = node_path_1.default.join(outputDir, './mcp');
        const didMcpCreated = (0, node_fs_1.existsSync)(mcpPath);
        const mcpSdkArtifact = (_a = build.artifacts) === null || _a === void 0 ? void 0 : _a.find((artifact) => { var _a; return ((_a = artifact.sdk) === null || _a === void 0 ? void 0 : _a.language) === 'MCP' && artifact.status === 'SUCCESS' && artifact.artifactUrl; });
        if (!didMcpCreated || !mcpSdkArtifact) {
            return;
        }
        this.spinners.add('mcp-postbuild', { prefixColor: 'yellow', text: cmds_json_1.BUILD.MCP_POST_BUILD_TEXT });
        const { stderr } = await (0, exec_promise_1.execPromise)('npm install --package-lock-only && npm run setup', {
            cwd: mcpPath,
        });
        const warningRegex = /.*warning.*/i;
        if ((stderr === null || stderr === void 0 ? void 0 : stderr.includes('WARN')) || warningRegex.test(stderr) || !stderr) {
            this.spinners.succeed('mcp-postbuild', { prefixColor: 'green' });
        }
        else {
            this.spinners.fail('mcp-postbuild', { prefixColor: 'red', text: cmds_json_1.BUILD.MCP_POST_BUILD_TEXT });
            this.error(cmds_json_1.BUILD.HANDLE_TS_POST_BUILD_WARNING, { exit: false });
        }
    }
    async handleTsPostBuild(build) {
        var _a;
        const tsPath = node_path_1.default.join(outputDir, './typescript');
        const didTsCreated = (0, node_fs_1.existsSync)(tsPath);
        const tsSdkArtifact = (_a = build.artifacts) === null || _a === void 0 ? void 0 : _a.find((artifact) => {
            var _a;
            return ((_a = artifact.sdk) === null || _a === void 0 ? void 0 : _a.language) === 'TYPESCRIPT' &&
                artifact.status === 'SUCCESS' &&
                artifact.artifactUrl;
        });
        if (!didTsCreated || !tsSdkArtifact) {
            return;
        }
        this.spinners.add('ts-postbuild', { prefixColor: 'yellow', text: cmds_json_1.BUILD.TS_POST_BUILD_TEXT });
        const { stderr } = await (0, exec_promise_1.execPromise)('npm install --package-lock-only', {
            cwd: tsPath,
        });
        const warningRegex = /.*warning.*/i;
        if ((stderr === null || stderr === void 0 ? void 0 : stderr.includes('WARN')) || warningRegex.test(stderr) || !stderr) {
            this.spinners.succeed('ts-postbuild', { prefixColor: 'green' });
        }
        else {
            this.spinners.fail('ts-postbuild', { prefixColor: 'red', text: cmds_json_1.BUILD.TS_POST_BUILD_TEXT });
            this.error(cmds_json_1.BUILD.HANDLE_TS_POST_BUILD_WARNING, { exit: false });
        }
    }
    initSpinners(liblabConfig) {
        const jobLanguages = liblabConfig.languages.map((language) => normalize_sdk_names_1.sdkNameToUiNameMapper[language]);
        jobLanguages.forEach((job) => this.spinners.add(job, { prefixColor: 'yellow', text: `Building ${job}` }));
        const languagesWithLiblabVersion = liblabConfig.languages.map((lang) => ({
            language: lang,
            liblabVersion: (0, liblab_version_1.getLiblabVersion)(liblabConfig, lang),
        }));
        const requestedLanguagesWithDocSupport = languagesWithLiblabVersion.filter((lang) => languages_1.languagesWithDocsSupport.some((docLang) => docLang.language === lang.language && docLang.liblabVersion === lang.liblabVersion));
        if (requestedLanguagesWithDocSupport.length > 0 && (0, docs_utils_1.shouldGenerateApiDocs)(liblabConfig)) {
            this.spinners.add('API Documentation', {
                prefixColor: 'yellow',
                text: `Building API Documentation`,
            });
        }
    }
    async isPayingCustomer() {
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!subscription || subscription.status === 'free' || subscription.status !== 'active') {
            return false;
        }
        return true;
    }
    async logAfterBuildSubscriptionInformation(currentUser) {
        const afterBuildSubscription = await this.liblabClient.getSubscriptionStatus();
        if (!afterBuildSubscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return;
        }
        if (afterBuildSubscription.status === 'free') {
            if (afterBuildSubscription.remainingCredits === 0) {
                this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role));
            }
            else {
                this.log(messages_1.default.SUCCESS.REMAINING_CREDITS(afterBuildSubscription.remainingCredits));
            }
        }
    }
    async logSuccessBuildToMixpanel({ artifacts, duration, id, }) {
        const artifactIds = artifacts === null || artifacts === void 0 ? void 0 : artifacts.map((artifact) => artifact.id);
        await this.analytics.logEvent({
            artifacts,
            build_duration: duration,
            build_id: id,
            artifact_ids: artifactIds,
            status: 'succeed',
        });
    }
    normalizeArtifacts(build) {
        return build.artifacts.reduce((acc, { doc, error, sdk, status }) => {
            if (!sdk && !doc) {
                return acc;
            }
            const objectToPush = sdk
                ? {
                    job: normalize_sdk_names_1.sdkNameToUiNameMapper[sdk.language],
                    status,
                    ...(error && { error }),
                }
                : {
                    job: 'API Documentation',
                    status,
                };
            acc.push(objectToPush);
            return acc;
        }, []);
    }
    async promptShouldDocsBeApproved() {
        const res = await inquirer_1.default.prompt({
            message: cmds_json_1.BUILD.APPROVE_DOCS_CONFIRMATION,
            name: 'shouldDocsBeApprovedPrompt',
            type: 'confirm',
        });
        return res.shouldDocsBeApprovedPrompt;
    }
    async publishDocs(builtDoc) {
        const docsResponseTable = (0, get_docs_response_table_1.getDocsResponseTable)(builtDoc);
        core_1.ux.stdout(docsResponseTable.render());
        const { approvedDocUrl, previewSlug } = builtDoc.doc;
        const canDocBeApproved = previewSlug && !approvedDocUrl;
        if (!canDocBeApproved) {
            return;
        }
        const shouldDocsBeApproved = this.shouldApproveDocs ||
            this.approveAllOperations ||
            (await this.promptShouldDocsBeApproved());
        if (!shouldDocsBeApproved) {
            return;
        }
        const approvedDoc = await this.liblabClient.approveDocs(previewSlug);
        (0, helpers_1.debugLog)('approved Doc', approvedDoc);
        if (!approvedDoc.approvedDocUrl) {
            this.log(cmds_json_1.BUILD.DOCS_APPROVED_WITHOUT_URL);
            return;
        }
        const dynamicApprovedDocMessage = (0, lodash_1.replace)(cmds_json_1.BUILD.DOCS_PUBLISHED, '{approvedDocUrl}', approvedDoc.approvedDocUrl);
        this.log(dynamicApprovedDocMessage);
    }
    async summarizeBuildStatus({ artifacts, duration, id }) {
        const failedArtifacts = artifacts === null || artifacts === void 0 ? void 0 : artifacts.filter((artifact) => artifact.status === 'FAIL');
        if (failedArtifacts === null || failedArtifacts === void 0 ? void 0 : failedArtifacts.length) {
            this.throwBuildFailedError(failedArtifacts);
        }
        await this.logSuccessBuildToMixpanel({ artifacts, duration, id });
    }
    terminateSpinners(errorMsg) {
        const isSpinnersRunning = this.spinners.hasActiveSpinners();
        (0, helpers_1.debugLog)('Spinners terminated due to error', errorMsg);
        if (isSpinnersRunning) {
            this.spinners.stopAll();
        }
    }
    throwBuildFailedError(failedArtifacts) {
        const failedLanguageNames = failedArtifacts === null || failedArtifacts === void 0 ? void 0 : failedArtifacts.filter((artifact) => { var _a; return (_a = artifact === null || artifact === void 0 ? void 0 : artifact.sdk) === null || _a === void 0 ? void 0 : _a.language; }).map((artifact) => normalize_sdk_names_1.sdkNameToUiNameMapper[artifact.sdk.language]);
        const isDocFailed = failedArtifacts === null || failedArtifacts === void 0 ? void 0 : failedArtifacts.some((artifact) => artifact.doc);
        throw new Error((0, messages_1.withContactSupport)(messages_1.default.ERROR.BUILD_FAILED_SDKS(failedLanguageNames, isDocFailed)));
    }
    updateSpinnersAfterBuildResponse(normalizedArtifacts) {
        normalizedArtifacts.forEach(({ error, job, status }) => {
            const spinnerToUpdate = this.spinners.pick(job);
            const spinnerShouldUpdate = (spinnerToUpdate === null || spinnerToUpdate === void 0 ? void 0 : spinnerToUpdate.status) === 'spinning' || status === 'IN_PROGRESS';
            // update only spinning spinners
            if (!spinnerShouldUpdate) {
                return;
            }
            if (status === 'FAIL') {
                const errorMsg = error ? ` ${error}` : '';
                this.spinners.fail(job, { prefixColor: 'red', text: `${job} failed${errorMsg}` });
            }
            else if (status === 'SUCCESS') {
                this.spinners.succeed(job, { prefixColor: 'green', text: `${job} built` });
            }
        });
    }
}
Build.description = 'Build SDK(s) and Docs for your API.';
Build.examples = ['liblab build'];
Build.flags = {
    'approve-docs': core_1.Flags.boolean({
        default: false,
        description: 'Approve docs without review',
    }),
    language: core_1.Flags.string({
        char: 'l',
        description: 'Sdk language to build',
    }),
    'liblab-config': core_1.Flags.string({
        description: 'Liblab config path',
    }),
    org: core_1.Flags.string({
        char: 'o',
        description: 'The organization to create the PR in',
    }),
    pr: core_1.Flags.boolean({
        default: false,
        description: 'Create PR for built sdks',
    }),
    'skip-validation': core_1.Flags.boolean({
        default: false,
        description: 'Ignore validation and attempt to build specs',
    }),
    test: core_1.Flags.boolean({
        hidden: true,
    }),
    yes: core_1.Flags.boolean({
        char: 'y',
        default: false,
        description: 'Approve all operations',
    }),
    platform: core_1.Flags.string({
        char: 'p',
        options: git_providers_1.gitProviders,
    }),
};
// This tells oclif to hide this command from the help menu
Build.hidden = true;
Build.tag = command_tags_enum_1.default.BUILD;
exports.default = Build;
