"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const config_default_1 = tslib_1.__importDefault(require("../../constants/config.default"));
const api_1 = require("../../services/liblab-config-v2-service/api");
const artifact_1 = require("../../services/liblab-config-v2-service/artifact");
const build_logger_1 = require("../../services/liblab-config-v2-service/build-logger");
const build_ux_1 = require("../../services/liblab-config-v2-service/build-ux");
const constants_1 = require("../../services/liblab-config-v2-service/constants");
const docs_integration_workflow_1 = require("../../services/liblab-config-v2-service/docs-integrations/docs-integration-workflow");
const hooks_1 = require("../../services/liblab-config-v2-service/hooks");
const liblab_config_service_1 = require("../../services/liblab-config-v2-service/liblab-config.service");
const logging_events_1 = require("../../services/liblab-config-v2-service/logging/logging-events");
const ocliff_config_1 = require("../../services/liblab-config-v2-service/ocliff-config");
const pr_workflow_1 = require("../../services/liblab-config-v2-service/pr/pr-workflow");
const spec_1 = require("../../services/liblab-config-v2-service/spec");
const sleep_1 = require("../../utils/sleep");
const base_command_1 = require("../base-command");
class Build extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.api = api_1.LiblabApi.getInstance();
        this.liblabConfigV2Service = new liblab_config_service_1.LiblabConfigV2Service();
        this.ux = new build_ux_1.BuildUx();
        this.logger = new build_logger_1.BuildLogger(Build, this.config, this.argv);
    }
    async run() {
        var _a;
        const { flags } = await this.setup();
        const buildParams = await this.prepareBuild(flags);
        const { build, api } = await this.build(buildParams);
        const completedBuild = await this.pollBuildStatus(build);
        await this.processSuccessfulBuilds({
            build: completedBuild,
            liblabConfig: buildParams.liblabConfig,
        });
        if (flags.pr) {
            await this.createPr(flags.platform, buildParams.liblabConfig);
        }
        if ((_a = buildParams.liblabConfig.docs) === null || _a === void 0 ? void 0 : _a.includes('enhancedApiSpec')) {
            docs_integration_workflow_1.DocsIntegrations.run(completedBuild, buildParams.liblabConfig, api, buildParams.spec);
        }
    }
    async build(buildParams) {
        try {
            const { build, api } = await this.api.startBuild(buildParams);
            this.ux.startBuild(api, build);
            return { build, api };
        }
        catch {
            this.error(`Failed to create build`);
        }
    }
    async createPr(platform, liblabConfig) {
        try {
            await pr_workflow_1.PrWorkflow.createPrForEachLangauge(platform, liblabConfig);
        }
        catch {
            this.error(`Failed to create pr`);
        }
    }
    /**
     * Creates an overrides object for the liblab config based on provided flags.
     * Only includes properties when flag values are defined to avoid overriding
     * config values with undefined.
     *
     * @param flags - Command line flags object containing optional values
     * @returns Partial LibLabConfigV2Input with only defined flag values
     */
    getOverrides(flags) {
        const overrides = {};
        if (flags.language) {
            overrides.languages = [flags.language];
        }
        if (flags.org) {
            overrides.publishing = {
                ...overrides.publishing,
                githubOrg: flags.org,
            };
        }
        if (flags.platform) {
            overrides.publishing = {
                ...overrides.publishing,
                platform: flags.platform,
            };
        }
        if (flags.org) {
            overrides.publishing = {
                ...overrides.publishing,
                githubOrg: flags.org,
            };
        }
        if (flags.platform) {
            overrides.publishing = {
                ...overrides.publishing,
                platform: flags.platform,
            };
        }
        return overrides;
    }
    async pollBuildStatus(build) {
        var _a;
        const { id } = build;
        const { artifacts = [] } = build;
        let updatedBuild = build;
        while ((_a = updatedBuild.artifacts) === null || _a === void 0 ? void 0 : _a.some((artifact) => artifact.status === artifact_1.ArtifactStatus.IN_PROGRESS)) {
            updatedBuild = await this.api.getBuild(id);
            this.ux.updateBuildsProgress(updatedBuild.artifacts || []);
            await (0, sleep_1.sleep)(constants_1.BUILD_STATUS_POLL_INTERVAL);
        }
        if (artifacts.some((artifact) => artifact.status === artifact_1.ArtifactStatus.FAIL)) {
            this.error('Failed to generate SDKs.');
        }
        return updatedBuild;
    }
    async prepareBuild(flags) {
        var _a;
        const overrides = this.getOverrides(flags);
        const isPayingCustomer = await this.api.isPayingCustomer();
        const { rawLiblabConfig, liblabConfig } = await this.liblabConfigV2Service.getLiblabConfig((_a = flags.liblabConfig) !== null && _a !== void 0 ? _a : constants_1.DEFAULT_LIBLAB_CONFIG_PATH, overrides, isPayingCustomer);
        const { spec, specFileFormat } = await (0, spec_1.getSpecFile)(liblabConfig);
        const hooks = await (0, hooks_1.getHookContents)();
        return { liblabConfig, rawLiblabConfig, spec, specFileFormat, hooks };
    }
    async processSuccessfulBuilds({ build, liblabConfig, }) {
        var _a;
        try {
            const successfulArtifacts = ((_a = build.artifacts) === null || _a === void 0 ? void 0 : _a.filter((artifact) => artifact.status === artifact_1.ArtifactStatus.SUCCESS).map((artifact) => new artifact_1.Artifact(artifact))) || [];
            await Promise.all(successfulArtifacts.map((artifact) => artifact.download()));
            this.ux.buildComplete(successfulArtifacts);
            this.logger.logEvent({
                config: liblabConfig,
                artifacts: successfulArtifacts.map((artifact) => artifact.artifactResponse),
                build,
                status: logging_events_1.EventStatus.SUCCEED,
                duration: null,
                isFreeBuild: config_default_1.default.specFilePath === liblabConfig.specFilePath,
            });
        }
        catch (error) {
            this.error(`Failed to download SDKs: ${error instanceof Error ? error.message : 'Unknown error'}`);
        }
    }
    async setup() {
        ocliff_config_1.OCliffConfig.getInstance().setConfig(this.config);
        const { flags } = await this.parse(Build);
        try {
            await this.api.login();
        }
        catch {
            this.error('Failed to login');
        }
        return {
            flags: {
                ...flags,
                skipValidation: flags.yes || flags['skip-validation'],
                approveDocs: flags.yes || flags['approve-docs'],
                pr: flags.pr,
                liblabConfig: flags['liblab-config'],
            },
        };
    }
}
Build.flags = {
    'approve-docs': core_1.Flags.boolean({
        default: false,
        description: 'Approve docs without review',
    }),
    language: core_1.Flags.string({
        char: 'l',
        description: 'Sdk language to build',
    }),
    'liblab-config': core_1.Flags.string({
        description: 'Liblab config path',
    }),
    org: core_1.Flags.string({
        char: 'o',
        description: 'The organization to create the PR in',
    }),
    pr: core_1.Flags.boolean({
        default: false,
        description: 'Create PR for built sdks',
    }),
    'skip-validation': core_1.Flags.boolean({
        default: false,
        description: 'Ignore validation and attempt to build specs',
    }),
    test: core_1.Flags.boolean({
        hidden: true,
        default: false,
    }),
    yes: core_1.Flags.boolean({
        char: 'y',
        default: false,
        description: 'Approve all operations',
    }),
    platform: core_1.Flags.string({
        char: 'p',
        default: 'github',
        options: ['github', 'gitlab', 'bitbucket'],
    }),
};
exports.default = Build;
