"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const spinnies_1 = tslib_1.__importDefault(require("@trufflesuite/spinnies"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const node_child_process_1 = require("node:child_process");
const node_util_1 = require("node:util");
const messages_1 = tslib_1.__importDefault(require("../../constants/messages"));
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const cmds_json_1 = require("../../logs/cmds.json");
const authenticate_user_1 = require("../../utils/authenticate-user");
const base_command_1 = require("../base-command");
const execAsync = (0, node_util_1.promisify)(node_child_process_1.exec);
const BUNDLE_SPINNER_NAME = 'bundle';
const DEFAULT_OUTPUT_SPEC = 'bundledSpec.yaml';
class Bundle extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.spinners = new spinnies_1.default();
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    async catch(error) {
        var _a;
        if (this.spinners.hasActiveSpinners()) {
            this.spinners.fail(BUNDLE_SPINNER_NAME, {
                prefixColor: 'red',
                text: 'Error: Unable to bundle the specs.',
            });
        }
        const errorMessage = ((_a = error.stderr) === null || _a === void 0 ? void 0 : _a.includes('Error was generated by the bundler rule.'))
            ? error.stderr.split('Error was generated by the bundler rule.')[0]
            : error.stderr || error.message;
        const logEventParams = { status: 'failed', reason: errorMessage };
        await super.catch(error, logEventParams);
    }
    async run() {
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!subscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return false;
        }
        if (subscription.status === 'free' && subscription.remainingCredits === 0) {
            this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role), 'error');
            return false;
        }
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command bundle' },
            commandType: Bundle,
            currentUser,
        });
        const { args, flags } = await this.parse(Bundle);
        const outputSpec = flags.output || DEFAULT_OUTPUT_SPEC;
        if (await fs_extra_1.default.pathExists(outputSpec)) {
            const { overwriteSpec } = await inquirer_1.default.prompt({
                message: cmds_json_1.BUNDLE.OVERWRITE_SPEC_PROMPT.replace('{spec}', outputSpec),
                name: 'overwriteSpec',
                type: 'confirm',
            });
            if (!overwriteSpec) {
                await this.analytics.logEvent({
                    reason: 'User chose not to overwrite spec file while running bundle',
                    status: 'abort',
                });
                this.log(cmds_json_1.BUNDLE.EXIT_BUNDLE);
                return false;
            }
        }
        this.spinners.add(BUNDLE_SPINNER_NAME, { prefixColor: 'yellow', text: 'Bundling spec' });
        const command = await this.createRedoclyBundleCommand(args.spec, outputSpec);
        const { stdout, stderr } = await execAsync(command);
        this.spinners.succeed(BUNDLE_SPINNER_NAME, { prefixColor: 'green', text: 'Spec bundled' });
        await this.analytics.logEvent({ status: 'succeed' });
        if (stdout) {
            this.log(stdout);
        }
        if (stderr) {
            // Redocly will output successful outputs as stderr
            this.log(stderr);
        }
        return true;
    }
    async createRedoclyBundleCommand(spec, outputFile) {
        const outputFileFlag = `-o ${outputFile}`;
        return `npx --yes @redocly/cli@latest bundle ${spec} ${outputFileFlag}`;
    }
}
Bundle.args = {
    spec: core_1.Args.string({
        description: 'The path to the spec file for bundling',
        required: true,
    }),
};
Bundle.description = 'Bundle an OpenAPI spec file to resolve all external references';
Bundle.examples = ['liblab bundle spec.yaml', 'liblab bundle spec.yaml -o bundledSpec.yaml'];
Bundle.flags = {
    output: core_1.Flags.string({
        char: 'o',
        description: 'The name and extension of the output OpenAPI spec',
        helpValue: '<output-spec-file>',
    }),
};
Bundle.tag = command_tags_enum_1.default.SPEC_MANAGEMENT;
Bundle.usage = 'SPEC [-o <output-spec-file>]';
exports.default = Bundle;
