"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const zod_1 = require("zod");
const paths_default_1 = require("../../constants/paths.default");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const liblab_config_1 = require("../../schemas/liblab.config");
const liblab_config_service_1 = require("../../services/liblab-config/liblab-config.service");
const text_utils_1 = require("../../utils/text.utils");
const base_command_1 = require("../base-command");
class ConfigCommand extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.liblabConfigService = new liblab_config_service_1.LiblabConfigService();
    }
    async run() {
        const { args: { key, value }, } = await this.parse(ConfigCommand);
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command config' },
            commandType: ConfigCommand,
        });
        const rawLiblabConfig = (await this.liblabConfigService.getRawLiblabConfig());
        if (value) {
            try {
                if (!(await this.setValue(rawLiblabConfig, key, value))) {
                    return;
                }
                this.log((0, text_utils_1.getColoredText)('green', `Successfully set ${key} to ${value}`));
            }
            catch (error) {
                this.log((0, text_utils_1.getColoredText)('yellow', error.message));
            }
            return;
        }
        const result = this.getValue(rawLiblabConfig, key);
        if (result === undefined) {
            await this.handleKeyNotFound(key, `The '${key}' key is not specified in your config file.`);
            return;
        }
        await this.analytics.logEvent({ status: 'succeed' });
        if (typeof result === 'object') {
            if (Array.isArray(result)) {
                this.log(`${result.join(', ')}`);
                return;
            }
            const message = `Please select a specific key from:\n${Object.keys(result).join('\n')}`;
            this.log((0, text_utils_1.getColoredText)('yellow', message));
            return;
        }
        this.log(result);
    }
    findCaseInsensitiveKey(obj, key) {
        return Object.keys(obj).find((k) => k.toLowerCase() === key.toLowerCase());
    }
    getValue(obj, propertyPath) {
        const keys = propertyPath.split('.');
        return keys.reduce((acc, curr) => {
            if (!acc)
                return;
            const innerKey = Object.keys(acc).find((k) => k.toLowerCase() === curr.toLowerCase());
            return innerKey === undefined ? undefined : acc[innerKey];
        }, obj);
    }
    async handleKeyNotFound(key, msg) {
        const message = msg || `'${key}' is not a supported configuration option.`;
        this.log((0, text_utils_1.getColoredText)('yellow', message));
        await this.analytics.logEvent({
            error: message,
            status: 'warn',
        });
    }
    async setConfigValueByPath(obj, propertyPath, lastKey, value, keys) {
        let currentSchema = liblab_config_1.liblabSchema.shape;
        let currentConfig = obj;
        for (const key of keys) {
            const innerKey = this.findCaseInsensitiveKey(currentSchema, key);
            if (!innerKey) {
                await this.handleKeyNotFound(propertyPath);
                return false;
            }
            if (!currentConfig[innerKey]) {
                currentConfig[innerKey] = {};
            }
            currentConfig = currentConfig[innerKey];
            const innerSchema = this.unwrapSchema(currentSchema[innerKey]);
            if (innerSchema instanceof zod_1.z.ZodObject) {
                currentSchema = innerSchema.shape;
            }
            else {
                await this.handleKeyNotFound(propertyPath);
                return false;
            }
        }
        const innerKey = this.findCaseInsensitiveKey(currentSchema, lastKey);
        if (!innerKey) {
            await this.handleKeyNotFound(propertyPath);
            return false;
        }
        currentConfig[innerKey] = this.tryParseWithCoercion(currentSchema[innerKey], value);
        node_fs_1.default.writeFileSync(paths_default_1.liblabConfigPath, JSON.stringify(obj, null, 2));
        return true;
    }
    async setRootOrFirstLevelValue(obj, lastKey, value) {
        const currentSchema = liblab_config_1.liblabSchema.shape;
        const newConfigObject = { ...obj };
        const innerKey = this.findCaseInsensitiveKey(currentSchema, lastKey);
        if (innerKey) {
            const schema = currentSchema[innerKey];
            newConfigObject[innerKey] = this.tryParseWithCoercion(schema, value);
            node_fs_1.default.writeFileSync(paths_default_1.liblabConfigPath, JSON.stringify(newConfigObject, null, 2));
            return true;
        }
        for (const [childKey, childValue] of Object.entries(currentSchema)) {
            const childSchema = this.unwrapSchema(childValue);
            if (childSchema instanceof zod_1.z.ZodObject) {
                const childInnerKey = this.findCaseInsensitiveKey(childSchema.shape, lastKey);
                if (!childInnerKey) {
                    continue;
                }
                const schema = childSchema.shape[childInnerKey];
                newConfigObject[childKey][childInnerKey] = this.tryParseWithCoercion(schema, value);
                node_fs_1.default.writeFileSync(paths_default_1.liblabConfigPath, JSON.stringify(newConfigObject, null, 2));
                return true;
            }
        }
        await this.handleKeyNotFound(lastKey);
        return false;
    }
    async setValue(obj, propertyPath, value) {
        const keys = propertyPath.split('.');
        const lastKey = keys.pop() || propertyPath;
        // For properties without path (e.g. sdkName), we look in the root or first level of the schema
        if (keys.length === 0) {
            return this.setRootOrFirstLevelValue(obj, lastKey, value);
        }
        // For properties with path (e.g. customizations.apiDev), we look in the nested schema
        return this.setConfigValueByPath(obj, propertyPath, lastKey, value, keys);
    }
    tryParseWithCoercion(schema, value) {
        const baseSchema = this.unwrapSchema(schema);
        if (baseSchema instanceof zod_1.z.ZodNumber) {
            const number = Number(value);
            if (!Number.isNaN(number)) {
                return number;
            }
            throw new TypeError('Value must be a number');
        }
        else if (baseSchema instanceof zod_1.z.ZodBoolean) {
            if (value.toLowerCase() === 'true' || value.toLowerCase() === 'false') {
                return value.toLowerCase() === 'true';
            }
            throw new TypeError('Value must be a boolean (true/false)');
        }
        else if (baseSchema instanceof zod_1.z.ZodArray) {
            const innerType = baseSchema._def.type;
            if (innerType instanceof zod_1.z.ZodEnum) {
                return value.split(/[\s,]+/).map((item) => {
                    const trimmed = item.trim();
                    if (!innerType._def.values.includes(trimmed)) {
                        throw new TypeError(`Invalid value: "${trimmed}". Valid values are: ${innerType._def.values.join(', ')}`);
                    }
                    return trimmed;
                });
            }
            return value.split(/[\s,]+/).map((item) => item.trim());
        }
        else if (baseSchema instanceof zod_1.z.ZodObject) {
            throw new TypeError('You cannot set a value to an object');
        }
        else if (baseSchema instanceof zod_1.z.ZodEnum) {
            if (!baseSchema._def.values.includes(value)) {
                throw new TypeError(`Invalid value: "${value}". Valid values are: ${baseSchema._def.values.join(', ')}`);
            }
            return value;
        }
        else if (baseSchema instanceof zod_1.z.ZodString) {
            return value;
        }
        throw new TypeError('Unsupported property type');
    }
    unwrapSchema(schema) {
        let currentSchema = schema;
        while (currentSchema instanceof zod_1.z.ZodOptional ||
            currentSchema instanceof zod_1.z.ZodDefault ||
            currentSchema instanceof zod_1.z.ZodEffects) {
            currentSchema =
                currentSchema instanceof zod_1.z.ZodEffects
                    ? currentSchema._def.schema
                    : currentSchema._def.innerType;
        }
        return currentSchema;
    }
}
ConfigCommand.args = {
    key: core_1.Args.string({
        description: 'Configuration key to retrieve',
        required: true,
    }),
    value: core_1.Args.string({
        description: 'Configuration value to set',
        required: false,
    }),
};
ConfigCommand.description = 'Get or set a specific configuration value from liblab config';
ConfigCommand.examples = [
    'Get a top level value: liblab config sdkName',
    'Get a nested value: liblab config customizations.retry.enabled',
    'Set a string: liblab config apiName test-api',
    'Set an array: liblab config languages "python, php"',
];
ConfigCommand.tag = command_tags_enum_1.default.BUILD;
exports.default = ConfigCommand;
