"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const languages_1 = tslib_1.__importDefault(require("../../constants/languages"));
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const helpers_1 = require("../../lib/helpers");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const liblab_config_service_1 = require("../../services/liblab-config/liblab-config.service");
const authenticate_user_1 = require("../../utils/authenticate-user");
const hooks_1 = require("../../utils/hooks");
const normalize_sdk_names_1 = require("../../utils/normalize-sdk-names");
const base_command_1 = require("../base-command");
class HooksAdd extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.liblabClient = new liblab_client_provider_1.default(config);
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command hooks:add' },
            commandType: HooksAdd,
        });
    }
    async catch(err) {
        core_1.ux.action.stop('failed');
        const reason = 'An error occurred while downloading and extracting hooks';
        this.log(reason);
        await super.catch(err);
    }
    async run() {
        var _a, _b;
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.updateUser(currentUser);
        const languagesWithVersions = await this.getLanguagesWithVersions();
        const shouldOverwrite = await this.shouldOverwriteHooks(languagesWithVersions);
        if (!shouldOverwrite) {
            this.log('Aborting');
            return;
        }
        const hooks = await this.getHooks();
        this.extractHooksToFs(languagesWithVersions, hooks);
        await ((_b = this.analytics) === null || _b === void 0 ? void 0 : _b.logEvent({ status: 'succeed' }));
    }
    extractHooksToFs(languagesWithVersions, hooks) {
        this.log(`Adding hooks for ${this.printHookLanguages(languagesWithVersions)}`);
        try {
            core_1.ux.action.start('Unzipping hooks');
            languagesWithVersions.forEach((lwv) => {
                hooks.extract(lwv);
            });
            core_1.ux.action.stop();
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to unzip hooks', error);
            core_1.ux.action.stop();
            throw new Error('Could not unzip hooks');
        }
        this.log(`Added hooks for ${this.printHookLanguages(languagesWithVersions)}`);
    }
    async getHooks() {
        try {
            core_1.ux.action.start('Downloading hooks');
            const hooks = await hooks_1.Hooks.getHooksFromS3();
            core_1.ux.action.stop();
            return hooks;
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to fetch hooks from s3 bucket', error);
            core_1.ux.action.stop();
            throw new Error('Could not fetch hooks');
        }
    }
    async getLanguagesWithVersions() {
        var _a, _b, _c, _d;
        const { flags } = await this.parse(HooksAdd);
        const { language, 'liblab-config': liblabConfigPath } = flags;
        const liblabConfigHelper = new liblab_config_service_1.LiblabConfigService();
        const rawLiblabConfig = await liblabConfigHelper.getRawLiblabConfig(liblabConfigPath);
        const liblabConfig = liblabConfigHelper.validate(rawLiblabConfig);
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.updateEventProperties(liblabConfig);
        (_b = this.analytics) === null || _b === void 0 ? void 0 : _b.addConfigData(liblabConfig);
        if (language) {
            return [
                {
                    language: language,
                    version: ((_d = (_c = liblabConfig === null || liblabConfig === void 0 ? void 0 : liblabConfig.languageOptions) === null || _c === void 0 ? void 0 : _c[language]) === null || _d === void 0 ? void 0 : _d.liblabVersion) || '',
                },
            ];
        }
        return liblabConfig.languages.map((configLanguage) => {
            var _a, _b;
            return ({
                language: configLanguage,
                version: ((_b = (_a = liblabConfig === null || liblabConfig === void 0 ? void 0 : liblabConfig.languageOptions) === null || _a === void 0 ? void 0 : _a[configLanguage]) === null || _b === void 0 ? void 0 : _b.liblabVersion) || '',
            });
        });
    }
    printHookLanguages(languagesWithVersions) {
        const languageToPrint = languagesWithVersions.map((lwv) => normalize_sdk_names_1.sdkNameToUiNameMapper[lwv.language]);
        return languageToPrint.join(', ').replace(/, ([^,]*)$/, ' and $1');
    }
    async shouldOverwriteHooks(langaugeWithVersion) {
        if (!hooks_1.Hooks.alreadyExist(langaugeWithVersion)) {
            return true;
        }
        const { shouldOverwrite } = await inquirer_1.default.prompt({
            message: 'This will overwrite your existing hooks code, are you sure you want to continue?',
            name: 'shouldOverwrite',
            type: 'confirm',
        });
        return shouldOverwrite;
    }
}
HooksAdd.description = 'Bootstrap an SDKs hooks.';
HooksAdd.examples = ['liblab hooks add', 'liblab hooks add -l typescript'];
HooksAdd.flags = {
    language: core_1.Flags.string({
        char: 'l',
        description: 'The language to add hooks for',
        options: languages_1.default,
        required: false,
    }),
    'liblab-config': core_1.Flags.string({
        description: 'Remote liblab config',
    }),
};
HooksAdd.tag = command_tags_enum_1.default.BUILD;
exports.default = HooksAdd;
