"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const languages_1 = tslib_1.__importDefault(require("../../constants/languages"));
const paths_default_1 = require("../../constants/paths.default");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const fs_1 = require("../../lib/fs");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const authenticate_user_1 = require("../../utils/authenticate-user");
const normalize_sdk_names_1 = require("../../utils/normalize-sdk-names");
const base_command_1 = require("../base-command");
const path = require("node:path");
class HooksRemove extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    async run() {
        var _a;
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command hooks:remove' },
            commandType: HooksRemove,
            currentUser,
        });
        const languageFlag = await this.getLanguageFlag();
        const hooksPaths = this.getHooksPaths(languageFlag);
        const languageUiName = languageFlag && normalize_sdk_names_1.sdkNameToUiNameMapper[languageFlag];
        // unfortunately there is no prettier way to do this, forEach will complain about async
        for (const hookPath of hooksPaths) {
            await this.removeHooksForLanguage(hookPath, languageUiName, languageFlag, this.analytics);
        }
        await ((_a = this.analytics) === null || _a === void 0 ? void 0 : _a.logEvent({ status: 'succeed' }));
    }
    getHooksPaths(languageFlag) {
        if (!languageFlag) {
            return [paths_default_1.customPlanModifiersPath, paths_default_1.hooksDir];
        }
        const hookPaths = [path.join(paths_default_1.hooksDir, languageFlag)];
        // TODO: we should separate `hooks remove` and `custom-plan-modifiers remove` commands
        if (languageFlag === 'terraform') {
            hookPaths.push(paths_default_1.customPlanModifiersPath);
        }
        return hookPaths;
    }
    async getLanguageFlag() {
        const { flags: { language }, } = await this.parse(HooksRemove);
        return language;
    }
    async removeHooksForLanguage(hookPath, languageUiName, languageFlag, analytics) {
        if (!(0, fs_1.exists)(hookPath)) {
            const errorMessage = languageFlag
                ? `Hooks do not exist for ${languageUiName}`
                : 'Hooks do not exist';
            this.log(errorMessage);
            await analytics.logEvent({ reason: errorMessage, status: 'failed' });
            return;
        }
        const { remove } = await inquirer_1.default.prompt({
            message: `This will remove your existing hooks code in ${hookPath}, are you sure you want to continue?`,
            name: 'remove',
            type: 'confirm',
        });
        if (!remove) {
            this.log('Aborting');
            return;
        }
        if (!(0, fs_1.rm)(hookPath)) {
            const errorMessage = 'An error occurred while removing the hooks';
            this.log(errorMessage);
            await analytics.logEvent({ reason: errorMessage, status: 'failed' });
            return;
        }
        this.log(languageUiName
            ? `Successfully removed hooks for ${languageUiName}`
            : 'Successfully removed hooks');
    }
}
HooksRemove.description = `Remove an SDK's hooks.`;
HooksRemove.examples = ['liblab hooks remove', 'liblab hooks remove -l typescript'];
HooksRemove.flags = {
    language: core_1.Flags.string({
        char: 'l',
        description: 'The language to remove hooks for',
        options: languages_1.default,
        required: false,
    }),
};
HooksRemove.tag = command_tags_enum_1.default.BUILD;
exports.default = HooksRemove;
