"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const build_yml_1 = tslib_1.__importDefault(require("../../constants/actions/build.yml"));
const config_default_1 = tslib_1.__importDefault(require("../../constants/config.default"));
const messages_1 = tslib_1.__importDefault(require("../../constants/messages"));
const paths_default_1 = require("../../constants/paths.default");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const fs_1 = require("../../lib/fs");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const authenticate_user_1 = require("../../utils/authenticate-user");
const file_utils_1 = require("../../utils/file.utils");
const text_utils_1 = require("../../utils/text.utils");
const base_command_1 = require("../base-command");
class Init extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.libLabConfig = structuredClone(config_default_1.default);
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    async run() {
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!this.hasSubscription(subscription, currentUser)) {
            return false;
        }
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command init' },
            commandType: Init,
            currentUser,
        });
        const { flags: { default: useDefaultConfig }, } = await this.parse(Init);
        if (!useDefaultConfig) {
            this.log(`Let's build your SDK with a few simple steps!\n`);
        }
        const proceed = await this.hasExistingConfig();
        if (!proceed) {
            return false;
        }
        await this.handleSpecFileInput();
        if (!useDefaultConfig) {
            await this.handleLanguages();
            await this.handleProjectName();
            await this.handleLicence();
            await this.handleMcpConfig(subscription);
        }
        await this.addGithubAction();
        node_fs_1.default.writeFileSync(paths_default_1.liblabConfigPath, JSON.stringify(this.libLabConfig, null, 2));
        this.log('Successfully created liblab.config.json file.');
        await this.handleHooks();
        await this.handleBuild();
        await this.analytics.logEvent({ status: 'succeed' });
        if (subscription.status === 'free' && subscription.remainingCredits) {
            this.log(messages_1.default.SUCCESS.REMAINING_CREDITS(subscription.remainingCredits));
        }
        return true;
    }
    async addGithubAction() {
        const { flags: { github: addGithubAction }, } = await this.parse(Init);
        if (addGithubAction) {
            try {
                if (!(0, fs_1.exists)(node_path_1.default.join(process.cwd(), '.github'))) {
                    node_fs_1.default.mkdirSync(node_path_1.default.join(process.cwd(), '.github'));
                }
                const workflowsPath = node_path_1.default.join(process.cwd(), '.github/workflows');
                if (!(0, fs_1.exists)(workflowsPath)) {
                    node_fs_1.default.mkdirSync(workflowsPath);
                }
                const buildPath = node_path_1.default.join(process.cwd(), '.github/workflows/build.yml');
                if ((0, fs_1.exists)(buildPath)) {
                    this.log('.github/workflows/build.yml already exists. Skipping creation.');
                }
                else {
                    node_fs_1.default.writeFileSync(buildPath, build_yml_1.default);
                    this.log('Added Github Action to .github/workflows/build.yml customize it as you wish.');
                }
            }
            catch (error) {
                const reason = "Couldn't create .github/workflows/build.yml. Please create it manually.";
                await this.analytics.logEvent({ error, reason, status: 'failed' });
                this.error(reason);
            }
        }
    }
    getLanguageConfiguration(language) {
        // Definition for the non-default languages.
        switch (language) {
            case 'csharp':
            case 'php':
            case 'swift': {
                return {
                    ignoreFiles: [],
                    liblabVersion: '2',
                    renameIllegalModelProperties: true,
                    sdkVersion: '1.0.0',
                };
            }
            case 'terraform': {
                return {
                    providerGoModuleName: 'github.com/go-provider-sdk',
                    providerName: 'provider_name',
                    providerVersion: '1.0.0',
                };
            }
            default: {
                throw new Error('Invalid language');
            }
        }
    }
    async handleBuild() {
        const { flags: { build: buildWithoutAsking, default: useDefaultConfig, force: forceConfigOveride }, } = await this.parse(Init);
        // If the user wants to build without asking, we just build.
        if (buildWithoutAsking) {
            await this.config.runCommand('build', forceConfigOveride ? ['--yes'] : []);
            return;
        }
        // If the user wants to use the default config, and not --force, we don't build.
        if (useDefaultConfig) {
            return;
        }
        let buildSdk = false;
        const response = await inquirer_1.default.prompt({
            message: 'Do you want to build the SDK now?',
            name: 'buildSdk',
            type: 'confirm',
        });
        if (!response)
            return;
        buildSdk = response.buildSdk;
        if (!buildSdk) {
            this.log(`You can run ${(0, text_utils_1.getColoredText)('green', 'liblab build')} to build the SDK.`);
            return;
        }
        await this.config.runCommand('build');
    }
    async handleHooks() {
        const { flags: { hooks: addHooks }, } = await this.parse(Init);
        if (addHooks && !(0, fs_1.exists)(node_path_1.default.join(process.cwd(), 'hooks'))) {
            await this.config.runCommand('hooks:add');
        }
    }
    async handleLanguages() {
        const { languages } = await inquirer_1.default.prompt([
            {
                type: 'checkbox',
                name: 'languages',
                message: 'Select languages for SDK generation',
                choices: [
                    { value: 'python', name: ' Python' },
                    { value: 'typescript', name: ' TypeScript' },
                    { value: 'go', name: ' Go' },
                    { value: 'java', name: ' Java' },
                    { value: 'csharp', name: ' C#' },
                    { value: 'php', name: ' PHP' },
                ],
                validate(answer) {
                    if (answer.length === 0) {
                        return 'You must choose at least one language.';
                    }
                    return true;
                },
                pageSize: 8,
            },
        ]);
        this.populateLanguagesConfiguration(languages);
    }
    async handleLicence() {
        var _a;
        const { licence } = await inquirer_1.default.prompt([
            {
                type: 'list',
                name: 'licence',
                message: 'Select the project licence',
                choices: [
                    { value: 'MIT', name: ' MIT' },
                    { value: 'GPL3', name: ' GPL3' },
                    { value: 'APACHE', name: ' APACHE' },
                    { value: 'NONE', name: ' NONE' },
                    { value: 'CUSTOM', name: ' CUSTOM' },
                ],
                validate(answer) {
                    if (!answer) {
                        return 'You must choose a licence.';
                    }
                    return true;
                },
            },
        ]);
        const licenceConfig = { type: licence };
        if (licence === 'CUSTOM') {
            const { licenceUrl } = await inquirer_1.default.prompt([
                {
                    type: 'input',
                    name: 'licenceUrl',
                    message: 'Please enter the URL for the custom licence',
                    validate(value) {
                        if (!value)
                            return 'Licence URL is required.';
                        if (!file_utils_1.urlRegex.test(value))
                            return 'Please enter a valid URL.';
                        return true;
                    },
                },
            ]);
            licenceConfig.url = licenceUrl;
        }
        if ((_a = this.libLabConfig.customizations) === null || _a === void 0 ? void 0 : _a.license) {
            this.libLabConfig.customizations.license = licenceConfig;
        }
    }
    async handleMcpConfig(subscription) {
        // Only for paying customers
        if (!this.isPayingCustomer(subscription))
            return;
        const { mcp } = await inquirer_1.default.prompt({
            message: 'Generate Model Context Protocol (MCP) server to enable SDK integration with AI models?',
            name: 'mcp',
            type: 'confirm',
        });
        this.libLabConfig.mcp = this.libLabConfig.mcp || { generate: false };
        this.libLabConfig.mcp.generate = mcp;
    }
    async handleProjectName() {
        const { projectName } = await inquirer_1.default.prompt([
            {
                type: 'input',
                name: 'projectName',
                message: 'Please enter your project name',
                validate(value) {
                    if (!value)
                        return 'Project name is required.';
                    return true;
                },
            },
        ]);
        this.libLabConfig.sdkName = projectName.toLowerCase().replaceAll(/\s+/g, '-');
        this.libLabConfig.apiName = `${this.libLabConfig.sdkName}-api`;
    }
    async handleSpecFileInput() {
        const { flags: { spec: specFilePath, default: useDefaultConfig }, } = await this.parse(Init);
        if (specFilePath && !this.isValidSpecFilePathOrUrl(specFilePath)) {
            this.error('Please provide a valid file path (.json, .yaml, or .yml) or URL');
        }
        if (specFilePath) {
            this.libLabConfig.specFilePath = specFilePath;
        }
        else {
            // Do not prompt if default config is used
            if (useDefaultConfig) {
                return;
            }
            const { specLocation } = await inquirer_1.default.prompt([
                {
                    type: 'input',
                    name: 'specLocation',
                    message: 'Location of your API specification file',
                    prefix: '',
                    suffix: ' (Press Enter to use our sample spec)',
                    validate: (value) => {
                        if (!value)
                            return true;
                        if (!this.isValidSpecFilePathOrUrl(value)) {
                            return 'Please provide a valid file path (.json, .yaml, or .yml) or URL';
                        }
                        return true;
                    },
                },
            ]);
            if (specLocation === null || specLocation === void 0 ? void 0 : specLocation.trim()) {
                this.libLabConfig.specFilePath = specLocation;
            }
        }
    }
    async hasExistingConfig() {
        const { flags: { force: forceConfigOveride, default: useDefaultConfig }, } = await this.parse(Init);
        // If config exists and force flag is not set
        if ((0, fs_1.exists)(paths_default_1.liblabConfigPath) && !forceConfigOveride) {
            // For default config, return false to prevent overwrite
            if (useDefaultConfig) {
                this.log('liblab.config.json already exists. Use --force with -y to overwrite.');
                return false;
            }
            // Otherwise prompt user for confirmation
            const { overwrite } = await inquirer_1.default.prompt({
                message: 'liblab.config.json already exists and will be overwritten. Do you want to proceed?',
                name: 'overwrite',
                type: 'confirm',
            });
            if (!overwrite) {
                this.log('Aborting');
                return false;
            }
        }
        return true;
    }
    hasSubscription(subscription, currentUser) {
        if (!subscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return false;
        }
        if (subscription.status === 'free' && subscription.remainingCredits === 0) {
            this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role, 'error'));
            return false;
        }
        return true;
    }
    isPayingCustomer(subscription) {
        if (!subscription || subscription.status === 'free' || subscription.status !== 'active') {
            return false;
        }
        return true;
    }
    isValidSpecFilePathOrUrl(value) {
        var _a;
        const fileExtension = (_a = value.split('.').pop()) === null || _a === void 0 ? void 0 : _a.toLowerCase();
        return ((node_fs_1.default.existsSync(value) && ['json', 'yaml', 'yml'].includes(fileExtension || '')) ||
            value.match(file_utils_1.urlRegex) !== null);
    }
    populateLanguagesConfiguration(languages) {
        this.libLabConfig.languages = languages;
        const filteredLanguageOptions = {};
        if (!this.libLabConfig.languageOptions) {
            return;
        }
        for (const lang of languages) {
            filteredLanguageOptions[lang] =
                lang in this.libLabConfig.languageOptions
                    ? this.libLabConfig.languageOptions[lang]
                    : this.getLanguageConfiguration(lang);
        }
        this.libLabConfig.languageOptions = filteredLanguageOptions;
    }
}
Init.description = 'Simplify SDK generation with a guided setup that prepares a LibLab configuration file.';
Init.examples = ['liblab init', 'liblab init --spec=spec.yaml', 'liblab init --github'];
Init.flags = {
    force: core_1.Flags.boolean({
        char: 'f',
        description: 'Overwrite the config file if it already exists',
    }),
    github: core_1.Flags.boolean({
        char: 'g',
        description: 'Add a Github Action to the directory',
    }),
    hooks: core_1.Flags.boolean({
        char: 'h',
        description: 'Bootstrap liblab hooks',
    }),
    spec: core_1.Flags.string({
        char: 's',
        description: 'The path or URL to your API spec',
    }),
    default: core_1.Flags.boolean({
        char: 'y',
        description: 'Use default configuration',
    }),
    build: core_1.Flags.boolean({
        char: 'b',
        description: 'Build the SDK after initialization',
    }),
    test: core_1.Flags.boolean({
        hidden: true,
    }),
};
Init.tag = command_tags_enum_1.default.BUILD;
exports.default = Init;
