"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const spinnies_1 = tslib_1.__importDefault(require("@trufflesuite/spinnies"));
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const node_child_process_1 = require("node:child_process");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const node_util_1 = require("node:util");
const messages_1 = tslib_1.__importDefault(require("../../constants/messages"));
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const cmds_json_1 = require("../../logs/cmds.json");
const authenticate_user_1 = require("../../utils/authenticate-user");
const base_command_1 = require("../base-command");
const execAsync = (0, node_util_1.promisify)(node_child_process_1.exec);
const MERGE_SPINNER_NAME = 'merge';
const DEFAULT_OUTPUT_SPEC = 'mergedSpec.yaml';
class Merge extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.spinners = new spinnies_1.default();
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    async catch(error) {
        var _a;
        const errorMessage = ((_a = error.stderr) === null || _a === void 0 ? void 0 : _a.replaceAll('running join', 'running merge')) || error.message;
        if (this.analytics) {
            await this.analytics.logEvent({ status: 'failed', reason: errorMessage });
        }
        if (this.spinners.hasActiveSpinners()) {
            this.spinners.fail(MERGE_SPINNER_NAME, {
                prefixColor: 'red',
                text: 'Error: Unable to merge the specs.',
            });
        }
        super.catch({ ...error, message: errorMessage });
    }
    async run() {
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!subscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return false;
        }
        if (subscription.status === 'free' && subscription.remainingCredits === 0) {
            this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role, 'error'));
            return false;
        }
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command merge' },
            commandType: Merge,
            currentUser,
        });
        const { args, flags } = await this.parse(Merge);
        const outputSpec = flags.output || DEFAULT_OUTPUT_SPEC;
        if (await fs_extra_1.default.pathExists(outputSpec)) {
            const { overwriteSpec } = await inquirer_1.default.prompt({
                message: cmds_json_1.MERGE.OVERWRITE_SPEC_PROMPT.replace('{spec}', outputSpec),
                name: 'overwriteSpec',
                type: 'confirm',
            });
            if (!overwriteSpec) {
                await this.analytics.logEvent({
                    reason: 'User chose not to overwrite spec file while running merge',
                    status: 'abort',
                });
                this.log(cmds_json_1.MERGE.EXIT_MERGE);
                return false;
            }
        }
        this.spinners.add(MERGE_SPINNER_NAME, { prefixColor: 'yellow', text: 'Merging specs' });
        const command = await this.createRedoclyJoinCommand(args.specs_folder, outputSpec, flags.main);
        const { stdout, stderr } = await execAsync(command);
        this.spinners.succeed(MERGE_SPINNER_NAME, { prefixColor: 'green', text: 'Specs merged' });
        await this.analytics.logEvent({ status: 'succeed' });
        if (stdout) {
            this.log(stdout.replaceAll('join processed', 'merge processed'));
        }
        if (stderr) {
            // Redocly will output successful outputs as stderr
            this.log(stderr.replaceAll('join processed', 'merge processed'));
        }
        return true;
    }
    async createRedoclyJoinCommand(specsFolder, outputFile, mainSpec) {
        const outputFileFlag = `-o ${outputFile}`;
        const specs = await this.getMatchingSpecs(specsFolder);
        if (!mainSpec) {
            const allSpecsPaths = specs.map((file) => `"${node_path_1.default.join(specsFolder, file)}"`).join(' ');
            return `npx --yes @redocly/cli@latest join ${allSpecsPaths} --without-x-tag-groups ${outputFileFlag}`;
        }
        const mainSpecPath = `${node_path_1.default.join(specsFolder, mainSpec)}`;
        const otherSpecsPaths = specs
            .filter((_, index) => index !== specs.indexOf(mainSpec))
            .map((file) => `"${node_path_1.default.join(specsFolder, file)}"`)
            .join(' ');
        return `npx --yes @redocly/cli@latest join ${mainSpecPath} ${otherSpecsPaths} --without-x-tag-groups ${outputFileFlag}`;
    }
    async getMatchingSpecs(specsFolder) {
        try {
            const specs = await fs_extra_1.default.readdir(specsFolder);
            return specs.filter((spec) => spec.match(/\.ya?ml$|\.json$/));
        }
        catch {
            throw new Error(`No specs with yaml, yml or json format found in folder ${specsFolder}`);
        }
    }
}
Merge.args = {
    specs_folder: core_1.Args.string({
        description: 'The folder with the OpenAPI specs for merging',
        required: true,
    }),
};
Merge.description = 'Merge two or more OpenAPI specs into one';
Merge.examples = [
    'liblab merge specsFolder',
    'liblab merge specsFolder -m mainSpec.yaml -o mergedSpec.yaml',
];
Merge.flags = {
    main: core_1.Flags.string({
        char: 'm',
        description: 'The name and extension of the main OpenAPI spec',
        helpValue: '<main-spec-file>',
    }),
    output: core_1.Flags.string({
        char: 'o',
        description: 'The name and extension of the output OpenAPI spec',
        helpValue: '<output-spec-file>',
    }),
};
Merge.tag = command_tags_enum_1.default.SPEC_MANAGEMENT;
Merge.usage = 'SPECS_FOLDER [-m <main-spec-file>] [-o <output-spec-file>]';
exports.default = Merge;
