"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const git_providers_1 = require("../../constants/git-providers");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const cmds_json_1 = tslib_1.__importDefault(require("../../logs/cmds.json"));
const liblab_config_service_1 = require("../../services/liblab-config/liblab-config.service");
const pr_service_1 = tslib_1.__importDefault(require("../../services/pr.service"));
const authenticate_user_1 = require("../../utils/authenticate-user");
const base_command_1 = require("../base-command");
class PR extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.liblabClient = new liblab_client_provider_1.default(config);
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command pr' },
            commandType: PR,
            eventProperties: { config_data: undefined },
        });
    }
    async run() {
        var _a, _b, _c, _d;
        const { languageFlag, org, remoteLiblabConfigPath, platform } = await this.parseFlags();
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.updateUser(currentUser);
        const liblabConfigHelper = new liblab_config_service_1.LiblabConfigService();
        const rawConfig = await liblabConfigHelper.getRawLiblabConfig(remoteLiblabConfigPath);
        const validatedConfig = liblabConfigHelper.validate(rawConfig);
        const liblabConfig = liblabConfigHelper.modify(validatedConfig, languageFlag);
        (_b = this.analytics) === null || _b === void 0 ? void 0 : _b.updateEventProperties({ config_data: liblabConfig });
        (_c = this.analytics) === null || _c === void 0 ? void 0 : _c.updateEventProperties({ config_data: liblabConfig });
        const owner = await this.getRepoOwner(liblabConfig, org);
        const prHelper = new pr_service_1.default(liblabConfig, this.liblabClient, owner, this.analytics, languageFlag, platform);
        await prHelper.run();
        await ((_d = this.analytics) === null || _d === void 0 ? void 0 : _d.logEvent({ status: 'succeed' }));
    }
    async getRepoOwner(liblabConfig, org) {
        var _a;
        const owner = org || ((_a = liblabConfig.publishing) === null || _a === void 0 ? void 0 : _a.githubOrg);
        if (!owner) {
            throw new Error(cmds_json_1.default.PR.ERROR.MISSING_ORG);
        }
        return owner;
    }
    async parseFlags() {
        const { flags } = await this.parse(PR);
        const { language, 'liblab-config': remoteLiblabConfigPath, org, platform } = flags;
        return {
            languageFlag: language,
            org,
            remoteLiblabConfigPath,
            platform: platform,
        };
    }
}
PR.description = 'Create a PR for all the generated SDKs.';
PR.examples = ['liblab pr -o {Github Organization Name}'];
PR.flags = {
    language: core_1.Flags.string({
        char: 'l',
        description: 'Sdk language to open a PR to',
    }),
    'liblab-config': core_1.Flags.string({
        description: 'Liblab config path',
    }),
    org: core_1.Flags.string({
        char: 'o',
        description: 'The organization to create the PR in',
    }),
    test: core_1.Flags.boolean({
        hidden: true,
    }),
    platform: core_1.Flags.string({
        char: 'p',
        options: git_providers_1.gitProviders,
        default: 'github',
    }),
};
PR.tag = command_tags_enum_1.default.PUBLISH;
exports.default = PR;
