"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const sleep_1 = require("../../utils/sleep");
const base_command_1 = require("../base-command");
class Publish extends base_command_1.BaseCommand {
    async run() {
        const { flags } = await this.parse(Publish);
        const config = {
            docs: false,
            langs: [],
            registeries: [],
        };
        if (flags.all) {
            config.langs = ['javascript', 'python', 'java', 'terraform'];
            config.docs = true;
        }
        else {
            if (flags.langs) {
                config.langs = flags.langs;
            }
            if (flags.docs) {
                config.docs = true;
            }
        }
        if (flags.npmToken) {
            config.registeries.push({
                name: 'npm',
                token: flags.npmToken,
            });
        }
        if (flags.pypiToken) {
            config.registeries.push({
                name: 'pypi',
                token: flags.pypiToken,
            });
        }
        if (flags.mavenToken) {
            config.registeries.push({
                name: 'maven',
                token: flags.mavenToken,
            });
        }
        if (flags.terraformRegistryToken) {
            config.registeries.push({
                name: 'terraformRegistry',
                token: flags.terraformRegistryToken,
            });
        }
        if (config.langs.includes('javascript') && !flags.npmToken) {
            this.error('You must provide an NPM token to publish Javascript artifacts');
        }
        if (config.langs.includes('python') && !flags.pypiToken) {
            this.error('You must provide a PyPI token to publish Python artifacts');
        }
        if (config.langs.includes('java') && !flags.mavenToken) {
            this.error('You must provide a Maven token to publish Java artifacts');
        }
        if (config.langs.includes('terraform') && !flags.terraformRegistryToken) {
            this.error('You must provide a Terraform Registry token to publish the provider');
        }
        if (config.langs.length === 0 && !config.docs) {
            const { publishLibJS, publishPortal } = await inquirer_1.default.prompt([
                {
                    message: 'Publish Dev Portal?',
                    name: 'publishPortal',
                    type: 'confirm',
                },
                {
                    message: 'Publish Javascript?',
                    name: 'publishLibJS',
                    type: 'confirm',
                },
            ]);
            config.docs = publishPortal;
            if (publishLibJS) {
                const { npmToken } = await inquirer_1.default.prompt([
                    {
                        mask: '*',
                        message: 'Insert your NPM token',
                        name: 'npmToken',
                        type: 'password',
                    },
                ]);
                config.registeries.push({ name: 'npm', token: npmToken });
            }
            const { publishLibPython } = await inquirer_1.default.prompt([
                {
                    message: 'Publish Python?',
                    name: 'publishLibPython',
                    type: 'confirm',
                },
            ]);
            if (publishLibPython) {
                const { pypiToken } = await inquirer_1.default.prompt([
                    {
                        mask: '*',
                        message: 'Insert your PyPI token',
                        name: 'pypiToken',
                        type: 'password',
                    },
                ]);
                config.registeries.push({ name: 'pypi', token: pypiToken });
            }
            const { publishLibJava } = await inquirer_1.default.prompt([
                {
                    message: 'Publish Java?',
                    name: 'publishLibJava',
                    type: 'confirm',
                },
            ]);
            if (publishLibJava) {
                const { mavenToken } = await inquirer_1.default.prompt([
                    {
                        mask: '*',
                        message: 'Insert your Maven token',
                        name: 'mavenToken',
                        type: 'password',
                    },
                ]);
                config.registeries.push({ name: 'maven', token: mavenToken });
            }
            const { publishTerraformProvider } = await inquirer_1.default.prompt([
                {
                    message: 'Publish Terraform Provider?',
                    name: 'publishTerraformProvider',
                    type: 'confirm',
                },
            ]);
            if (publishTerraformProvider) {
                const { terraformRegistryToken } = await inquirer_1.default.prompt([
                    {
                        mask: '*',
                        message: 'Insert your Terraform Registry token',
                        name: 'terraformRegistryToken',
                        type: 'password',
                    },
                ]);
                config.registeries.push({ name: 'terraformRegistry', token: terraformRegistryToken });
            }
        }
        core_1.ux.action.start('Publishing');
        await (0, sleep_1.sleep)(1000);
        core_1.ux.action.stop('Publishing successful');
        this.log('Publishing complete!');
    }
}
Publish.description = 'Publish the liblab artifacts.';
Publish.examples = [
    'liblab publish',
    'liblab publish --docs true',
    'liblab publish --all true',
    'liblab publish --langs=java',
];
Publish.flags = {
    all: core_1.Flags.boolean({
        char: 'a',
        default: false,
        description: 'Publish all language artifacts',
    }),
    docs: core_1.Flags.boolean({
        char: 'd',
        default: false,
        description: 'Publish docs',
    }),
    langs: core_1.Flags.string({
        char: 'l',
        description: 'Publish artifacts for the specified languages',
        multiple: true,
        options: ['javascript', 'python', 'java', 'terraform'],
        required: false,
    }),
    mavenToken: core_1.Flags.string({
        description: 'Maven token used for publishing',
        required: false,
    }),
    npmToken: core_1.Flags.string({
        description: 'NPM token used for publishing',
        required: false,
    }),
    pypiToken: core_1.Flags.string({
        description: 'PyPI token used for publishing',
        required: false,
    }),
    terraformRegistryToken: core_1.Flags.string({
        description: 'Terraform Registry token used for publishing',
        required: false,
    }),
};
Publish.hidden = true;
Publish.tag = command_tags_enum_1.default.PUBLISH;
exports.default = Publish;
