"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const liblab_1 = require("../../lib/liblab");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const authenticate_user_1 = require("../../utils/authenticate-user");
const base_command_1 = require("../base-command");
class TokenCreate extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    async catch(error) {
        const errorMessage = error.message || 'Unable to retrieve token';
        const logEventParams = { error, reason: errorMessage, status: 'failed' };
        await super.catch(error, logEventParams);
    }
    async run() {
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command token:create' },
            commandType: TokenCreate,
            currentUser,
        });
        const { args, flags } = await this.parse(TokenCreate);
        const token = await (0, liblab_1.createLongLivedToken)(args.token_name, flags.durationInDays);
        this.log(`Token successfully generated, it will be valid for ${flags.durationInDays} days`);
        this.log('');
        this.log(`-TOKEN-${'-'.repeat(token.length - 7)}`);
        this.log(token);
        this.log(`${'-'.repeat(token.length)}`);
        this.log('');
        this.log('Make sure to copy your liblab token, as you will not be able to see the token again.');
        await this.analytics.logEvent({ status: 'succeed' });
        return true;
    }
}
TokenCreate.args = {
    token_name: core_1.Args.string({
        required: true,
    }),
};
TokenCreate.description = 'Creates a long lived token for CI/CD operations';
TokenCreate.examples = [
    'liblab token create TOKEN_NAME',
    'liblab token create TOKEN_NAME --durationInDays=364',
    'liblab token create TOKEN_NAME -d=30',
    'liblab token create TOKEN_NAME -d 30',
];
TokenCreate.flags = {
    durationInDays: core_1.Flags.integer({
        char: 'd',
        default: 90,
        description: 'The amount of days the token will live, if not provided defaults to 90',
        max: 364,
        required: false,
    }),
};
TokenCreate.tag = command_tags_enum_1.default.AUTH;
exports.default = TokenCreate;
