"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiagnosisOption = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const axios_1 = require("axios");
const messages_1 = tslib_1.__importDefault(require("../../constants/messages"));
const paths_default_1 = require("../../constants/paths.default");
const command_tags_enum_1 = tslib_1.__importDefault(require("../../enums/command-tags.enum"));
const analytics_provider_1 = tslib_1.__importDefault(require("../../lib/analytics-provider"));
const helpers_1 = require("../../lib/helpers");
const liblab_client_provider_1 = tslib_1.__importDefault(require("../../lib/liblab-client-provider"));
const liblab_config_service_1 = require("../../services/liblab-config/liblab-config.service");
const spec_format_service_1 = require("../../services/spec-format.service");
const spec_service_1 = require("../../services/spec/spec.service");
const validate_service_1 = require("../../services/validate.service");
const authenticate_user_1 = require("../../utils/authenticate-user");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const file_utils_1 = require("../../utils/file.utils");
const text_utils_1 = require("../../utils/text.utils");
const liblab_config_validator_1 = require("../../validators/liblab-config/liblab-config-validator");
const base_command_1 = require("../base-command");
var DiagnosisOption;
(function (DiagnosisOption) {
    DiagnosisOption["CONTINUE"] = "continue";
    DiagnosisOption["CONTINUE_ALL"] = "continue-all";
    DiagnosisOption["IGNORE"] = "ignore";
    DiagnosisOption["IGNORE_ALL"] = "ignore-all";
})(DiagnosisOption || (exports.DiagnosisOption = DiagnosisOption = {}));
class Validate extends base_command_1.BaseCommand {
    constructor(argv, config) {
        super(argv, config);
        this.specService = new spec_service_1.SpecService();
        this.liblabClient = new liblab_client_provider_1.default(config);
        this.analytics = new analytics_provider_1.default({
            commandParams: { argv: this.argv, config: this.config, name: 'command validate' },
            commandType: Validate,
            eventProperties: { config_data: this.config },
        });
    }
    async catch(err) {
        const logEventParams = {
            error: err,
            reason: err.message,
            status: 'failed',
        };
        await super.catch(err, logEventParams);
    }
    async getSpecFile(specFilePath) {
        try {
            return await (0, file_utils_1.getFile)(specFilePath);
        }
        catch (specFileReadError) {
            (0, helpers_1.debugLog)('commands/validate#run.readFile: ', { specFileReadError });
            if (specFileReadError instanceof axios_1.AxiosError) {
                throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_URL_INVALID, specFileReadError);
            }
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_PATH_INVALID, specFileReadError);
        }
    }
    async run() {
        var _a, _b, _c;
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        const { flags } = await this.parse(Validate);
        const remoteConfigUrl = flags['liblab-config'];
        const liblabConfigHelper = new liblab_config_service_1.LiblabConfigService();
        const rawLiblabConfig = await liblabConfigHelper.getRawLiblabConfig(remoteConfigUrl || paths_default_1.liblabConfigPath);
        /**
         * Subset of liblab config data that is needed to proceed with the spec file validation
         */
        const minimalLiblabConfigData = liblabConfigHelper.validateMinimalConfig(rawLiblabConfig);
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.updateEventProperties({ config_data: minimalLiblabConfigData });
        (_b = this.analytics) === null || _b === void 0 ? void 0 : _b.updateUser(currentUser);
        const liblabConfigValidator = new liblab_config_validator_1.LiblabConfigValidator();
        const configValidationIssues = liblabConfigValidator.getValidationIssues(minimalLiblabConfigData);
        if (configValidationIssues.result === 'success') {
            this.log(`${(0, text_utils_1.getMarker)('success')} ${messages_1.default.INFO.LIBLAB_CONFIG_VALID}\n`);
        }
        else {
            const { errors, warnings, formattedIssuesMessage } = configValidationIssues;
            this.log(`${messages_1.default.ERROR.LIBLAB_CONFIG_ISSUES_DETECTED(formattedIssuesMessage, errors.length, warnings.length)}\n`);
        }
        const specFileContents = await this.specService.getSpecFile(minimalLiblabConfigData);
        this.fileFormat = (0, file_utils_1.identifyFileContentFormat)(specFileContents);
        const identifier = new spec_format_service_1.SpecFormatService(specFileContents, this.fileFormat);
        const format = identifier.getSpecFormat();
        if (format === spec_format_service_1.AllowedSpecFormats.Postman) {
            throw new Error('Validation for postman collections is not supported');
        }
        const api = await this.liblabClient.createApi({
            liblabConfig: minimalLiblabConfigData,
            specFormat: this.fileFormat,
            specContent: specFileContents,
        });
        const validator = new validate_service_1.ValidateService(api.id, minimalLiblabConfigData);
        await validator.validateSpec();
        validator.logViolations();
        if (validator.getViolations().length === 0) {
            this.log(`${(0, text_utils_1.getMarker)('success')} ${messages_1.default.INFO.SPEC_IS_VALID}`);
            return;
        }
        if (flags.nonInteractive) {
            await this.handleNonInteractiveMode(validator);
            return;
        }
        const selectedValidationOption = await validator.promptValidationAction();
        let validationMessage = '';
        switch (selectedValidationOption) {
            case validate_service_1.ValidateOption.REVIEW: {
                await validator.handleDiagnosisInteractively();
                validationMessage = 'liblab validate has completed.';
                break;
            }
            case validate_service_1.ValidateOption.SKIP_ALL: {
                validationMessage = 'All detected API spec issues skipped. liblab validate has completed.';
                break;
            }
            case validate_service_1.ValidateOption.SKIP_ALL_IGNORE: {
                await validator.addAllDiagnosticsToIgnoreList();
                validationMessage =
                    'liblab config file ignore list has been updated with all detected API spec issues. liblab validate has completed successfully.';
                break;
            }
            default: {
                break;
            }
        }
        await validator.exportValidation();
        if (validationMessage) {
            this.log(validationMessage);
        }
        await ((_c = this.analytics) === null || _c === void 0 ? void 0 : _c.logEvent({ status: 'succeed', result: validationMessage }));
    }
    async handleNonInteractiveMode(validator) {
        var _a;
        const severitiesCount = await validator.exportValidation();
        if ((severitiesCount === null || severitiesCount === void 0 ? void 0 : severitiesCount.errors) > 0) {
            const warningsMessage = (severitiesCount === null || severitiesCount === void 0 ? void 0 : severitiesCount.warnings) > 0 ? ` and ${severitiesCount.warnings} warnings` : '';
            throw new Error(`liblab validate has completed with ${severitiesCount.errors} errors${warningsMessage}.`);
        }
        let result;
        if ((severitiesCount === null || severitiesCount === void 0 ? void 0 : severitiesCount.warnings) > 0) {
            result = `liblab validate has completed with ${severitiesCount.warnings} warnings.`;
            this.warn(`liblab validate has completed with ${severitiesCount.warnings} warnings.`);
        }
        await ((_a = this.analytics) === null || _a === void 0 ? void 0 : _a.logEvent({ status: 'succeed', result }));
    }
}
Validate.description = 'Validate spec file';
Validate.examples = ['liblab validate'];
Validate.flags = {
    nonInteractive: core_1.Flags.boolean({
        default: false,
        description: 'Runs liblab validate non-interactively. Exits with code 0 if there are no errors, and with code 1 if there are errors.',
    }),
    'liblab-config': core_1.Flags.string({
        description: 'Liblab config path',
    }),
    test: core_1.Flags.boolean({
        hidden: true,
    }),
};
Validate.tag = command_tags_enum_1.default.SPEC_MANAGEMENT;
exports.default = Validate;
