"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withDocsLink = exports.withContactSupport = exports.supportLink = exports.bold = void 0;
const tslib_1 = require("tslib");
const build_portal_url_1 = require("../services/build-portal-url");
const text_utils_1 = require("../utils/text.utils");
const env_1 = tslib_1.__importDefault(require("./env"));
const paths_default_1 = require("./paths.default");
const { dashboardUrl } = env_1.default;
const bold = (text) => `\u001B[1m${text}\u001B[0m`;
exports.bold = bold;
const billingLink = `${dashboardUrl}/org-admin/billing-settings`;
exports.supportLink = 'https://liblab.com/contact';
const defaultDocsLink = 'https://developers.liblab.com/';
const morePricingInfoMessage = `For more information on pricing, please visit: https://liblab.com/pricing#faq`;
const messages = {
    COMMON: {
        CONTACT_SUPPORT: `\n\nPlease contact our support: ${exports.supportLink}`,
        SEE_DOCUMENTATION: (docsURL = defaultDocsLink) => `\n\nSee our documentation for more details at: ${docsURL}`,
    },
    ERROR: {
        ERROR_DUE_TO_FAILURE_TO_ENHANCE_SPEC: ({ outputName, outputPath, reason, }) => `Failed to write ${outputName} to ${outputPath} due to ${reason}`,
        // TODO: https://liblab.atlassian.net/browse/SER-2213
        ERROR_DUE_TO_UNAVAILABLE_ARTIFACTS: `Built SDKs does not supports spec enrichment`,
        // TODO: https://liblab.atlassian.net/browse/SER-2213
        ENHANCE_SPEC_MISSING_SUCCESSFUL_SDKS: (output) => `${(0, text_utils_1.getMarker)('error')} ${output}.json or enhancedApiSpec.json failed to generate`,
        BUILD_FAILED_TO_RETRIEVE_LOCAL_CUSTOMIZATIONS: 'Failed to retrieve local endpoint customization',
        BUILD_FAILED_TO_RETRIEVE_REMOTE_CUSTOMIZATIONS: 'Failed to retrieve remote endpoint customization',
        BUILD_FAILED_CUSTOMIZATIONS_IN_POSTMAN_COLLECTION: 'Customizations are not supported for postman collections. \n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview-customizations/#endpointcustomizations',
        BUILD_INIT_FAILED: 'Failed to initialize the build.',
        BUILD_FAILED_BUILDING_SDK: 'Failed to build the requested sdks',
        BUILD_FAILED_CHECK_BUILD_STATUS: 'Failed to check the status of the SDK builds',
        BUILD_FAILED_TO_APPLY_OAUTH_CUSTOMIZATIONS: 'Failed to apply OAuth customizations',
        BUILD_FAILED_SDKS(unsuccessfulSdks, isDocGenFailed) {
            if (!(unsuccessfulSdks === null || unsuccessfulSdks === void 0 ? void 0 : unsuccessfulSdks.length)) {
                return isDocGenFailed
                    ? 'Failed to generate API Documentation.'
                    : 'Failed to generate SDKs.';
            }
            const failedSdksMessage = `Failed to generate ${(0, text_utils_1.pluralize)('SDK', unsuccessfulSdks.length)} for ${unsuccessfulSdks.join(', ')}`;
            return isDocGenFailed ? `${failedSdksMessage} and API Documentation.` : failedSdksMessage;
        },
        BUILD_FAILED_TO_ENHANCE_API_SPEC: `Failed to generate enhanceApiSpec`,
        BUILD_FAILED_UPLOAD_WORKFLOW_SPEC: `Failed to upload workflow spec to the liblab Service.`,
        BUILD_FAILED_UPLOAD_ENHANCED_API_SPEC: `Failed to upload enhanced api spec to the liblab Service.`,
        BUILD_FAILED_UPLOAD_SPEC: `Failed to upload spec to the liblab Service.`,
        BUILD_FAILED_SPEC_PATH_INVALID: "The spec file could not be found at path specified in the 'specFilePath' property. Please ensure the path is correct and try again. \n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview-root/#specfilepath",
        BUILD_FAILED_WORKFLOWS_PATH_INVALID: 'The workflows file could not be found at path specified in the workflows/path property. Please ensure the path is correct and try again. \n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview-root/#workflows',
        BUILD_FAILED_SPEC_URL_INVALID: 'The URL specified in the workflows/path property could not be accessed. Please ensure the URL is correct and verify that sufficient permissions are granted to access the resource.  \n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview-root/#specfilepath',
        BUILD_FAILED_WORKFLOWS_URL_INVALID: "The URL specified in the 'specFilePath' property could not be accessed. Please ensure the URL is correct and verify that sufficient permissions are granted to access the resource.  \n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview-root/#workflows",
        BUILD_NO_SDKS_GENERATED: 'No SDKs were generated',
        BUILD_SDK_BUILD_TIMED_OUT: 'SDK build timed out',
        BUILD_SDK_NAME_MISSING: 'One of the following is required: spec title or sdk name',
        LIBLAB_CONFIG_INVALID: 'Error parsing the liblab config file: {message}',
        LIBLAB_CONFIG_MISSING: `The liblab config file, cannot be found in path: {path}.\nTry creating a new one using ${(0, text_utils_1.getColoredText)('green', 'liblab init')} or re-running liblab in a folder that contains the config file.\n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview`,
        LIBLAB_CONFIG_UNABLE_TO_RETRIEVE_REMOTE_CONFIG: `Unable to retrieve remote liblab config in the following address: {url}.\nTry creating a new one using ${(0, text_utils_1.getColoredText)('green', 'liblab init')} or re-running liblab in a folder that contains the config file.\n\nSee our documentation for more details at https://developers.liblab.com/cli/config-file-overview`,
        LIBLAB_CONFIG_UNKNOWN_ISSUE: 'An unknown error occurred while parsing the liblab config file. {error}',
        LIBLAB_CONFIG_ISSUES_DETECTED: (formattedIssuesMessage, errorsCount, warningsCount) => `Detected ${errorsCount ? `${errorsCount} ${(0, text_utils_1.pluralize)('error', errorsCount)}${warningsCount ? ' and ' : ''}` : ''}${warningsCount ? `${warningsCount} ${(0, text_utils_1.pluralize)('warning', warningsCount)}` : ''} in the liblab config:\n\n${formattedIssuesMessage}`,
        FAILED_PARSING_LANGUAGE_FLAG: 'There was an error parsing the language flag.',
        NO_OUTPUT_FOR_SDK: (languages) => `No output directory found for the following SDKs: ${languages}. Cannot create PR. \nTry running the liblab build command ${(0, exports.bold)('liblab build')}.`,
        NO_REPOSITORY_FOR_LANGUAGE: (language) => `No repo name found for ${language}, skipping...`,
        SUBSCRIPTION_MISSING: `Subscription information is missing. Please contact our support: ${exports.supportLink}`,
        SUBSCRIPTION_NO_CREDITS(role, remainingCredits, requestedCredits) {
            const prefix = `${(0, text_utils_1.getColoredText)('red', 'Free Plan:')}`;
            const mainMessage = `You have insufficient monthly credits to execute the requested build`;
            const requestedCreditsText = `\nSDK builds attempted: ${requestedCredits}`;
            const remainingCreditsText = `\nRemaining credits: ${remainingCredits}`;
            const adminSuggestion = `\nTo upgrade your plan, please go to: ${billingLink}\n${morePricingInfoMessage}`;
            const memberSuggestion = `\nPlease contact your organisation admin to upgrade your plan.\n${morePricingInfoMessage}`;
            return role === 'SUPER_ADMIN'
                ? `${prefix} ${mainMessage} ${requestedCreditsText} ${remainingCreditsText} ${adminSuggestion}`
                : `${prefix} ${mainMessage} ${requestedCreditsText} ${remainingCreditsText} ${memberSuggestion}`;
        },
        CREDITS_SPENT(role, severity = 'warn') {
            const prefix = `${(0, text_utils_1.getColoredText)(severity === 'warn' ? 'yellow' : 'red', 'Free Plan:')}`;
            const mainMessage = 'You have utilized all your available monthly credits.';
            const updatePlanMessage = role === 'SUPER_ADMIN'
                ? `To upgrade your plan, please go to: ${billingLink}`
                : 'Please contact your organisation admin to upgrade your plan.';
            return `\n${prefix} ${mainMessage}\n${updatePlanMessage}\n${morePricingInfoMessage}`;
        },
        USER_ERROR_LONG_LIVED_TOKEN: 'Unable to get user information with the long lived token.',
    },
    WARNING: {
        CREATE_DOCS_DEPRECATED: 'Deprecated. Please remove it or use the "docs" property instead. (https://developers.liblab.com/cli/config-file-overview-root/#createdocs)',
        DOCS_NOT_SUPPORTS_LANGUAGES: (unsupportedLanguages) => `liblab config contains languages without docs support, API Documentation will not be generated for: ${unsupportedLanguages} (https://developers.liblab.com/reference/spec-support/)`,
        ENHANCED_SPEC_NOT_SUPPORTS_LANGUAGES: (unsupportedLanguage, unsupportedVersion) => 
        // TODO: update after documentation is written
        `enhancedAPISpec is not supported for ${unsupportedLanguage}${unsupportedVersion ? `-${unsupportedVersion}` : ''}. See more details on supported languages and version in the liblab developer docs (https://developers.liblab.com/cli/config-file-overview-customizations/#snippets)`,
    },
    INFO: {
        BUILD_INVITE_TO_PORTAL: (portal) => `Your SDKs are being generated. Visit the ${(0, exports.bold)(`liblab portal (${(0, build_portal_url_1.buildPortalUrl)(portal)})`)} to view more details on your build(s).`,
        LIBLAB_CONFIG_VALID: `No issues detected in the liblab config file.`,
        SPEC_IS_VALID: 'No issues detected in the spec file.',
    },
    SUCCESS: {
        BUILD_SUCCESS: (successfulSdkNames) => `Successfully generated ${(0, text_utils_1.pluralize)('SDK', successfulSdkNames.length)} for ${successfulSdkNames.join(', ')}. ♡ You can find ${successfulSdkNames.length > 1 ? 'them' : 'it'} inside: ${paths_default_1.outputDir}`,
        PR_SUCCESS: (language) => `Successfully created PR for ${language}`,
        REMAINING_CREDITS(remainingCredits) {
            const prefix = `${(0, text_utils_1.getColoredText)('yellow', 'Free Plan:')}`;
            const creditsMessage = `${remainingCredits} ${(0, text_utils_1.pluralize)('credit', remainingCredits)} remaining this month.`;
            return `\n${prefix} ${creditsMessage}\n${morePricingInfoMessage}`;
        },
        GENERATED_ENRICHED_SPEC_SUCCESSFULLY: (specPath, generatedLanguages) => `Generated Enhanced Api Spec in: ${specPath} for: ${generatedLanguages}`,
    },
};
const withContactSupport = (message) => `${message}${messages.COMMON.CONTACT_SUPPORT}`;
exports.withContactSupport = withContactSupport;
const withDocsLink = (message, docsUrl) => `${message}${messages.COMMON.SEE_DOCUMENTATION(docsUrl)}`;
exports.withDocsLink = withDocsLink;
exports.default = messages;
