"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const base_command_1 = require("./commands/base-command");
const env_1 = tslib_1.__importDefault(require("./constants/env"));
const messages_1 = tslib_1.__importDefault(require("./constants/messages"));
const command_tags_enum_1 = tslib_1.__importDefault(require("./enums/command-tags.enum"));
const liblab_client_provider_1 = tslib_1.__importDefault(require("./lib/liblab-client-provider"));
const authenticate_user_1 = require("./utils/authenticate-user");
const title = (value) => `\u001B[1m${value}\u001B[22m\n`;
class CustomHelp extends core_1.Help {
    constructor(config, opts) {
        super(config, opts);
        this.liblabClient = new liblab_client_provider_1.default(config);
    }
    formatCommands(commands) {
        if (commands.length === 0)
            return '';
        const groups = {};
        for (const command of commands) {
            // Base command is an abstract class, we don't want it in help output
            if (command.id === base_command_1.BASE_COMMAND_ID) {
                continue;
            }
            const currentTag = command.tag || command_tags_enum_1.default.OTHERS;
            groups[currentTag] = [...(groups[currentTag] || []), command];
        }
        let result = `${title('COMMANDS')}`;
        for (const key of Object.values(command_tags_enum_1.default)) {
            if (!groups[key])
                continue;
            result += '\n';
            const list = this.renderList(groups[key].map((_command) => {
                let id;
                if (this.config.topicSeparator !== ':') {
                    id = _command.id.replaceAll(':', this.config.topicSeparator);
                }
                return [id, this.summary(_command)];
            }), {
                indentation: 2,
                spacer: '\n',
                stripAnsi: this.opts.stripAnsi,
            });
            result += this.section(key, list);
            result += '\n';
        }
        return result;
    }
    formatRoot() {
        let root = super.formatRoot();
        root = root.replaceAll(/VERSION[\S\s]+?(?=USAGE)/gm, '');
        root += `\n  $ liblab [COMMAND] --help`;
        root += `\n\n${title('DOCS')}  ${env_1.default.docURL}`;
        return root;
    }
    async showRootHelp() {
        var _a, _b;
        let rootTopics = this.sortedTopics.map((_topic) => ({ ..._topic, id: _topic.name }));
        let rootCommands = this.sortedCommands;
        const state = (_b = (_a = this.config.pjson) === null || _a === void 0 ? void 0 : _a.oclif) === null || _b === void 0 ? void 0 : _b.state;
        if (state) {
            this.log(state === 'deprecated'
                ? `${this.config.bin} is deprecated`
                : `${this.config.bin} is in ${state}.\n`);
        }
        this.log(this.formatRoot());
        this.log('');
        if (!this.opts.all) {
            rootTopics = rootTopics.filter((_topic) => !_topic.name.includes(':'));
            rootCommands = rootCommands.filter((_command) => !_command.id.includes(':'));
        }
        if (rootCommands.length > 0) {
            rootCommands = rootCommands.filter((_command) => _command.id);
            this.log(this.formatCommands([...rootCommands, ...rootTopics]));
        }
        await this.liblabClient.init();
        // help is public command, subscription information is shown only to logged in users
        if (this.liblabClient.currentUser) {
            await this.logSubscriptionInformation();
        }
    }
    summary(_command) {
        return _command.description && this.render(_command.description).split('\n')[0];
    }
    async logSubscriptionInformation() {
        const currentUser = await (0, authenticate_user_1.authenticateUser)({ liblabClientProvider: this.liblabClient });
        const subscription = await this.liblabClient.getSubscriptionStatus();
        if (!subscription) {
            this.log(messages_1.default.ERROR.SUBSCRIPTION_MISSING);
            return;
        }
        if (subscription.status === 'active') {
            return;
        }
        if (subscription.remainingCredits === 0) {
            this.log(messages_1.default.ERROR.CREDITS_SPENT(currentUser.role));
            return;
        }
        this.log(messages_1.default.SUCCESS.REMAINING_CREDITS(subscription.remainingCredits));
    }
}
exports.default = CustomHelp;
