"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const lodash_1 = require("lodash");
const mixpanel_1 = tslib_1.__importDefault(require("mixpanel"));
const node_crypto_1 = require("node:crypto");
const env_1 = tslib_1.__importDefault(require("../constants/env"));
const ip_utils_1 = require("../utils/ip.utils");
const user_utils_1 = require("../utils/user.utils");
const datadog_1 = require("./datadog");
const helpers_1 = require("./helpers");
class AnalyticsProvider {
    constructor(params) {
        this.sessionId = (0, node_crypto_1.randomUUID)();
        this.datadog = datadog_1.datadogClient;
        this.eventProperties = {};
        this.source = 'cli';
        this.mixpanel = mixpanel_1.default.init(env_1.default.mixpanelToken, {
            keepAlive: false,
        });
        const { commandParams, commandType, currentUser, eventProperties = {} } = params;
        this.commandType = commandType;
        this.commandParams = commandParams;
        this.eventProperties = eventProperties;
        if (currentUser) {
            this.updateUser(currentUser);
        }
    }
    static async logSimpleEventToDataDog(status, message, err, logEventParams) {
        const logEventProperties = {
            ...logEventParams,
            status,
            reason: (err === null || err === void 0 ? void 0 : err.message) || message,
            message: `[${status}]: ${message || (err === null || err === void 0 ? void 0 : err.message)}`,
            error: err,
            ...this.getStaticEventProperties(),
        };
        try {
            const event = AnalyticsProvider.prepareDataDogEvent(message, logEventProperties);
            (0, helpers_1.debugLog)(`[${status}]: ${message}`, event);
            await datadog_1.datadogClient.submitLog(event);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`AnalyticsProvider.logSimpleEventToDataDog`, error);
        }
    }
    static getErrorStackTrace(error) {
        return error instanceof Error ? error.stack : '';
    }
    static getStaticEventProperties() {
        return {
            $insert_id: (0, node_crypto_1.randomUUID)(),
            env: env_1.default.environment,
            ip_address: (0, ip_utils_1.getLocalIp)(),
            time: Date.now(),
        };
    }
    static prepareDataDogEvent(message, eventProperties, version) {
        const stackTrace = AnalyticsProvider.getErrorStackTrace(eventProperties.error);
        const stringProperties = Object.entries(eventProperties).reduce((acc, [key, value]) => {
            if (value === undefined)
                return acc;
            return { ...acc, [key]: typeof value === 'string' ? value : JSON.stringify(value) };
        }, {});
        const datadogEnv = datadog_1.LiblabEnvToDatadogEnvMap[env_1.default.environment] || '';
        return {
            body: [
                {
                    additionalProperties: {
                        ...stringProperties,
                        env: datadogEnv,
                        level: (0, datadog_1.getStatusLevel)(eventProperties.status),
                        ...(stackTrace && { stack: stackTrace }),
                    },
                    ddsource: 'nodejs',
                    ddtags: `env:${datadogEnv}, version:${version}`,
                    message,
                    service: 'cli',
                },
            ],
        };
    }
    addConfigData(liblabConfig) {
        this.liblabConfig = liblabConfig;
    }
    async logEvent(params) {
        var _a, _b;
        const initialEventProperties = this.getEventProperties(params);
        if (!initialEventProperties) {
            return;
        }
        const eventProperties = {
            ...initialEventProperties,
            sdkName: (_a = this.liblabConfig) === null || _a === void 0 ? void 0 : _a.sdkName,
            apiName: (_b = this.liblabConfig) === null || _b === void 0 ? void 0 : _b.apiName,
        };
        try {
            await Promise.all([this.logToMixpanel(eventProperties), this.logToDatadog(eventProperties)]);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`AnalyticsProvider.logEvent#command ${this.commandType.id}`, error);
        }
    }
    async logEventToDatadog(status, err, params) {
        const eventProperties = this.getEventProperties(params);
        if (!eventProperties) {
            return;
        }
        eventProperties.error = err;
        eventProperties.reason = err === null || err === void 0 ? void 0 : err.message;
        eventProperties.status = status;
        try {
            await this.logToDatadog(eventProperties);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`AnalyticsProvider.logEventToDatadog#command ${this.commandType.id}`, error);
        }
    }
    updateEventProperties(newEventProperties) {
        this.eventProperties = newEventProperties;
    }
    updateUser(newUser) {
        var _a, _b;
        this.currentUser = newUser;
        this.org = (_b = (_a = newUser === null || newUser === void 0 ? void 0 : newUser.orgs) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.org;
    }
    buildDatadogEventName(eventProperties) {
        var _a, _b, _c;
        const eventStatus = eventProperties.status ? `[Status]: ${eventProperties.status}` : '';
        const eventName = this.eventProperties.event_name || this.commandParams.name;
        const userId = ((_a = this.currentUser) === null || _a === void 0 ? void 0 : _a.id) ? `[UserId]: ${(_b = this.currentUser) === null || _b === void 0 ? void 0 : _b.id}` : '';
        const buildId = eventProperties.build_id ? ` [BuildId]: ${eventProperties.build_id}` : '';
        const artifactIds = eventProperties.artifact_ids && ((_c = eventProperties.artifact_ids) === null || _c === void 0 ? void 0 : _c.length) > 0
            ? ` [ArtifactIds]: ${eventProperties.artifact_ids.join(' ')}`
            : '';
        const event = `[Event]: ${eventName}`;
        return `${userId} ${event} ${eventStatus}${buildId}${artifactIds}`;
    }
    getEventProperties(params) {
        if (!this.currentUser) {
            (0, helpers_1.debugLog)(`AnalyticsProvider.logEvent#command ${this.commandType.id}`, {
                error: 'invalid current user',
            });
            return null;
        }
        return {
            ...this.eventProperties,
            $insert_id: (0, node_crypto_1.randomUUID)(),
            $os: process.platform,
            action: this.commandType.id,
            arch: this.commandParams.config.arch,
            args: this.commandParams.argv,
            category: this.commandType.tag,
            ci_run: this.isCiRun(),
            cli_version: this.commandParams.config.version,
            env: env_1.default.environment,
            ip_address: (0, ip_utils_1.getLocalIp)(),
            label: this.commandType.description,
            platform: this.commandParams.config.platform,
            session: this.sessionId,
            shell: this.commandParams.config.shell,
            source: this.source,
            test: (0, user_utils_1.isLiblabUser)(this.currentUser),
            time: Date.now(),
            ...(this.currentUser && {
                distinct_id: this.currentUser.id.toString() || '',
                user_id: this.currentUser.id,
            }),
            ...(this.org && {
                org_id: this.org.id,
                org_name: this.org.name,
            }),
            ...params,
        };
    }
    isCiRun() {
        const envValue = process.env.LIBLAB_CI;
        if (!envValue) {
            return false;
        }
        return envValue.toLowerCase() === 'true';
    }
    async logToDatadog(eventProperties) {
        const message = this.buildDatadogEventName(eventProperties);
        const logParams = AnalyticsProvider.prepareDataDogEvent(message, eventProperties, this.commandParams.config.version);
        try {
            await this.datadog.submitLog(logParams);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`AnalyticsProvider.logToDatadog#command ${this.commandType.id}`, error);
        }
    }
    async logToMixpanel(eventProperties) {
        return new Promise((resolve) => {
            this.mixpanel.track(eventProperties.event_name || this.commandParams.name, (0, lodash_1.omit)(eventProperties, ['error']), 
            /** the callback is called when the request is finished or when an error occurs */
            (error) => {
                if (error) {
                    (0, helpers_1.debugLog)(`AnalyticsProvider.logToMixpanel#command ${this.commandType.id}`, error);
                }
                resolve();
            });
        });
    }
}
exports.default = AnalyticsProvider;
