"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitClient = void 0;
const tslib_1 = require("tslib");
const isomorphic_git_1 = tslib_1.__importDefault(require("isomorphic-git"));
const node_1 = tslib_1.__importDefault(require("isomorphic-git/http/node"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const helpers_1 = require("../helpers");
class GitClient {
    constructor(owner, token) {
        this.owner = owner;
        this.token = token;
        this.ALL_FILEPATHS = '.';
        this.localRepoPath = './';
    }
    async checkout(branch) {
        try {
            (0, helpers_1.debugLog)('git#checkout: Checking out branch:', branch);
            await isomorphic_git_1.default.checkout({
                fs: node_fs_1.default,
                dir: this.localRepoPath,
                ref: branch,
            });
        }
        catch (error) {
            this.handleCheckoutError(error, branch);
        }
    }
    async clone(repoUrl, specialParams) {
        try {
            (0, helpers_1.debugLog)('git#clone: Cloning repository:', repoUrl);
            await isomorphic_git_1.default.clone({
                fs: node_fs_1.default,
                http: node_1.default,
                dir: this.localRepoPath,
                url: repoUrl,
                onAuth: (url) => {
                    (0, helpers_1.debugLog)('git#clone: Authenticating for:', url);
                    return {
                        username: (specialParams === null || specialParams === void 0 ? void 0 : specialParams.username) || this.owner,
                        password: this.token,
                    };
                },
            });
        }
        catch (error) {
            this.handleCloneError(error, repoUrl);
        }
    }
    async commit(message, author) {
        try {
            (0, helpers_1.debugLog)('git#commit: Committing changes with message:', message);
            (0, helpers_1.debugLog)('git#commit: Committing changes for author:', author);
            await isomorphic_git_1.default.commit({
                fs: node_fs_1.default,
                dir: this.localRepoPath,
                message,
                author,
            });
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError('Unexpected error occurred while trying to commit changes', error);
        }
    }
    async createNewBranch(branch) {
        try {
            (0, helpers_1.debugLog)('git#createNewBranch: Creating new branch:', branch);
            await isomorphic_git_1.default.branch({
                fs: node_fs_1.default,
                dir: this.localRepoPath,
                ref: branch,
                checkout: true,
            });
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to create branch ${branch}`, error);
        }
    }
    async push(branch, specialParams) {
        try {
            (0, helpers_1.debugLog)('git#push: Pushing changes to branch:', branch);
            await isomorphic_git_1.default.push({
                fs: node_fs_1.default,
                http: node_1.default,
                dir: this.localRepoPath,
                ref: branch,
                onAuth: (url) => {
                    (0, helpers_1.debugLog)('git#push: Authenticating for:', url);
                    return {
                        username: (specialParams === null || specialParams === void 0 ? void 0 : specialParams.username) || this.owner,
                        password: this.token,
                    };
                },
            });
        }
        catch (error) {
            this.handlePushError(error, branch);
        }
    }
    async stageAll(filepathsToRemove, filepathsToAdd = this.ALL_FILEPATHS) {
        try {
            (0, helpers_1.debugLog)('git#stageAll: Staging all changes');
            await isomorphic_git_1.default.add({ fs: node_fs_1.default, dir: this.localRepoPath, filepath: filepathsToAdd });
            for (const filepathToRemove of filepathsToRemove) {
                (0, helpers_1.debugLog)('git#stageAll Removing file:', filepathToRemove);
                await isomorphic_git_1.default.remove({ fs: node_fs_1.default, dir: this.localRepoPath, filepath: filepathToRemove });
            }
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError('Unexpected error occurred while trying to stage changes for commit', error);
        }
    }
    handleCheckoutError(error, targetBranch) {
        if (error instanceof isomorphic_git_1.default.Errors.NotFoundError) {
            throw new rethrown_error_1.RethrownError(`Target branch '${targetBranch}' could not be found`, error);
        }
        throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to fetch target branch ${targetBranch}`, error);
    }
    handleCloneError(error, repoUrl) {
        var _a;
        if (!(error instanceof isomorphic_git_1.default.Errors.HttpError)) {
            throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to reach repository ${repoUrl}`, error);
        }
        switch ((_a = error === null || error === void 0 ? void 0 : error.data) === null || _a === void 0 ? void 0 : _a.statusCode) {
            case 401: {
                throw new rethrown_error_1.RethrownError(`Unauthorized access to repository ${repoUrl}`, error);
            }
            case 403: {
                throw new rethrown_error_1.RethrownError(`Access to repository ${repoUrl} is forbidden`, error);
            }
            case 404: {
                throw new rethrown_error_1.RethrownError(`Repository ${repoUrl} could not be found`, error);
            }
            default: {
                throw new rethrown_error_1.RethrownError(`Unexpected http error occurred while trying to reach repository ${repoUrl}`, error);
            }
        }
    }
    handlePushError(error, branch) {
        var _a;
        if (!(error instanceof isomorphic_git_1.default.Errors.HttpError)) {
            throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to push changes to the branch ${branch}`, error);
        }
        switch ((_a = error === null || error === void 0 ? void 0 : error.data) === null || _a === void 0 ? void 0 : _a.statusCode) {
            case 401: {
                throw new rethrown_error_1.RethrownError(`Unauthorized access to push changes to branch ${branch}`, error);
            }
            case 403: {
                throw new rethrown_error_1.RethrownError(`Access to push changes to branch ${branch} is forbidden`, error);
            }
            default: {
                throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to push changes to the branch ${branch}`, error);
            }
        }
    }
}
exports.GitClient = GitClient;
