"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.GitLabClient = void 0;
const rest_1 = require("@gitbeaker/rest");
const cmds_json_1 = require("../../logs/cmds.json");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const helpers_1 = require("../helpers");
const git_client_1 = require("./git-client");
class GitLabClient extends git_client_1.GitClient {
    constructor(owner, token, host = 'https://gitlab.com') {
        super(owner, token);
        this.host = host;
        this.gitlabClient = new rest_1.Gitlab({
            host,
            token,
        });
    }
    async createPullRequest({ body, branchName, repo, targetBranch, title, owner, }) {
        const repoId = await this.getRepoId(owner, repo);
        return this.createGitlabMergeRequest(repoId, branchName, targetBranch, title, body);
    }
    formatRepoUrl(owner, repo) {
        return `${this.host}/${owner}/${repo}.git`;
    }
    getCommitAuthor() {
        if (!this.userInfo) {
            throw new Error(cmds_json_1.PR.ERROR.FAILED_TO_GET_AUTHOR);
        }
        try {
            return {
                email: this.userInfo.email,
                name: this.userInfo.name,
            };
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to get commit author', error);
            throw new rethrown_error_1.RethrownError(cmds_json_1.PR.ERROR.FAILED_TO_GET_AUTHOR, error);
        }
    }
    async setupAuthenticatedUser() {
        this.userInfo = await this.gitlabClient.Users.showCurrentUser();
    }
    async createGitlabMergeRequest(repoId, branchName, targetBranch, title, body) {
        try {
            const data = await this.gitlabClient.MergeRequests.create(repoId, branchName, targetBranch, title, {
                description: body,
            });
            return {
                ...data,
                html_url: data.web_url,
            };
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to create pull request', error);
            throw new rethrown_error_1.RethrownError(cmds_json_1.PR.ERROR.FAILED_TO_CREATE_PULL_REQUEST, error);
        }
    }
    /**
     * Formats the repository URL by:
     * 1. Removing the host (e.g., https://gitlab.com/) from the full URL.
     * 2. Stripping off the `.git` extension to get a clean GitLab project path.
     *
     * @param owner - The owner or namespace of the repository.
     * @param repo - The repository name.
     * @returns A formatted project path suitable for GitLab API requests.
     */
    formatRepoUrlForGitlab(owner, repo) {
        return this.formatRepoUrl(owner, repo).replace(`${this.host}/`, '').replace('.git', '');
    }
    async getProjectByUrl(url) {
        return this.gitlabClient.Projects.show(url);
    }
    async getRepoId(owner, repo) {
        try {
            const repoUrl = this.formatRepoUrlForGitlab(owner, repo);
            const response = await this.getProjectByUrl(repoUrl);
            return response.id;
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to get project by URL', error);
            throw new rethrown_error_1.RethrownError('Failed to get project by URL', error);
        }
    }
}
exports.GitLabClient = GitLabClient;
