"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const axios_1 = tslib_1.__importStar(require("axios"));
const fs_extra_1 = require("fs-extra");
const node_fs_1 = require("node:fs");
const promises_1 = require("node:fs/promises");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const node_stream_1 = require("node:stream");
const open_1 = tslib_1.__importDefault(require("open"));
const openid_client_1 = require("openid-client");
const index_1 = tslib_1.__importDefault(require("../constants/env/index"));
const liblab_version_1 = require("../constants/liblab-version");
const messages_1 = tslib_1.__importStar(require("../constants/messages"));
const paths_default_1 = require("../constants/paths.default");
const internal_json_1 = tslib_1.__importDefault(require("../logs/internal.json"));
const liblab_client_json_1 = require("../logs/liblab-client.json");
const custom_plan_modifiers_service_1 = require("../services/custom-plan-modifiers.service");
const enhanced_api_spec_service_1 = require("../services/enhanced-api-spec/enhanced-api-spec.service");
const docs_utils_1 = require("../utils/docs.utils");
const rethrown_error_1 = require("../utils/error/rethrown-error");
const json_utils_1 = require("../utils/json.utils");
const analytics_provider_1 = tslib_1.__importDefault(require("./analytics-provider"));
const helpers_1 = require("./helpers");
const liblab_1 = require("./liblab");
const { auth0ClientId, auth0Issuer, auth0audience, baseUrl, environment } = index_1.default;
const ONE_DAY_IN_MILLISECONDS = 24 * 3600 * 1000;
const envName = environment.toLowerCase();
const liblabTokenKey = envName === 'production' ? 'token' : `${envName}-token`;
class LiblabClientProvider {
    constructor(config) {
        this.config = config;
        this.currentUser = null;
        this.isLongLivedTokenInvalid = false;
        this.auth0grantType = 'urn:ietf:params:oauth:grant-type:device_code';
        this.auth0logoutTarget = `${auth0Issuer}/oidc/logout`;
        this.auth0Scopes = 'openid email';
        this.customPlanModifiersService = new custom_plan_modifiers_service_1.CustomPlanModifiersService();
        this.liblab = liblab_1.client;
        this.longLivedToken = process.env.LIBLAB_TOKEN;
        this.storedAccessTokenKey = liblabTokenKey;
        this.tokenFilePath = node_path_1.default.join(this.config.dataDir, 'token.json');
    }
    async approveDocs(previewSlug) {
        try {
            const docResponse = await this.liblab.doc.approve(previewSlug);
            return docResponse;
        }
        catch (error) {
            (0, helpers_1.debugLog)(`Liblab-client-provider: failed to approve doc`, error);
            throw new Error('API Documentation Approval failed');
        }
    }
    async createApi({ liblabConfig, specFormat, specContent, }) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const currentUserOrgId = (_e = (_d = (_c = (_b = (_a = (await this.getCurrentUser())) === null || _a === void 0 ? void 0 : _a.orgs) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.org) === null || _d === void 0 ? void 0 : _d.id) !== null && _e !== void 0 ? _e : null;
        const form = (0, axios_1.toFormData)({
            name: liblabConfig.apiName,
            version: liblabConfig.apiVersion,
            languages: liblabConfig.languages,
            orgId: currentUserOrgId,
        });
        form.append('spec', Buffer.from(specContent), {
            filename: `spec.${specFormat}`,
            contentType: 'application/octet-stream',
        });
        const token = this.accessToken || this.longLivedToken;
        try {
            // SDK is passing the buffer as a bad request - https://liblab.atlassian.net/browse/CODEGEN-2193
            const response = await axios_1.default.post(`${baseUrl}/apis`, form, {
                headers: {
                    Authorization: `Bearer ${token}`,
                    'Content-Type': 'multipart/form-data',
                },
            });
            return response.data;
        }
        catch (error) {
            let detailedMessage = '';
            if (axios_1.default.isAxiosError(error)) {
                const status = (_g = (_f = error.response) === null || _f === void 0 ? void 0 : _f.data) === null || _g === void 0 ? void 0 : _g.status;
                const errorMessages = (_j = (_h = error.response) === null || _h === void 0 ? void 0 : _h.data) === null || _j === void 0 ? void 0 : _j.message;
                if (status !== axios_1.HttpStatusCode.Created)
                    detailedMessage = errorMessages === null || errorMessages === void 0 ? void 0 : errorMessages.join('\n');
            }
            const errorMessage = (0, messages_1.withContactSupport)(detailedMessage || messages_1.default.ERROR.BUILD_FAILED_UPLOAD_SPEC);
            throw new rethrown_error_1.RethrownError(errorMessage, error);
        }
    }
    async getBuild(id, currentPollAttempt) {
        try {
            const build = await this.liblab.build.getById(id);
            (0, helpers_1.debugLog)(`commands/build#run.monitorBuildProgress: current poll attempt: ${currentPollAttempt}`, build);
            return build;
        }
        catch (_error) {
            const reason = (_error === null || _error === void 0 ? void 0 : _error.message) || messages_1.default.ERROR.BUILD_FAILED_BUILDING_SDK;
            throw new Error(reason);
        }
    }
    async getCurrentUser() {
        try {
            return await this.liblab.user.getCurrentUser();
        }
        catch (error) {
            const e = error;
            if (this.longLivedToken && (e === null || e === void 0 ? void 0 : e.statusCode) === 401) {
                (0, helpers_1.debugLog)(`Failed to get user, long lived token is not valid.`, e);
                this.isLongLivedTokenInvalid = true;
            }
            return null;
        }
    }
    async getSubscriptionStatus() {
        var _a, _b, _c, _d;
        try {
            const orgId = (_c = (_b = (_a = this === null || this === void 0 ? void 0 : this.currentUser) === null || _a === void 0 ? void 0 : _a.orgs) === null || _b === void 0 ? void 0 : _b[0].org) === null || _c === void 0 ? void 0 : _c.id;
            if (!orgId) {
                (0, helpers_1.debugLog)('Failed to extract orgId from user:', (_d = this.currentUser) === null || _d === void 0 ? void 0 : _d.id);
                return null;
            }
            return await this.liblab.orgSubscriptions.getActiveSubscriptionStatus(orgId);
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to get user subscription', JSON.stringify(error));
            return null;
        }
    }
    async init() {
        await this.initLocalTokenStorageFile();
        this.setLiblabClient();
        await this.setOpenIdClient();
        await this.loadStoredTokenToClient();
        this.currentUser = await this.getCurrentUser();
    }
    async initLocalTokenStorageFile() {
        const doesDataDirExists = (0, node_fs_1.existsSync)(this.config.dataDir);
        if (!doesDataDirExists) {
            (0, helpers_1.debugLog)('Initializing CLI data directory', this.config.dataDir);
            await (0, fs_extra_1.emptyDir)(this.config.dataDir);
        }
        const doesFileExists = (0, node_fs_1.existsSync)(this.tokenFilePath);
        if (!doesFileExists) {
            (0, helpers_1.debugLog)('Initializing token file', this.tokenFilePath);
            await (0, promises_1.writeFile)(this.tokenFilePath, JSON.stringify({}));
        }
    }
    async listLongLivedTokens() {
        if (!this.currentUser) {
            (0, helpers_1.debugLog)(`/commands/listLongLivedTokens user must be defined to list tokens`, this.currentUser);
            throw new Error(liblab_client_json_1.LIBLAB_API_ERRORS.GET_USER);
        }
        try {
            return await this.liblab.token.findTokensByUserId(this.currentUser.id);
        }
        catch (error) {
            throw new Error((0, json_utils_1.messageFromLiblabSdkError)(error));
        }
    }
    async login() {
        if (!this.auth0Client) {
            throw await this.getAuthClientNotSetError();
        }
        if (this.currentUser) {
            (0, helpers_1.debugLog)('User already exists in memory', this.currentUser);
            return this.currentUser;
        }
        await this.obtainAccessTokenFromAuth0(this.auth0Client);
        await this.saveTokenLocally();
        this.currentUser = await this.getCurrentUser();
        if (!this.currentUser) {
            (0, helpers_1.debugLog)(`/commands/login liblab API could not find user`, this.currentUser);
            throw new Error(liblab_client_json_1.LIBLAB_API_ERRORS.GET_USER);
        }
        return this.currentUser;
    }
    async logout() {
        await this.openAuth0Client(this.auth0logoutTarget);
        await this.deleteLocallyStoredToken();
    }
    async openAuth0Client(target, options) {
        return (0, open_1.default)(target, options);
    }
    async postBuild({ apiId, apiName, hooks, liblabConfig, rawLiblabConfig, }) {
        var _a;
        const params = {
            ...liblabConfig,
            ...liblabConfig.customizations,
            multiTenant: true,
        };
        delete params.customizations;
        const customPlanModifiers = liblabConfig.languages.includes('terraform')
            ? await this.customPlanModifiersService.getReadStream()
            : null;
        const payload = {
            apiId,
            deliveryMethods: ['zip'],
            languages: liblabConfig.languages,
            liblabVersion: (_a = liblabConfig.liblabVersion) !== null && _a !== void 0 ? _a : liblab_version_1.DEFAULT_LIBLAB_VERSION,
            sdkName: liblabConfig.sdkName || apiName,
            sdkVersion: liblabConfig.apiVersion || '1.0.0',
            ...(hooks instanceof node_fs_1.ReadStream && { hooks }),
            buildType: this.getBuildType(liblabConfig),
            params: JSON.stringify(params),
            rawConfig: JSON.stringify(rawLiblabConfig),
            // TODO: remove condition when the SDK is updated
            ...(customPlanModifiers &&
                customPlanModifiers instanceof node_stream_1.Readable && { customPlanModifiers }),
        };
        (0, helpers_1.debugLog)('/lib/liblab#buildSDK.params', params);
        (0, helpers_1.debugLog)('/lib/liblab#buildSDK.payload', payload);
        try {
            return await liblab_1.client.build.createBuild(payload);
        }
        catch (error) {
            (0, helpers_1.debugLog)('/lib/liblab#buildSDK.error', error);
            throw new rethrown_error_1.RethrownError((0, messages_1.withContactSupport)(messages_1.default.ERROR.BUILD_INIT_FAILED), error);
        }
    }
    async postEnhancedSpecArtifact(liblabConfig, buildId, { error, status }) {
        var _a, _b, _c;
        const enrichedSpecFileName = ((_c = (_b = (_a = liblabConfig.customizations) === null || _a === void 0 ? void 0 : _a.documentation) === null || _b === void 0 ? void 0 : _b.enhancedApiSpec) === null || _c === void 0 ? void 0 : _c.customOutputName) ||
            enhanced_api_spec_service_1.DEFAULT_ENRICHED_SPEC_NAME;
        try {
            const enhancedApiSpecFile = await (0, node_fs_1.createReadStream)(node_path_1.default.join(paths_default_1.outputDir, enrichedSpecFileName));
            const updatedBuild = await this.liblab.build.createBuildArtifact({
                enhancedApiSpecFile,
                status,
                ...(error && { error }),
                // @ts-expect-error - SDK bug, api expects type to be passed over, but the sdk is passing type_
                type: 'ENHANCED_API_SPEC',
            }, buildId);
            return updatedBuild;
        }
        catch (artifactUploadError) {
            const errorMessage = (0, messages_1.withContactSupport)(messages_1.default.ERROR.BUILD_FAILED_UPLOAD_ENHANCED_API_SPEC);
            throw new rethrown_error_1.RethrownError(errorMessage, artifactUploadError);
        }
    }
    async trackPublishPREvent(params) {
        const { language, sdk, success } = params;
        return this.liblab.userEvent.trackUserPublishPrEvent({
            language,
            sdk,
            success,
        });
    }
    async uploadWorkflows(apiId, workflowSpec, outputName) {
        const token = this.accessToken || this.longLivedToken;
        const form = (0, axios_1.toFormData)({
            apiId,
            ...(outputName && { outputName }),
        });
        form.append('workflowsSpec', Buffer.from(workflowSpec), {
            filename: `workflowSpec.json`,
            contentType: 'application/octet-stream',
        });
        try {
            // SDK is passing the buffer as a bad request - https://liblab.atlassian.net/browse/CODEGEN-2193
            const response = await axios_1.default.post(`${baseUrl}/workflows`, form, {
                headers: {
                    Authorization: `Bearer ${token}`,
                    'Content-Type': 'multipart/form-data',
                },
            });
            return response.data;
        }
        catch (error) {
            const errorMessage = (0, messages_1.withContactSupport)(messages_1.default.ERROR.BUILD_FAILED_UPLOAD_WORKFLOW_SPEC);
            throw new rethrown_error_1.RethrownError(errorMessage, error);
        }
    }
    async deleteLocallyStoredToken() {
        try {
            const isTokenExists = (0, node_fs_1.existsSync)(this.tokenFilePath);
            if (!isTokenExists) {
                (0, helpers_1.debugLog)(`/commands/logout#deleteLocallyStoredToken token does not exists in: ${this.tokenFilePath}`, {});
                return;
            }
            const tokenJson = await this.getAllStoredTokens();
            if (tokenJson[this.storedAccessTokenKey]) {
                (0, helpers_1.debugLog)('/commands/logout#deleteLocallyStoredToken removing token', {
                    tokenFile: this.tokenFilePath,
                    tokenJson,
                    tokenKey: this.storedAccessTokenKey,
                });
                delete tokenJson[this.storedAccessTokenKey];
            }
            await this.saveTokenFile(tokenJson);
        }
        catch (error) {
            (0, helpers_1.debugLog)('/commands/logout#deleteLocallyStoredToken error while deleteLocallyStoredToken', error);
            throw new Error(internal_json_1.default.TOKEN_UPDATED);
        }
    }
    async getAllStoredTokens() {
        (0, helpers_1.debugLog)('/commands/logout#getAllStoredTokens attempting to get tokens from', {
            path: this.tokenFilePath,
        });
        const tokenFile = await (0, promises_1.readFile)(this.tokenFilePath, 'utf8');
        return JSON.parse(tokenFile);
    }
    async getAuthClientNotSetError() {
        const error = new Error(liblab_client_json_1.AUTH0_ERRORS.DISCOVER);
        (0, helpers_1.debugLog)("No auth0 client, can't login", this.currentUser);
        await analytics_provider_1.default.logSimpleEventToDataDog('error', 'Not able to login, auth0Client not set', error, {
            user: this.currentUser,
        });
        return error;
    }
    getBuildType(liblabConfig) {
        if ((0, docs_utils_1.shouldGenerateApiDocs)(liblabConfig)) {
            return ['SDK', 'DOC'];
        }
        return ['SDK'];
    }
    async loadStoredTokenToClient() {
        try {
            if (this.longLivedToken) {
                (0, helpers_1.debugLog)(`/commands/login#liblabClientInit.loadStoredTokenToClient long lived token found process`, { longLivedToken: this.longLivedToken });
                return this.liblab.setAccessToken(this.longLivedToken);
            }
            const storedTokens = await this.getAllStoredTokens();
            (0, helpers_1.debugLog)(`/commands/login#liblabClientInit.loadStoredTokenToClient short lived token found in file system`, storedTokens);
            this.accessToken = storedTokens[this.storedAccessTokenKey];
            (0, helpers_1.debugLog)(`/commands/login#liblabClientInit.loadStoredTokenToClient setting Liblab client token`, { accessToken: this.accessToken });
            return this.liblab.setAccessToken(this.accessToken);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`/commands/login#liblabClientInit.loadStoredTokenToClient failed to load stored token to client`, error);
            throw new Error(internal_json_1.default.LOAD_STORED_TOKEN_TO_CLIENT);
        }
    }
    async obtainAccessTokenFromAuth0(deviceFlowHandler) {
        core_1.ux.stdout('A browser will now open, confirm the code and login');
        core_1.ux.stdout(`The confirmation code is ${deviceFlowHandler.user_code}`);
        core_1.ux.stdout(`If you have any issues you may also use the link: ${deviceFlowHandler.verification_uri_complete}`);
        try {
            core_1.ux.action.start('Waiting for confirmation and login at browser');
            (0, helpers_1.debugLog)('Opening Auth0 Client', deviceFlowHandler.verification_uri_complete);
            await this.openAuth0Client(deviceFlowHandler.verification_uri_complete);
            (0, helpers_1.debugLog)('Auth0 Client opened, waiting for user to login and polling for access token');
            const tokenSet = await deviceFlowHandler.poll();
            core_1.ux.action.stop();
            if (!tokenSet.access_token) {
                (0, helpers_1.debugLog)(`/commands/login#obtainAccesstTokenFromAuth0 token from polling does not have access token.
          process will error out`, tokenSet);
                const error = new Error(liblab_client_json_1.AUTH0_ERRORS.DISCOVER);
                await analytics_provider_1.default.logSimpleEventToDataDog('error', 'Polling access toke failed', error);
                throw error;
            }
            (0, helpers_1.debugLog)('Found access token, setting it on Liblab SDK Client');
            this.accessToken = tokenSet.access_token;
            this.liblab.setAccessToken(tokenSet.access_token);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`/commands/login#obtainAccesstTokenFromAuth0 Error while obtaining access token from auth0`, error);
            const isCliError = error.message === liblab_client_json_1.AUTH0_ERRORS.DISCOVER;
            const message = isCliError
                ? error.message
                : `Failed to log in to auth0 reason: ${error.message}`;
            // certain auth errors can be messages to the user,
            // so we ux.log it as a messages instead of error
            // this.cmd.ux.stdout((error as Error)?.message || 'Unable to validate confirmation in time.');
            // ux.stdout(JSON.stringify(error));
            throw new Error(message);
        }
    }
    async saveTokenFile(tokenFileContent) {
        await (0, promises_1.writeFile)(this.tokenFilePath, JSON.stringify(tokenFileContent, null, 2));
    }
    async saveTokenLocally() {
        try {
            const isTokenFileExists = (0, node_fs_1.existsSync)(this.config.dataDir);
            if (!isTokenFileExists) {
                (0, helpers_1.debugLog)('/commands/login#saveLogin.isTokenFileExists token file does not exists, creating file in', { destination: this.config.dataDir });
                await (0, promises_1.mkdir)(this.config.dataDir, { recursive: true });
            }
            // Check if token.json exists if so get it's value
            const storedTokens = await this.getAllStoredTokens();
            // Update the token value
            if (!this.accessToken) {
                (0, helpers_1.debugLog)('/commands/login#saveLogin access token does not exists, process will error out', {
                    accessToken: this.accessToken,
                });
                throw new Error('Failed to store access token');
            }
            storedTokens[this.storedAccessTokenKey] = this.accessToken;
            // Write the token to the file
            (0, helpers_1.debugLog)('/commands/login#saveLogin.saveTokenLocally storing token', {
                storedTokens,
                tokenFile: this.tokenFilePath,
                tokenKey: this.storedAccessTokenKey,
            });
            await this.saveTokenFile(storedTokens);
        }
        catch (error) {
            (0, helpers_1.debugLog)('/commands/login#saveLogin.saveTokenLocally error while storing token', error);
            throw new Error(internal_json_1.default.TOKEN_UPDATED);
        }
    }
    setLiblabClient() {
        (0, helpers_1.debugLog)('Setting Liblab SDK client base url', baseUrl);
        this.liblab.setBaseUrl(baseUrl);
    }
    async setOpenIdClient() {
        try {
            (0, helpers_1.debugLog)('Discovering Auth0 issuer');
            const issuer = await openid_client_1.Issuer.discover(auth0Issuer);
            (0, helpers_1.debugLog)('Creating Auth0 Client');
            const auth0Client = new issuer.Client({
                client_id: auth0ClientId,
                grant_types: [this.auth0grantType],
                max_age: ONE_DAY_IN_MILLISECONDS,
                token_endpoint_auth_method: 'none',
            });
            (0, helpers_1.debugLog)('Authorizing device with Auth0 Client');
            this.auth0Client = await auth0Client.deviceAuthorization({
                audience: auth0audience,
                scope: this.auth0Scopes,
            });
            (0, helpers_1.debugLog)('/commands/login#liblabClientProviderInit.setOpenIdClient setting openIdClient', {
                audience: auth0audience,
                client_id: auth0ClientId,
                grant_types: [this.auth0grantType],
                max_age: ONE_DAY_IN_MILLISECONDS,
                scope: this.auth0Scopes,
                token_endpoint_auth_method: 'none',
            });
        }
        catch (error) {
            (0, helpers_1.debugLog)('/commands/login#liblabClientProviderInit.setOpenIdClient error while storing token', error);
            await analytics_provider_1.default.logSimpleEventToDataDog('error', 'Failed to set openIdClient', error);
            throw new Error(liblab_client_json_1.AUTH0_ERRORS.DISCOVER);
        }
    }
}
exports.default = LiblabClientProvider;
