"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArtifactsService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const adm_zip_1 = tslib_1.__importDefault(require("adm-zip"));
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_extra_1 = require("fs-extra");
const inquirer_1 = tslib_1.__importDefault(require("inquirer"));
const lodash_1 = require("lodash");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const messages_1 = tslib_1.__importDefault(require("../constants/messages"));
const paths_default_1 = require("../constants/paths.default");
const helpers_1 = require("../lib/helpers");
const cmds_json_1 = require("../logs/cmds.json");
const normalize_sdk_names_1 = require("../utils/normalize-sdk-names");
class ArtifactsService {
    constructor(artifacts, approveAllOperations) {
        this.approveAllOperations = approveAllOperations;
        this.sdkZipArchiveMap = {};
        this.successfulSdks = [];
        this.successfulDoc = this.getSuccessfulDoc(artifacts);
        this.successfulSdks = this.getSuccessfulSdks(artifacts);
    }
    /**
     * Downloads the sdks from s3 and saves it in memory as Buffers
     */
    async downloadSdks() {
        var _a;
        const getReadStreamFromS3Promises = (_a = this.successfulSdks) === null || _a === void 0 ? void 0 : _a.map(this.getSdkBuffer);
        const getReadStreamFromS3Allsettled = await Promise.allSettled(getReadStreamFromS3Promises);
        this.sdkZipArchiveMap = getReadStreamFromS3Allsettled.reduce((acc, promiseResult, index) => {
            const { language } = this.successfulSdks[index].sdk;
            if (promiseResult.status === 'fulfilled') {
                acc[(0, lodash_1.toLower)(language)] = {
                    languageUiName: normalize_sdk_names_1.sdkNameToUiNameMapper[language],
                    zipArchive: new adm_zip_1.default(promiseResult.value.data),
                };
            }
            else {
                const errorLogMessage = (0, lodash_1.replace)(cmds_json_1.BUILD.SDK_DOWNLOAD_FAILURE, '{sdk}', normalize_sdk_names_1.sdkNameToUiNameMapper[language]);
                (0, helpers_1.debugLog)(errorLogMessage, promiseResult.reason);
                core_1.ux.stdout(errorLogMessage);
            }
            return acc;
        }, {});
    }
    /**
     * exports sdks to output dir
     */
    async exportSdks(approveAllOperations, providerName) {
        const archiveMapEntries = Object.entries(this.sdkZipArchiveMap);
        for (const [language, { languageUiName, zipArchive }] of archiveMapEntries) {
            const outputPath = language === 'terraform'
                ? node_path_1.default.join(paths_default_1.outputDir, `terraform-provider-${providerName}`)
                : node_path_1.default.join(paths_default_1.outputDir, language);
            const doesSdkExistsInDir = (0, fs_extra_1.existsSync)(outputPath);
            if (doesSdkExistsInDir && !approveAllOperations) {
                await this.handleExistingSdkDirs({
                    languageUiName,
                    outputPath,
                    zipArchive,
                });
            }
            else {
                this.unzipSdks({ outputPath, zipArchive });
            }
        }
    }
    getLanguageArchive(language) {
        return this.sdkZipArchiveMap[language];
    }
    logBuiltSdks() {
        if (this.successfulSdks.length === 0) {
            return;
        }
        const successfulArtifacts = this.successfulSdks.map((artifact) => normalize_sdk_names_1.sdkNameToUiNameMapper[artifact.sdk.language]);
        const logMessage = messages_1.default.SUCCESS.BUILD_SUCCESS(successfulArtifacts);
        core_1.ux.stdout(logMessage);
    }
    async getSdkBuffer({ artifactUrl }) {
        if (!artifactUrl) {
            throw new Error('Cannot download file, url not defined');
        }
        return axios_1.default.get(artifactUrl, {
            responseType: 'arraybuffer',
        });
    }
    getSuccessfulDoc(artifacts) {
        const artifactWithDoc = artifacts.find((artifact) => { var _a; return artifact.status === 'SUCCESS' && ((_a = artifact.doc) === null || _a === void 0 ? void 0 : _a.previewUrl); });
        return artifactWithDoc;
    }
    getSuccessfulSdks(artifacts) {
        const artifactsWithSdks = artifacts.filter(({ artifactUrl, sdk, status }) => status === 'SUCCESS' && artifactUrl && sdk);
        return artifactsWithSdks;
    }
    async handleExistingSdkDirs({ languageUiName, outputPath, zipArchive, }) {
        const shouldOverWrite = this.approveAllOperations ||
            (await this.promptShouldOverWriteSdk({ languageUiName, outputPath }));
        if (!shouldOverWrite) {
            return core_1.ux.stdout(`skipped download for ${languageUiName} sdk`);
        }
        (0, fs_extra_1.rmSync)(outputPath, { force: true, recursive: true });
        return this.unzipSdks({ outputPath, zipArchive });
    }
    async promptShouldOverWriteSdk({ languageUiName, outputPath, }) {
        const res = await inquirer_1.default.prompt({
            message: `${languageUiName} sdk already exists in: ${outputPath}, do you want to overwrite it?`,
            name: 'shouldOverWrite',
            type: 'confirm',
        });
        return res.shouldOverWrite;
    }
    unzipSdks({ outputPath, zipArchive }) {
        try {
            zipArchive.extractAllTo(outputPath, true);
            zipArchive
                .getEntries()
                .forEach(({ entryName }) => (0, fs_extra_1.utimesSync)(node_path_1.default.join(outputPath, entryName), new Date(), new Date()));
        }
        catch (error) {
            (0, helpers_1.debugLog)(`Failed to extract zip from buffer, to ${outputPath}, msg: ${error.message}`, error);
            throw new Error('Failed to extract zip');
        }
    }
}
exports.ArtifactsService = ArtifactsService;
