"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiblabApi = void 0;
const tslib_1 = require("tslib");
const sdk_1 = require("@liblab/sdk");
const axios_1 = tslib_1.__importStar(require("axios"));
const node_fs_1 = require("node:fs");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const env_1 = tslib_1.__importDefault(require("../../constants/env"));
const messages_1 = require("../../constants/messages");
const auth_1 = require("./auth/auth");
const constants_1 = require("./constants");
const { baseUrl } = env_1.default;
/**
 * Class for interacting with the LibLab API
 * Provides methods for managing builds and artifacts
 */
class LiblabApi {
    constructor() {
        this.auth = auth_1.Auth.getInstance();
        this.liblab = new sdk_1.Liblab({
            environment: baseUrl,
        });
        this.token = '';
        // Private constructor to enforce singleton pattern
    }
    static getInstance() {
        if (!LiblabApi.instance) {
            LiblabApi.instance = new LiblabApi();
        }
        return LiblabApi.instance;
    }
    get accessToken() {
        return this.token;
    }
    set accessToken(accessToken) {
        this.token = accessToken;
        this.liblab.setAccessToken(accessToken);
    }
    async createEnhancedSpec(filename, buildId) {
        const enhancedApiSpecFile = await (0, node_fs_1.createReadStream)(node_path_1.default.join(constants_1.OUTPUT_DIR, filename));
        return this.liblab.build.createBuildArtifact({
            enhancedApiSpecFile,
            status: 'SUCCESS',
            type: 'ENHANCED_API_SPEC',
        }, buildId);
    }
    /**
     * Retrieves the artifacts associated with a specific build ID.
     *
     * @param buildId - The numeric ID of the build to get artifacts for
     * @returns Promise that resolves to an array of build artifacts
     */
    async getBuild(buildId) {
        return this.liblab.build.getById(buildId);
    }
    /**
     * Gets the currently authenticated user from the LibLab API.
     * This method is used to validate access tokens by attempting to fetch the user profile with the provided token.
     * If the token is invalid or expired, the request will fail.
     *
     * @param accessToken - Optional access token to use for the request. If not provided, uses the stored token.
     * @returns Promise that resolves to the current user's profile
     */
    async getCurrentUser(accessToken) {
        if (this.currentUser) {
            return this.currentUser;
        }
        this.liblab.setAccessToken(accessToken || this.token);
        this.currentUser = await this.liblab.user.getCurrentUser();
        return this.currentUser;
    }
    async isPayingCustomer() {
        const subscription = await this.getSubscriptionStatus();
        if (!subscription || subscription.status === 'free' || subscription.status !== 'active') {
            return false;
        }
        return true;
    }
    async login() {
        const token = await this.auth.getAccessToken();
        this.accessToken = token || '';
    }
    /**
     * Starts a new build process with the LibLab API.
     *
     * @param params - The build parameters, including liblabConfig, rawLiblabConfig, spec, and specFileFormat.
     * @returns Promise that resolves to the created build
     */
    async startBuild(params) {
        const api = await this.createApi(params.liblabConfig, params.spec, params.specFileFormat);
        const build = await this.postBuild(params.liblabConfig, params.rawLiblabConfig, api.id, params.hooks);
        return { build, api };
    }
    async createApi(liblabConfig, specContent, specFormat) {
        var _a, _b, _c, _d, _e, _f, _g;
        const currentUser = await this.getCurrentUser();
        const currentUserOrgId = (_d = (_c = (_b = (_a = currentUser === null || currentUser === void 0 ? void 0 : currentUser.orgs) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.org) === null || _c === void 0 ? void 0 : _c.id) !== null && _d !== void 0 ? _d : null;
        const form = (0, axios_1.toFormData)({
            name: liblabConfig.apiName,
            version: liblabConfig.apiVersion,
            languages: liblabConfig.languages,
            orgId: currentUserOrgId,
        });
        form.append('spec', Buffer.from(specContent), {
            filename: `spec.${specFormat}`,
            contentType: 'application/octet-stream',
        });
        try {
            const response = await axios_1.default.post(`${baseUrl}/apis`, form, {
                headers: {
                    Authorization: `Bearer ${this.token}`,
                    'Content-Type': 'multipart/form-data',
                },
            });
            return response.data;
        }
        catch (error) {
            if ((0, axios_1.isAxiosError)(error)) {
                const status = (_e = error.response) === null || _e === void 0 ? void 0 : _e.status;
                if (status !== axios_1.HttpStatusCode.Created) {
                    const errorMessages = (_g = (_f = error.response) === null || _f === void 0 ? void 0 : _f.data) === null || _g === void 0 ? void 0 : _g.message.join('\n');
                    throw new Error(`${errorMessages}\n\nPlease visit our support page for help: ${messages_1.supportLink}`);
                }
            }
            throw error;
        }
    }
    async getSubscriptionStatus() {
        var _a, _b;
        const currentUser = await (this === null || this === void 0 ? void 0 : this.getCurrentUser());
        const orgId = (_b = (_a = currentUser === null || currentUser === void 0 ? void 0 : currentUser.orgs) === null || _a === void 0 ? void 0 : _a[0].org) === null || _b === void 0 ? void 0 : _b.id;
        if (!orgId) {
            return null;
        }
        const subscription = await this.liblab.orgSubscriptions.getActiveSubscriptionStatus(orgId);
        return subscription;
    }
    async postBuild(liblabConfig, rawLiblabConfig, apiId, hooks) {
        const params = {
            ...liblabConfig,
            ...liblabConfig.customizations,
            multiTenant: true,
        };
        delete params.customizations;
        const payload = {
            apiId,
            deliveryMethods: ['zip'],
            languages: liblabConfig.languages,
            liblabVersion: liblabConfig.liblabVersion,
            sdkName: liblabConfig.sdkName,
            sdkVersion: liblabConfig.apiVersion || '1.0.0',
            buildType: ['SDK'],
            params: JSON.stringify(params),
            rawConfig: JSON.stringify(rawLiblabConfig),
        };
        if (hooks) {
            payload.hooks = hooks;
        }
        return this.liblab.build.createBuild(payload);
    }
}
exports.LiblabApi = LiblabApi;
