"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MintlifyIntegration = exports.LOCAL_OPENAPI_IN_MINTLIFY_REPO = exports.MINT_JSON_PATH = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = require("fs-extra");
const node_child_process_1 = require("node:child_process");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const paths_default_1 = require("../../../constants/paths.default");
const fs_1 = require("../fs");
exports.MINT_JSON_PATH = node_path_1.default.join(process.cwd(), paths_default_1.TMP_GIR_DIR_NAME, 'docs.json');
exports.LOCAL_OPENAPI_IN_MINTLIFY_REPO = node_path_1.default.join(paths_default_1.TMP_GIR_DIR_NAME, 'api-reference', 'openapi.json');
class MintlifyIntegration {
    constructor() {
        this.addExamplesToMintJson = (mintJson, openapiNavigation, liblabConfig) => {
            const mintlifyWithNavigation = this.getMintlifyObjectWithNavigation(mintJson, openapiNavigation);
            return this.updateMintlifyCodeSampleLanguages(mintlifyWithNavigation, liblabConfig);
        };
        this.fs = new fs_1.FsService();
        this.getMintJson = async () => {
            if (!(0, fs_extra_1.existsSync)(exports.MINT_JSON_PATH)) {
                throw new Error(`Failed to get mint.json file, file does not exists in path: ${exports.MINT_JSON_PATH}`);
            }
            try {
                return this.fs.readJsonFile(exports.MINT_JSON_PATH);
            }
            catch {
                throw new Error('Failed to read mint.json file');
            }
        };
    }
    async run(spec, liblabConfig) {
        const mintJson = await this.getMintJson();
        const { dirPath, fileName } = this.getOpenApiPath(mintJson);
        await this.fs.writeObjectFile(node_path_1.default.join(dirPath, fileName), spec);
        const openapiNavigation = this.getMintlifyNavigation(dirPath, fileName);
        const updatedMintJson = this.addExamplesToMintJson(mintJson, openapiNavigation, liblabConfig);
        await this.fs.writeJsonFile(exports.MINT_JSON_PATH, updatedMintJson);
    }
    getMintlifyNavigation(openApiDirPath, openApiFileName) {
        try {
            const relativePath = node_path_1.default.relative(process.cwd(), openApiDirPath);
            const scraperOutput = (0, node_child_process_1.execSync)(`npx @mintlify/scraping@latest openapi-file ${node_path_1.default.join(relativePath, openApiFileName)} -o ${relativePath}`, { encoding: 'utf8' });
            const openapiNavigation = JSON.parse(scraperOutput.replace('navigation object suggestion:', ''));
            return openapiNavigation;
        }
        catch {
            throw new Error('Mintlify integration failed to scrape openapi');
        }
    }
    getMintlifyObjectWithNavigation(mintJson, openapiNavigation) {
        // Create a copy of the original mint.json to avoid mutations
        const newMintJson = { ...mintJson };
        // Remove the temporary git directory prefix from navigation paths
        const cleanedNavigation = this.removeTemporaryGitPrefix(openapiNavigation);
        // Find where to insert the API reference section
        const apiReferenceIndex = newMintJson.navigation.tabs.findIndex((tab) => tab.tab === 'API Reference');
        // Replace or insert the API reference section
        const insertPosition = apiReferenceIndex === -1 ? 0 : apiReferenceIndex;
        newMintJson.navigation.tabs.splice(insertPosition, insertPosition === 0 ? 0 : 1, {
            tab: 'API Reference',
            groups: cleanedNavigation,
        });
        return newMintJson;
    }
    getOpenApiPath(mintJson) {
        const fullPath = mintJson.openapi
            ? node_path_1.default.join(paths_default_1.TMP_GIR_DIR_NAME, mintJson.openapi)
            : node_path_1.default.relative('.', exports.LOCAL_OPENAPI_IN_MINTLIFY_REPO);
        const fileName = node_path_1.default.normalize(fullPath).split(node_path_1.default.sep).pop() || '';
        const dirPath = node_path_1.default.normalize(fullPath).split(node_path_1.default.sep).slice(0, -1).join(node_path_1.default.sep);
        if (!(0, fs_extra_1.existsSync)(dirPath)) {
            (0, fs_extra_1.mkdirSync)(dirPath, { recursive: true });
        }
        return {
            fileName,
            dirPath,
        };
    }
    // npx @mintlify/scraping@latest openapi-file ./.temp-git/api-reference/openapi.json -o ./.temp-git/api-reference
    removeTemporaryGitPrefix(navigation) {
        return navigation.map((item) => ({
            ...item,
            pages: item.pages.map((page) => {
                const pageComponents = page.split(node_path_1.default.sep);
                const gitPrefixIndex = pageComponents.indexOf(paths_default_1.TMP_GIR_DIR_NAME);
                return pageComponents.slice(gitPrefixIndex + 1).join(node_path_1.default.sep);
            }),
        }));
    }
    /**
     * Modifies the Mintlify configuration object to include the specified code sample languages
     * @param mintJson - The original Mintlify configuration object
     * @param liblabConfigV2 - LibLab configuration containing integration settings
     * @returns Updated Mintlify configuration with modified language settings
     */
    updateMintlifyCodeSampleLanguages(mintJson, liblabConfigV2) {
        var _a, _b, _c, _d, _e, _f;
        const configuredLanguages = (_c = (_b = (_a = liblabConfigV2.integrations) === null || _a === void 0 ? void 0 : _a.enhancedApiSpec) === null || _b === void 0 ? void 0 : _b.mintlify) === null || _c === void 0 ? void 0 : _c.codeSamplesLanguages;
        const hasConfiguredLanguages = configuredLanguages && configuredLanguages.length > 0;
        const languages = hasConfiguredLanguages ? configuredLanguages : liblabConfigV2.languages;
        // Bash should always be included regardless of the selection of the user
        // This ensures curl examples are always available
        languages.unshift('bash');
        // Create new configuration object with updated language settings
        // while preserving all other existing configuration
        const updatedJson = {
            ...mintJson,
            api: {
                ...mintJson.api,
                request: {
                    ...(_d = mintJson.api) === null || _d === void 0 ? void 0 : _d.request,
                    example: {
                        ...(_f = (_e = mintJson.api) === null || _e === void 0 ? void 0 : _e.request) === null || _f === void 0 ? void 0 : _f.example,
                        languages,
                    },
                },
            },
        };
        return updatedJson;
    }
}
exports.MintlifyIntegration = MintlifyIntegration;
