"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.readFileFromPath = readFileFromPath;
exports.getLiblabConfigFile = getLiblabConfigFile;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
const node_fs_1 = require("node:fs");
const node_os_1 = require("node:os");
const node_path_1 = require("node:path");
const path_1 = require("./path");
async function readFileFromPath(path) {
    return (0, node_fs_1.readFileSync)(path, 'utf8');
}
/**
 * Fetches a LibLab config file from a remote URL.
 *
 * @param path - The URL of the LibLab config file to fetch
 * @returns A promise that resolves to the config file contents as a string
 * @throws {Error} If the request fails or times out after 20 seconds
 */
async function getLiblabConfigFileFromRemoteUrl(path) {
    const { data } = await axios_1.default.get(path, {
        responseType: 'text',
        timeout: 20000,
    });
    return data;
}
/**
 * Gets the contents of a LibLab config file from various sources based on the path type.
 *
 * @param path - The path to the file. Can be:
 *   - A URL (http/https/ftp)
 *   - An absolute path (OS-specific format)
 *   - A relative path from current working directory
 *   - A tilde path (~) representing user's home directory
 * @returns A promise that resolves to the file contents as a string
 * @throws {Error} If the file cannot be read or the request fails
 */
async function getLiblabConfigFile(path) {
    const specPathType = (0, path_1.identifyPathType)(path);
    switch (specPathType) {
        case path_1.PathType.RELATIVE_PATH: {
            return readFileFromPath((0, node_path_1.join)(process.cwd(), path));
        }
        case path_1.PathType.TILDE_PATH: {
            const homeDir = (0, node_os_1.homedir)();
            return readFileFromPath((0, node_path_1.join)(homeDir, path.slice(1)));
        }
        case path_1.PathType.URL: {
            return getLiblabConfigFileFromRemoteUrl(path);
        }
        default: {
            return readFileFromPath((0, node_path_1.resolve)(path));
        }
    }
}
