"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiblabConfigV2Service = void 0;
const liblab_config_v2_1 = require("../../liblab-config-v2");
const get_liblab_config_file_1 = require("./get-liblab-config-file");
/**
 * Service class for handling LibLab configuration files (v2).
 * Provides functionality to read, parse and validate LibLab configuration.
 */
class LiblabConfigV2Service {
    /**
     * Retrieves and processes a LibLab configuration file.
     *
     * @param path - Optional path to the config file. Defaults to './liblab.config.json'
     * @returns Promise containing the raw file contents, parsed config object, and validated config
     * @throws Error if file cannot be read or parsed
     */
    async getLiblabConfig(path, overrides, isPayingCustomer = false) {
        const fileContentsAsString = await this.getFile(path);
        const rawLiblabConfig = this.toObject(fileContentsAsString);
        const input = {
            ...rawLiblabConfig,
            ...overrides,
        };
        if (overrides.publishing && rawLiblabConfig.publishing) {
            input.publishing = {
                ...rawLiblabConfig.publishing,
                ...overrides.publishing,
            };
        }
        const liblabConfig = await this.toLiblabConfig(input, isPayingCustomer);
        return {
            fileContentsAsString,
            rawLiblabConfig,
            liblabConfig,
        };
    }
    /**
     * Reads a file from the given path.
     *
     * @param path - Path to the config file
     * @returns Promise containing the file contents as a string
     * @throws Error if file cannot be read
     */
    async getFile(path) {
        return (0, get_liblab_config_file_1.getLiblabConfigFile)(path);
    }
    /**
     * Validates and parses a raw config object against the LibLab v2 schema.
     *
     * @param rawLiblabConfig - Config object to validate
     * @returns Validated LibLabConfigV2 object
     * @throws Error if validation fails
     */
    async toLiblabConfig(rawLiblabConfig, isPayingCustomer) {
        return (0, liblab_config_v2_1.parse)(rawLiblabConfig, { isPayingCustomer });
    }
    /**
     * Converts a JSON string into an object.
     *
     * @param rawLiblabConfig - JSON string to parse
     * @returns Parsed object
     * @throws Error if JSON parsing fails
     */
    toObject(rawLiblabConfig) {
        return JSON.parse(rawLiblabConfig);
    }
}
exports.LiblabConfigV2Service = LiblabConfigV2Service;
