"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PathType = exports.WIN_32_ABSOLUTE_PATH_REGEX = void 0;
exports.isUrl = isUrl;
exports.identifyPathType = identifyPathType;
const node_os_1 = require("node:os");
const node_path_1 = require("node:path");
exports.WIN_32_ABSOLUTE_PATH_REGEX = /^[A-Za-z]:\\/;
const URL_REGEX = /^(https?|ftp):\/\/[^\s#$./?].\S*$/i;
function isUrl(path) {
    return URL_REGEX.test(path);
}
var PathType;
(function (PathType) {
    PathType["ABSOLUTE_PATH"] = "ABSOLUTE_PATH";
    PathType["RELATIVE_PATH"] = "RELATIVE_PATH";
    PathType["TILDE_PATH"] = "TILDE_PATH";
    PathType["URL"] = "URL";
})(PathType || (exports.PathType = PathType = {}));
/**
 * Identifies the type of path from a given string input.
 *
 * @param path - The path string to analyze. Can be:
 *   - A URL (http/https/ftp)
 *   - An absolute path (Windows or Unix format)
 *   - A relative path from current working directory
 *   - A tilde path (~) representing user's home directory
 * @returns The identified PathType enum value
 */
function identifyPathType(path) {
    // Check if the path is a URL
    if (isUrl(path)) {
        return PathType.URL;
    }
    // Handle different OS path conventions
    const currentPlatform = (0, node_os_1.platform)();
    const isWin = currentPlatform === 'win32';
    // Check for Windows-specific absolute paths
    if (isWin && exports.WIN_32_ABSOLUTE_PATH_REGEX.test(path)) {
        return PathType.ABSOLUTE_PATH;
    }
    // Check for Unix-specific absolute paths
    if (!isWin && (0, node_path_1.isAbsolute)(path)) {
        return PathType.ABSOLUTE_PATH;
    }
    // Check for tilde paths
    if (path.startsWith('~')) {
        return PathType.TILDE_PATH;
    }
    // If none of the above, it's a relative path
    return PathType.RELATIVE_PATH;
}
