"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BaseGitOperations = void 0;
const tslib_1 = require("tslib");
const isomorphic_git_1 = tslib_1.__importDefault(require("isomorphic-git"));
const node_1 = tslib_1.__importDefault(require("isomorphic-git/http/node"));
const node_fs_1 = tslib_1.__importDefault(require("node:fs"));
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const rimraf_1 = require("rimraf");
const paths_default_1 = require("../../../../constants/paths.default");
const rethrown_error_1 = require("../../../../utils/error/rethrown-error");
const constants_1 = require("./constants");
class BaseGitOperations {
    constructor(token, prData) {
        this.token = token;
        this.prData = prData;
    }
    async checkout() {
        try {
            isomorphic_git_1.default.branch({
                fs: node_fs_1.default,
                dir: constants_1.LOCAL_REPO_PATH,
                ref: this.prData.branchName,
                checkout: true,
                force: true,
            });
        }
        catch (error) {
            if (error instanceof isomorphic_git_1.default.Errors.NotFoundError) {
                throw new rethrown_error_1.RethrownError(`Target branch '${this.prData.branchName}' could not be found`, error);
            }
            throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to fetch target branch ${this.prData.branchName}`, error);
        }
    }
    async clean() {
        try {
            await (0, rimraf_1.rimraf)(constants_1.LOCAL_REPO_PATH);
        }
        catch {
            // if rimraf fails we don't need to do anything.
        }
    }
    async clone() {
        await isomorphic_git_1.default.clone({
            fs: node_fs_1.default,
            http: node_1.default,
            dir: node_path_1.default.join(process.cwd(), paths_default_1.TMP_GIR_DIR_NAME),
            url: this.prData.repoUrl,
            onAuth: () => ({
                username: this.prData.org,
                password: this.token,
            }),
        });
    }
    async commit() {
        try {
            await isomorphic_git_1.default.commit({
                fs: node_fs_1.default,
                dir: constants_1.LOCAL_REPO_PATH,
                message: this.prData.commitMessage,
                author: {
                    name: this.prData.authorName,
                    email: this.prData.authorEmail,
                },
            });
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError('Unexpected error occurred while trying to commit changes', error);
        }
    }
    async createNewBranch() {
        try {
            await isomorphic_git_1.default.branch({
                fs: node_fs_1.default,
                dir: constants_1.LOCAL_REPO_PATH,
                ref: this.prData.branchName,
                checkout: true,
            });
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to create branch ${this.prData.branchName}`, error);
        }
    }
    async push() {
        var _a;
        try {
            await isomorphic_git_1.default.push({
                fs: node_fs_1.default,
                http: node_1.default,
                dir: constants_1.LOCAL_REPO_PATH,
                ref: this.prData.branchName,
                onAuth: () => ({
                    username: this.prData.username,
                    password: this.token,
                }),
            });
        }
        catch (error) {
            if (!(error instanceof isomorphic_git_1.default.Errors.HttpError)) {
                throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to push changes to the branch ${this.prData.branchName}`, error);
            }
            switch ((_a = error === null || error === void 0 ? void 0 : error.data) === null || _a === void 0 ? void 0 : _a.statusCode) {
                case 401: {
                    throw new rethrown_error_1.RethrownError(`Unauthorized access to push changes to branch ${this.prData.branchName}`, error);
                }
                case 403: {
                    throw new rethrown_error_1.RethrownError(`Access to push changes to branch ${this.prData.branchName} is forbidden`, error);
                }
                default: {
                    throw new rethrown_error_1.RethrownError(`Unexpected error occurred while trying to push changes to the branch ${this.prData.branchName}`, error);
                }
            }
        }
    }
    async stage() {
        try {
            await isomorphic_git_1.default.add({ fs: node_fs_1.default, dir: constants_1.LOCAL_REPO_PATH, filepath: constants_1.ALL_FILEPATHS });
            for (const filepathToRemove of this.prData.filePathsToRemove) {
                await isomorphic_git_1.default.remove({ fs: node_fs_1.default, dir: constants_1.LOCAL_REPO_PATH, filepath: filepathToRemove });
            }
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError('Unexpected error occurred while trying to stage changes for commit', error);
        }
    }
}
exports.BaseGitOperations = BaseGitOperations;
