"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrWorkflow = void 0;
const core_1 = require("@oclif/core");
const bitbucket_git_operations_1 = require("./git-operations/bitbucket-git-operations");
const github_git_operations_1 = require("./git-operations/github-git-operations");
const gitlab_git_operations_1 = require("./git-operations/gitlab-git-operations");
const bitbucket_pr_data_1 = require("./pr-data/bitbucket-pr-data");
const github_pr_data_1 = require("./pr-data/github-pr-data");
const gitlab_pr_data_1 = require("./pr-data/gitlab-pr-data");
class PrWorkflow {
    static async createPr(gitPlatform, token, prData, makeChanges) {
        const gitOps = this.getGitOperations(gitPlatform, token, prData);
        await gitOps.clean();
        await gitOps.clone();
        await gitOps.checkout();
        await gitOps.createNewBranch();
        if (makeChanges) {
            await makeChanges();
        }
        await gitOps.stage();
        await gitOps.commit();
        await gitOps.push();
        const data = await gitOps.createPullRequest();
        return { htmlUrl: data.html_url };
    }
    static createPrForEachLangauge(gitPlatform, liblabConfig) {
        const token = this.getGitPlatformToken(gitPlatform);
        const promises = liblabConfig.languages.map(async (language) => {
            var _a;
            const prData = await this.getPrData((_a = liblabConfig.publishing) === null || _a === void 0 ? void 0 : _a.platform, token, language, liblabConfig);
            await prData.init();
            this.createPr(gitPlatform, token, prData);
        });
        return Promise.all(promises);
    }
    static getGitOperations(platform, token, prData) {
        switch (platform) {
            case 'bitbucket': {
                return new bitbucket_git_operations_1.BitbucketGitOperations(token, prData);
            }
            case 'gitlab': {
                return new gitlab_git_operations_1.GitlabGitOperations(token, prData);
            }
            default: {
                return new github_git_operations_1.GithubGitOperations(token, prData);
            }
        }
    }
    static getGitPlatformToken(platform) {
        switch (platform) {
            case 'bitbucket': {
                return process.env.LIBLAB_BITBUCKET_TOKEN || '';
            }
            case 'gitlab': {
                return process.env.LIBLAB_GITLAB_TOKEN || '';
            }
            default: {
                if (process.env.GITHUB_TOKEN && !process.env.LIBLAB_GITHUB_TOKEN) {
                    core_1.ux.stdout('Warning: Using the GITHUB_TOKEN environment variable with liblab is deprecated. Please use LIBLAB_GITHUB_TOKEN instead. \nSee our documentation for more details at https://developers.liblab.com/integrate-with-cicd/pr-creation/#the-pr-creation-process');
                }
                return process.env.LIBLAB_GITHUB_TOKEN || process.env.GITHUB_TOKEN || '';
            }
        }
    }
    static getPrData(gitPlatform, token, language, liblabConfig) {
        switch (gitPlatform) {
            case 'bitbucket': {
                const prData = new bitbucket_pr_data_1.BitbucketPrData(token, language, liblabConfig);
                prData.init();
                return prData;
            }
            case 'gitlab': {
                const prData = new gitlab_pr_data_1.GitlabPrData(token, language, liblabConfig);
                prData.init();
                return prData;
            }
            default: {
                const prData = new github_pr_data_1.GithubPrData(token, language, liblabConfig);
                prData.init();
                return prData;
            }
        }
    }
}
exports.PrWorkflow = PrWorkflow;
