"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiblabConfigService = void 0;
const tslib_1 = require("tslib");
const axios_1 = require("axios");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const zod_1 = require("zod");
const config_default_1 = tslib_1.__importDefault(require("../../constants/config.default"));
const languages_1 = tslib_1.__importDefault(require("../../constants/languages"));
const messages_1 = tslib_1.__importStar(require("../../constants/messages"));
const paths_default_1 = require("../../constants/paths.default");
const helpers_1 = require("../../lib/helpers");
const internal_json_1 = require("../../logs/internal.json");
const docs_schema_1 = require("../../schemas/docs.schema");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const file_utils_1 = require("../../utils/file.utils");
const json_utils_1 = require("../../utils/json.utils");
const text_utils_1 = require("../../utils/text.utils");
const liblab_config_validator_1 = require("../../validators/liblab-config/liblab-config-validator");
const constants_1 = require("./constants");
const schemas_1 = require("./schemas");
const RequiredDocsPropertiesForEnhancedApiSpec = [
    docs_schema_1.DocsOptions.Snippets,
    docs_schema_1.DocsOptions.EnhancedApiSpec,
];
class LiblabConfigService {
    constructor() {
        this.validator = new liblab_config_validator_1.LiblabConfigValidator();
    }
    async getRawLiblabConfig(pathAsOption) {
        const rawLiblabConfig = await this.getLiblabConfigFile(pathAsOption);
        (0, helpers_1.debugLog)('commands/build#run.parseConfig: ', {
            data: rawLiblabConfig,
            error: false,
            success: true,
        });
        return rawLiblabConfig;
    }
    getValidationIssues(liblabConfig) {
        return this.validator.getValidationIssues(liblabConfig);
    }
    handleMcp(liblabConfig, isPayingCustomer) {
        var _a, _b, _c, _d;
        if (!((_a = liblabConfig.mcp) === null || _a === void 0 ? void 0 : _a.generate) || !isPayingCustomer) {
            if ((_b = liblabConfig === null || liblabConfig === void 0 ? void 0 : liblabConfig.languages) === null || _b === void 0 ? void 0 : _b.length) {
                liblabConfig.languages.filter((l) => l !== 'mcp');
            }
            return liblabConfig;
        }
        const newLiblabConfig = { ...liblabConfig };
        newLiblabConfig.languages.push('mcp');
        if (!((_c = newLiblabConfig.languageOptions) === null || _c === void 0 ? void 0 : _c.typescript)) {
            newLiblabConfig.languageOptions = {
                ...newLiblabConfig.languageOptions,
                typescript: (_d = config_default_1.default.languageOptions) === null || _d === void 0 ? void 0 : _d.typescript,
            };
        }
        return newLiblabConfig;
    }
    modify(liblabConfig, languageFlag) {
        const configWithBuildFlag = this.updateConfigToIncludeParsedFlags(liblabConfig, languageFlag);
        const configWithAuthModifier = this.authConfigAuthModifier(configWithBuildFlag);
        const configWithDocsOptions = this.setDocumentationArray(configWithAuthModifier);
        return configWithDocsOptions;
    }
    validate(rawConfig) {
        return this.validator.validate(rawConfig);
    }
    validateMinimalConfig(liblabConfig) {
        return this.validator.validateMinimalValidateConfig(liblabConfig);
    }
    authConfigAuthModifier(liblabConfig) {
        var _a;
        const authArrayIsNotPopulatedOrNotDefined = !liblabConfig.auth || liblabConfig.auth.length === 0;
        const authenticationSettings = (_a = liblabConfig.customizations) === null || _a === void 0 ? void 0 : _a.authentication;
        if (!authenticationSettings && authArrayIsNotPopulatedOrNotDefined) {
            return liblabConfig;
        }
        if (authArrayIsNotPopulatedOrNotDefined && authenticationSettings) {
            throw new Error(internal_json_1.UNSUPPORTED_AUTH_CONFIGURATIONS_USAGE_OF_BASIC_AND_ACCESS);
        }
        this.validateAuthSettings(liblabConfig);
        return this.populateAuthConfigSettings(liblabConfig);
    }
    formatInvalidLanguageFlagError(error) {
        if (!(error instanceof zod_1.ZodError)) {
            return (0, messages_1.withContactSupport)(messages_1.default.ERROR.FAILED_PARSING_LANGUAGE_FLAG);
        }
        const enumIssue = error.errors.find((zodIssue) => zodIssue.code === 'invalid_enum_value');
        const receivedLanguage = enumIssue === null || enumIssue === void 0 ? void 0 : enumIssue.received;
        const baseMessage = `${(0, text_utils_1.getMarker)('error')} Invalid language ${receivedLanguage ? `"${receivedLanguage}"` : ''} provided. Supported languages are: ${languages_1.default.join(', ')}`;
        return (0, messages_1.withDocsLink)(baseMessage, constants_1.LANGUAGES_DOCS_URL);
    }
    async getLiblabConfigFile(pathAsOption) {
        try {
            const file = await (0, file_utils_1.getFile)(pathAsOption || paths_default_1.liblabConfigPath);
            return JSON.parse(file);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`Failed to get or parse the liblab config file`, error);
            if (error instanceof SyntaxError) {
                throw new TypeError(messages_1.default.ERROR.LIBLAB_CONFIG_INVALID.replace('{message}', error.message));
            }
            if (error.code === 'ENOENT') {
                throw new Error(messages_1.default.ERROR.LIBLAB_CONFIG_MISSING.replace('{path}', pathAsOption || paths_default_1.liblabConfigPath));
            }
            if (error.code === 'ENOTFOUND' || error instanceof axios_1.AxiosError) {
                throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.LIBLAB_CONFIG_UNABLE_TO_RETRIEVE_REMOTE_CONFIG.replace('{url}', pathAsOption || paths_default_1.liblabConfigPath), error);
            }
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.LIBLAB_CONFIG_UNKNOWN_ISSUE.replace('{error}', error.message), error);
        }
    }
    getValidatedLanguageFlag(languageFlag) {
        try {
            return schemas_1.languageFlagSchema.parse(languageFlag);
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(this.formatInvalidLanguageFlagError(error), error);
        }
    }
    populateAuthConfigSettings(liblabConfig) {
        const populatedConfig = { ...liblabConfig };
        const newAuth = new Set([]);
        populatedConfig.auth.forEach((authOption) => {
            var _a, _b, _c, _d, _e, _f;
            switch (authOption) {
                case 'apikey': {
                    newAuth.add('apikey');
                    if ((_c = (_b = (_a = populatedConfig.customizations) === null || _a === void 0 ? void 0 : _a.authentication) === null || _b === void 0 ? void 0 : _b.apiKey) === null || _c === void 0 ? void 0 : _c.header) {
                        break;
                    }
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.apiKey', {});
                    break;
                }
                case 'basic': {
                    newAuth.add('basic');
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.basic.enabled', true);
                    break;
                }
                case 'bearer': {
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.access.prefix', 'Bearer');
                    newAuth.add('bearer');
                    break;
                }
                case 'custom': {
                    const prefix = (_f = (_e = (_d = populatedConfig.customizations) === null || _d === void 0 ? void 0 : _d.authentication) === null || _e === void 0 ? void 0 : _e.access) === null || _f === void 0 ? void 0 : _f.prefix;
                    if ((prefix === null || prefix === void 0 ? void 0 : prefix.toLowerCase()) === 'bearer') {
                        lodash_1.default.set(populatedConfig, 'customizations.authentication.access.prefix', 'Bearer');
                        delete populatedConfig.customizations.authentication.custom;
                        newAuth.add('bearer');
                        return;
                    }
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.custom.prefix', prefix);
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.access.prefix', prefix);
                    newAuth.add('custom');
                    break;
                }
                case 'oauth': {
                    newAuth.add('oauth');
                    lodash_1.default.set(populatedConfig, 'customizations.authentication.oauth.enabled', true);
                    break;
                }
                default: {
                    break;
                }
            }
        });
        return { ...populatedConfig, auth: [...newAuth] };
    }
    populateDocsArrayForEnhancedApiSpec(liblabConfig) {
        var _a;
        const configWithValidatedEnhancedApiSpec = {
            ...liblabConfig,
            docs: (_a = liblabConfig.docs) !== null && _a !== void 0 ? _a : [],
        };
        RequiredDocsPropertiesForEnhancedApiSpec.forEach((docOption) => {
            if (!configWithValidatedEnhancedApiSpec.docs.includes(docOption)) {
                configWithValidatedEnhancedApiSpec.docs.push(docOption);
            }
        });
        return configWithValidatedEnhancedApiSpec;
    }
    setDocumentationArray(liblabConfig) {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        let configWithDocumentationArray = (0, json_utils_1.deepCopy)(liblabConfig);
        if ((((_b = (_a = configWithDocumentationArray.integrations) === null || _a === void 0 ? void 0 : _a.enhancedApiSpec) === null || _b === void 0 ? void 0 : _b.provider) &&
            ((_d = (_c = configWithDocumentationArray.integrations) === null || _c === void 0 ? void 0 : _c.enhancedApiSpec) === null || _d === void 0 ? void 0 : _d.githubRepoName)) ||
            ((_e = liblabConfig.docs) === null || _e === void 0 ? void 0 : _e.includes(docs_schema_1.DocsOptions.EnhancedApiSpec)) ||
            ((_h = (_g = (_f = configWithDocumentationArray.customizations) === null || _f === void 0 ? void 0 : _f.documentation) === null || _g === void 0 ? void 0 : _g.enhancedApiSpec) === null || _h === void 0 ? void 0 : _h.customOutputName)) {
            configWithDocumentationArray = this.populateDocsArrayForEnhancedApiSpec(configWithDocumentationArray);
        }
        return configWithDocumentationArray;
    }
    updateConfigToIncludeParsedFlags(liblabConfig, languageFlag) {
        const languages = languageFlag
            ? [this.getValidatedLanguageFlag(languageFlag)]
            : liblabConfig.languages;
        return { ...liblabConfig, languages };
    }
    /**
     * This is a temporary function that aims to normalize the liblab config sent to the API.
     * see ticket for more information: https://liblab.atlassian.net/browse/SER-844
     */
    validateAuthSettings(liblabConfig) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j;
        const authArrayIncludesBearer = (_a = liblabConfig.auth) === null || _a === void 0 ? void 0 : _a.includes('bearer');
        const authArrayIncludesBasic = (_b = liblabConfig.auth) === null || _b === void 0 ? void 0 : _b.includes('basic');
        const authArrayIncludesCustom = (_c = liblabConfig.auth) === null || _c === void 0 ? void 0 : _c.includes('custom');
        const authArrayIncludesApikey = (_d = liblabConfig.auth) === null || _d === void 0 ? void 0 : _d.includes('apikey');
        const apiKeyInAuthentication = (_f = (_e = liblabConfig.customizations) === null || _e === void 0 ? void 0 : _e.authentication) === null || _f === void 0 ? void 0 : _f.apiKey;
        const accessProperty = (_h = (_g = liblabConfig.customizations) === null || _g === void 0 ? void 0 : _g.authentication) === null || _h === void 0 ? void 0 : _h.access;
        const accessPrefix = (_j = accessProperty === null || accessProperty === void 0 ? void 0 : accessProperty.prefix) === null || _j === void 0 ? void 0 : _j.toLocaleLowerCase();
        const accessPrefixIsCustom = accessPrefix && accessPrefix !== 'bearer';
        const accessCustomizationIsAllowed = authArrayIncludesCustom || authArrayIncludesBearer;
        const forbiddenCombinations = [
            // forbidden combinations in auth array
            authArrayIncludesBearer && authArrayIncludesBasic,
            authArrayIncludesCustom && authArrayIncludesBasic,
            authArrayIncludesCustom && authArrayIncludesBearer,
            // access is required
            authArrayIncludesCustom && !accessProperty,
            // access cant have custom prefix if bearer is set in auth array
            authArrayIncludesBearer && accessPrefixIsCustom,
            // apikey is forbidden
            apiKeyInAuthentication && !authArrayIncludesApikey,
            // conditions with exactions
            authArrayIncludesApikey && Boolean(accessProperty) && !accessCustomizationIsAllowed,
            authArrayIncludesCustom && !accessPrefixIsCustom && !accessCustomizationIsAllowed,
        ];
        const isForbiddenConditionExists = forbiddenCombinations.some(Boolean);
        if (isForbiddenConditionExists) {
            (0, helpers_1.debugLog)('Invalid config detected, auth array or customizations contains illegal combination', liblabConfig);
            throw new Error(internal_json_1.UNSUPPORTED_AUTH_CONFIGURATIONS_USAGE_OF_BASIC_AND_ACCESS);
        }
    }
}
exports.LiblabConfigService = LiblabConfigService;
