"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const rimraf_1 = require("rimraf");
const messages_1 = tslib_1.__importDefault(require("../constants/messages"));
const paths_default_1 = require("../constants/paths.default");
const fs_1 = require("../lib/fs");
const bitbucket_client_1 = require("../lib/git/bitbucket-client");
const github_client_1 = require("../lib/git/github-client");
const gitlab_client_1 = require("../lib/git/gitlab-client");
const cmds_json_1 = require("../logs/cmds.json");
const construct_ui_array_1 = require("../utils/construct-ui-array");
const combined_error_1 = require("../utils/error/combined-error");
const rethrown_error_1 = require("../utils/error/rethrown-error");
const git_platform_token_1 = require("../utils/git-platform-token");
const normalize_sdk_names_1 = require("../utils/normalize-sdk-names");
const pr_utils_1 = require("../utils/pr.utils");
const missingTokenErrorMessage = {
    github: cmds_json_1.PR.ERROR.LIBLAB_GITHUB_TOKEN_MISSING,
    gitlab: cmds_json_1.PR.ERROR.LIBLAB_GITLAB_TOKEN_MISSING,
    bitbucket: cmds_json_1.PR.ERROR.LIBLAB_BITBUCKET_TOKEN_MISSING,
};
class PrService {
    constructor(liblabConfig, liblabClient, repoOwner, analytics, languageFlag, platform = 'github') {
        var _a;
        this.liblabConfig = liblabConfig;
        this.liblabClient = liblabClient;
        this.repoOwner = repoOwner;
        this.analytics = analytics;
        this.languageFlag = languageFlag;
        this.platform = platform;
        this.failedOperations = [];
        this.TEMP_GIT_DIRNAME = '.temp-git';
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.updateEventProperties({ config_data: this.liblabConfig });
        const liblabGitPlatformToken = (0, git_platform_token_1.getLiblabGitPlatformToken)(platform);
        if (!liblabGitPlatformToken) {
            if (!missingTokenErrorMessage[platform]) {
                throw new Error(`An unknown error occured with your git platform token. Please try again.`);
            }
            throw new Error(missingTokenErrorMessage[platform]);
        }
        this.gitPlatformClient = this.getGitPlatformClient(platform, repoOwner, liblabGitPlatformToken);
    }
    async run() {
        const { languages } = this.liblabConfig;
        await this.validateNoMissingSdks(languages);
        await this.gitPlatformClient.setupAuthenticatedUser();
        for (const prData of await this.getPrData(languages)) {
            try {
                this.logSeparatorLine(prData.language);
                await this.cloneAndPrepareLocalRepo(prData);
                await this.commitAndPushChanges(prData);
                await this.createPullRequest(prData);
            }
            catch (error) {
                this.handlePrError(error, prData);
            }
            finally {
                await this.submitAuditLog(prData);
                this.logSeparatorLine();
            }
        }
        if (this.failedOperations.length > 0) {
            const errorMessage = this.getFailedOperationsMessage();
            const errors = this.failedOperations.map((failedOperation) => failedOperation.error);
            throw new combined_error_1.CombinedError(errorMessage, errors);
        }
    }
    async cloneAndPrepareLocalRepo({ language, repoUrl, targetBranch }) {
        this.gitPlatformClient.localRepoPath = this.getTempLocalGitRepoPathForLanguage(language);
        // Just in case, make sure the temp folder is removed before cloning
        await this.removeTempGitFolder();
        await this.gitPlatformClient.clone(repoUrl);
        await this.gitPlatformClient.checkout(targetBranch);
    }
    async commitAndPushChanges({ language, repo, branchName, versions }) {
        this.logCommitNotifications({
            branchName,
            language,
            owner: this.repoOwner,
            repo,
        });
        await this.gitPlatformClient.createNewBranch(branchName);
        core_1.ux.stdout(cmds_json_1.PR.INFO.PR_COMMITTING_FILES);
        // Important: Order matters here. We need to remove the files first and then copy the new files.
        const filepathsToRemove = await this.getFilepathsForRemoval(language, this.gitPlatformClient.localRepoPath);
        this.copyOutputLanguageToLocalGitFolder(language, this.gitPlatformClient.localRepoPath);
        await this.gitPlatformClient.stageAll(filepathsToRemove);
        await this.gitPlatformClient.commit(this.getCommitMessage(versions.sdkVersion), this.gitPlatformClient.getCommitAuthor());
        await this.gitPlatformClient.push(branchName);
    }
    copyOutputLanguageToLocalGitFolder(language, tempLocalGitRepoPath) {
        try {
            const outputLanguageDir = node_path_1.default.join(paths_default_1.outputDir, language);
            (0, fs_1.copyRecursiveSync)(outputLanguageDir, tempLocalGitRepoPath);
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(`Error occurred while trying to copy output directory for ${language} to the local git repo`, error);
        }
    }
    async createPullRequest({ language, repo, targetBranch, branchName, versions }) {
        this.logPrMessage(targetBranch, branchName);
        const { html_url: prUrl } = await this.gitPlatformClient.createPullRequest({
            body: this.getPrBody(versions),
            branchName,
            owner: this.repoOwner,
            repo,
            targetBranch,
            title: this.getPrTitle(versions),
        });
        this.logSuccessPrMessage(language, prUrl);
        await this.removeTempGitFolder();
    }
    getBranchName(sdkVersion) {
        const currentDate = new Date();
        if (!sdkVersion) {
            return `liblab-codegen-${currentDate.getTime()}`;
        }
        return `v${sdkVersion}-${currentDate.getTime()}`;
    }
    getCommitMessage(sdkVersion) {
        return `liblab SDK update for version v${sdkVersion}`;
    }
    getFailedOperationsMessage() {
        const languages = this.failedOperations.map((failedOperation) => failedOperation.pr.language);
        const failedPrs = (0, construct_ui_array_1.constructUiArray)(languages);
        return cmds_json_1.PR.ERROR.PR_OPERATION_FAILED.replace('{operations}', failedPrs);
    }
    /**
     * Get the filepaths that need to be removed from the remote git repo for the given language.
     * It is calculated by comparing the local manifest file with the remote manifest file.
     * Files that are present in the remote manifest but not in the local manifest are return as the result.
     */
    async getFilepathsForRemoval(language, tempLocalGitRepoPath) {
        var _a;
        const localManifestPath = node_path_1.default.join(paths_default_1.outputDir, language, '.manifest.json');
        const remoteManifestPath = node_path_1.default.join(tempLocalGitRepoPath, '.manifest.json');
        try {
            return await (0, pr_utils_1.getLocalToRemoteManifestDiff)({
                language,
                localManifestPath,
                remoteManifestPath,
            });
        }
        catch (error) {
            (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.logEventToDatadog('warn', error);
            return [];
        }
    }
    getGitPlatformClient(platform, repoOwner, token) {
        switch (platform) {
            case 'bitbucket': {
                return new bitbucket_client_1.BitbucketClient(repoOwner, token);
            }
            case 'github': {
                return new github_client_1.GitHubClient(repoOwner, token);
            }
            case 'gitlab': {
                return new gitlab_client_1.GitLabClient(repoOwner, token);
            }
            default: {
                throw new Error(`Unsupported platform: ${platform}`);
            }
        }
    }
    getLocalTempGitRepoPath() {
        return node_path_1.default.join(process.cwd(), this.TEMP_GIT_DIRNAME);
    }
    getPrBody(versions) {
        return ('## Automatically generated PR with SDK updates by liblab.\n\n' +
            `- API version: **v${versions.apiVersion}**\n` +
            `- SDK version: **v${versions.sdkVersion}**`);
    }
    async getPrData(languages) {
        const sdkPrData = languages.reduce((acc, language) => {
            var _a, _b;
            const repo = ((_b = (_a = this.liblabConfig.languageOptions) === null || _a === void 0 ? void 0 : _a[language]) === null || _b === void 0 ? void 0 : _b.githubRepoName) || '';
            if (!repo) {
                const missingRepoNameForSdkMessage = messages_1.default.ERROR.NO_REPOSITORY_FOR_LANGUAGE(language);
                core_1.ux.stdout(missingRepoNameForSdkMessage);
                return acc;
            }
            const versions = this.getVersions(language);
            const repoUrl = this.gitPlatformClient.formatRepoUrl(this.repoOwner, repo);
            const prData = {
                branchName: this.getBranchName(versions.sdkVersion),
                language,
                repo,
                repoUrl,
                targetBranch: this.getTargetBranch(language),
                versions,
            };
            return [...acc, prData];
        }, []);
        if (sdkPrData.length === 0) {
            throw new Error(cmds_json_1.PR.INFO.NO_REPO_NAMES_IN_LIBLAB_CONFIG);
        }
        return sdkPrData;
    }
    getPrTitle(versions) {
        return `liblab SDK update v${versions.sdkVersion}`;
    }
    getTargetBranch(language) {
        var _a, _b;
        const targetBranch = (_b = (_a = this.liblabConfig.languageOptions) === null || _a === void 0 ? void 0 : _a[language]) === null || _b === void 0 ? void 0 : _b.targetBranch;
        return targetBranch || 'main';
    }
    getTempLocalGitRepoPathForLanguage(language) {
        return node_path_1.default.join(this.getLocalTempGitRepoPath(), language);
    }
    getVersions(language) {
        var _a, _b;
        const apiVersion = this.liblabConfig.apiVersion || '1.0.0';
        const sdkVersion = ((_b = (_a = this.liblabConfig.languageOptions) === null || _a === void 0 ? void 0 : _a[language]) === null || _b === void 0 ? void 0 : _b.sdkVersion) || apiVersion;
        return { sdkVersion, apiVersion };
    }
    handlePrError(error, prData) {
        var _a;
        (_a = this.analytics) === null || _a === void 0 ? void 0 : _a.logEventToDatadog('error', error);
        this.failedOperations.push({ pr: prData, error });
        core_1.ux.stdout(`Error occurred while creating PR for ${normalize_sdk_names_1.sdkNameToUiNameMapper[prData.language]}: ${error === null || error === void 0 ? void 0 : error.message}`);
    }
    logCommitNotifications({ branchName, language, owner, repo, }) {
        const creatingBranchLog = cmds_json_1.PR.INFO.PR_CREATING_NEW_BRANCH.replace('{language}', normalize_sdk_names_1.sdkNameToUiNameMapper[language]);
        core_1.ux.stdout(creatingBranchLog);
        const repoOwnerLog = cmds_json_1.PR.INFO.PR_OWNER.replace('{owner}', owner);
        core_1.ux.stdout(repoOwnerLog);
        const prRepoLog = cmds_json_1.PR.INFO.PR_REPOSITORY.replace('{repository}', repo);
        core_1.ux.stdout(prRepoLog);
        const prBranchLog = cmds_json_1.PR.INFO.PR_BRANCH.replace('{branchName}', branchName);
        core_1.ux.stdout(prBranchLog);
    }
    logPrMessage(targetBranch, branchName) {
        const createPrMessage = cmds_json_1.PR.INFO.PR_CREATING.replace('{targetBranch}', targetBranch).replace('{baseBranch}', branchName);
        core_1.ux.stdout(createPrMessage);
    }
    logSeparatorLine(language) {
        if (language) {
            core_1.ux.stdout(`\n- ${language.toUpperCase()} --------------------------------------`);
            return;
        }
        core_1.ux.stdout('---------------------------------------------------\n');
    }
    logSuccessPrMessage(language, url) {
        const message = cmds_json_1.PR.INFO.PR_CREATED.replace('{language}', normalize_sdk_names_1.sdkNameToUiNameMapper[language]);
        const messageWithUrl = `${message}: ${url}`;
        core_1.ux.stdout(messageWithUrl);
    }
    async removeTempGitFolder() {
        try {
            await (0, rimraf_1.rimraf)(this.getLocalTempGitRepoPath());
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(`Error occurred while trying to remove ${this.TEMP_GIT_DIRNAME} directory`, error);
        }
    }
    async submitAuditLog(prData) {
        const isFailedOperation = this.failedOperations
            .map((failedOperation) => failedOperation.pr.language)
            .includes(prData.language);
        const sdk = await (0, pr_utils_1.toSdkName)(this.liblabConfig);
        await this.liblabClient.trackPublishPREvent({
            language: prData.language,
            sdk,
            success: !isFailedOperation,
        });
    }
    async validateNoMissingSdks(languages) {
        const missingSdks = (0, pr_utils_1.findMissingSdks)(languages);
        if (missingSdks.length > 0) {
            const reason = messages_1.default.ERROR.NO_OUTPUT_FOR_SDK(missingSdks.join(', '));
            throw new Error(reason);
        }
    }
}
PrService.description = 'Create a PR for all the generated SDKs.';
PrService.examples = ['liblab pr -o {Github Organization Name}'];
exports.default = PrService;
