"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SpecFormatService = exports.AllowedSpecFormats = void 0;
const tslib_1 = require("tslib");
const lodash_1 = tslib_1.__importDefault(require("lodash"));
const semver_1 = require("semver");
const yaml_1 = tslib_1.__importDefault(require("yaml"));
const internal_json_1 = require("../logs/internal.json");
var AllowedSpecFormats;
(function (AllowedSpecFormats) {
    AllowedSpecFormats["OpenAPI"] = "openapi";
    AllowedSpecFormats["Postman"] = "postman";
    AllowedSpecFormats["Swagger"] = "swagger";
})(AllowedSpecFormats || (exports.AllowedSpecFormats = AllowedSpecFormats = {}));
const POSTMAN_SCHEMA_CHECK_REGEX = /^https?:\/\/schema.(get)?postman.com\//;
const MINMUM_SWAGGER_VERSION = '2.0.0';
const MINMUM_OPENAPI_VERSION = '3.0.0';
const ALLOWED_FORMATS_TOOLKIT = [
    {
        expectedKeyInSpec: 'info.schema',
        format: AllowedSpecFormats.Postman,
        getSpecName: (specAsJson) => lodash_1.default.get(specAsJson, 'info.name'),
        getSpecVersion: (specAsJson) => lodash_1.default.get(specAsJson, 'info.version'),
        validationFunc: (suspectedValue, fileFormat) => POSTMAN_SCHEMA_CHECK_REGEX.test(suspectedValue) && fileFormat === 'json',
    },
    {
        expectedKeyInSpec: 'swagger',
        format: AllowedSpecFormats.Swagger,
        getSpecName: (specAsJson) => lodash_1.default.get(specAsJson, 'info.title'),
        getSpecVersion: (specAsJson) => lodash_1.default.get(specAsJson, 'info.version'),
        validationFunc(suspectedValue) {
            const isValidSemver = (0, semver_1.valid)((0, semver_1.coerce)(suspectedValue));
            return isValidSemver !== null && (0, semver_1.gte)(isValidSemver, MINMUM_SWAGGER_VERSION);
        },
    },
    {
        expectedKeyInSpec: 'openapi',
        format: AllowedSpecFormats.OpenAPI,
        getSpecName: (specAsJson) => lodash_1.default.get(specAsJson, 'info.title'),
        getSpecVersion: (specAsJson) => lodash_1.default.get(specAsJson, 'info.version'),
        validationFunc(suspectedValue) {
            const isValidSemver = (0, semver_1.valid)((0, semver_1.coerce)(suspectedValue));
            return isValidSemver !== null && (0, semver_1.gte)(isValidSemver, MINMUM_OPENAPI_VERSION);
        },
    },
];
class SpecFormatService {
    constructor(specFileContents, fileFormat) {
        this.specFileContents = specFileContents;
        this.fileFormat = fileFormat;
    }
    getSpecFormat() {
        this.specAsJson = this.normalizedSpecToJs();
        this.foundFormatToolKit = ALLOWED_FORMATS_TOOLKIT.find(({ expectedKeyInSpec, validationFunc }) => {
            const suspectedValue = lodash_1.default.get(this.specAsJson, expectedKeyInSpec);
            return Boolean(suspectedValue && validationFunc(suspectedValue, this.fileFormat));
        });
        if (!this.foundFormatToolKit) {
            throw new Error(internal_json_1.UNSUPPORTED_SPEC);
        }
        return this.foundFormatToolKit.format;
    }
    getSpecName() {
        const toolKit = this.foundFormatToolKit;
        return toolKit.getSpecName(this.specAsJson);
    }
    getSpecTitleAndVersion(liblabConfig) {
        const { apiName, apiVersion } = {
            apiName: liblabConfig.apiName || this.getSpecName(),
            apiVersion: liblabConfig.apiVersion || this.getSpecVersion(),
        };
        if (!apiName || !apiVersion) {
            throw new Error('apiName and apiVersion are mandatory fields and must be included in liblab config OR provided spec');
        }
        return { apiName, apiVersion };
    }
    getSpecVersion() {
        const toolKit = this.foundFormatToolKit;
        return toolKit.getSpecVersion(this.specAsJson);
    }
    normalizedSpecToJs() {
        if (this.fileFormat === 'json') {
            return JSON.parse(this.specFileContents);
        }
        return yaml_1.default.parse(this.specFileContents);
    }
}
exports.SpecFormatService = SpecFormatService;
