"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SpecService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const axios_1 = require("axios");
const lodash_1 = require("lodash");
const yaml_1 = tslib_1.__importDefault(require("yaml"));
const messages_1 = tslib_1.__importDefault(require("../../constants/messages"));
const helpers_1 = require("../../lib/helpers");
const apply_oauth_customization_1 = require("../../utils/apply-oauth-customization");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const file_utils_1 = require("../../utils/file.utils");
const json_utils_1 = require("../../utils/json.utils");
const text_utils_1 = require("../../utils/text.utils");
const spec_format_service_1 = require("../spec-format.service");
class SpecService {
    async customize(spec, liblabConfig, specFileFormat, specFormat) {
        var _a, _b, _c, _d;
        const specAsJs = (0, file_utils_1.getParseFileContent)(spec);
        let customizedSpec = (0, lodash_1.cloneDeep)(specAsJs);
        const oauthCustomization = (_b = (_a = liblabConfig.customizations) === null || _a === void 0 ? void 0 : _a.authentication) === null || _b === void 0 ? void 0 : _b.oauth;
        if (oauthCustomization) {
            customizedSpec = this.customizeOauth(customizedSpec, oauthCustomization);
        }
        customizedSpec = await this.setEndpointCustomizations(customizedSpec, liblabConfig, specFormat);
        if ((_d = (_c = liblabConfig.customizations) === null || _c === void 0 ? void 0 : _c.readme) === null || _d === void 0 ? void 0 : _d.apiDescription) {
            customizedSpec.info.description = liblabConfig.customizations.readme.apiDescription;
        }
        if (specFormat === spec_format_service_1.AllowedSpecFormats.Postman) {
            customizedSpec = this.removeScriptsFromPostmanCollection(customizedSpec);
        }
        return this.convertToFormat(specFileFormat, customizedSpec);
    }
    async getSpecFile(liblabConfig) {
        try {
            return await (0, file_utils_1.getFile)(liblabConfig.specFilePath);
        }
        catch (specFileReadError) {
            (0, helpers_1.debugLog)('commands/validate#run.readFile: ', { specFileReadError });
            if (specFileReadError instanceof axios_1.AxiosError) {
                throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_URL_INVALID, specFileReadError);
            }
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_PATH_INVALID, specFileReadError);
        }
    }
    convertToFormat(format, spec) {
        return format === 'json' ? JSON.stringify(spec) : yaml_1.default.stringify(spec);
    }
    customizeOauth(specAsJs, oauthCustomization) {
        try {
            return (0, apply_oauth_customization_1.applyOauthCustomization)(specAsJs, oauthCustomization);
        }
        catch (error) {
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_TO_APPLY_OAUTH_CUSTOMIZATIONS, error);
        }
    }
    async getStringifiedEndpointCustomization(liblabConfig) {
        var _a;
        const endpointCustomization = (_a = liblabConfig.customizations) === null || _a === void 0 ? void 0 : _a.endpointCustomizations;
        // If there is no endpoint customization, return the spec as is
        if (!endpointCustomization) {
            return null;
        }
        // If endpointCustomization is a string, it is a path to a remote or local file
        if (typeof endpointCustomization === 'string') {
            try {
                const file = await (0, file_utils_1.getFile)(endpointCustomization);
                // convert to JSON and then stringify
                return JSON.stringify((0, file_utils_1.getParseFileContent)(file));
            }
            catch (endpointCustomizationGetFileError) {
                if (endpointCustomizationGetFileError instanceof axios_1.AxiosError) {
                    throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_TO_RETRIEVE_REMOTE_CUSTOMIZATIONS, endpointCustomizationGetFileError);
                }
                throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_TO_RETRIEVE_LOCAL_CUSTOMIZATIONS, endpointCustomizationGetFileError);
            }
        }
        return JSON.stringify(endpointCustomization);
    }
    removeScriptsFromPostmanCollection(collection) {
        const cloned = structuredClone(collection);
        function cleanItem(originalItem) {
            const item = { ...originalItem };
            if (item.event) {
                item.event = item.event.filter((event) => !['prerequest', 'test'].includes(event.listen));
            }
            if (Array.isArray(item.item)) {
                item.item = item.item.map((child) => cleanItem(child));
            }
            return item;
        }
        if (cloned.event) {
            cloned.event = cloned.event.filter((event) => !['prerequest', 'test'].includes(event.listen));
        }
        if (Array.isArray(cloned.item)) {
            cloned.item = cloned.item.map((element) => cleanItem(element));
        }
        return cloned;
    }
    /**
     * @param {string} spec - spec file content
     * @returns {string} - updated spec
     */
    async setEndpointCustomizations(specAsJs, liblabConfig, specFormat) {
        // postman collections should not be modified
        const stringifiedEndpointCustomization = await this.getStringifiedEndpointCustomization(liblabConfig);
        if (!stringifiedEndpointCustomization) {
            return specAsJs;
        }
        if (specFormat === spec_format_service_1.AllowedSpecFormats.Postman) {
            throw new Error(messages_1.default.ERROR.BUILD_FAILED_CUSTOMIZATIONS_IN_POSTMAN_COLLECTION);
        }
        const customizationsAsJs = (0, file_utils_1.getParseFileContent)(stringifiedEndpointCustomization);
        return this.updateSpecWithEndpointCustomizations(specAsJs, customizationsAsJs);
    }
    updateSpecWithEndpointCustomizations(parsedSpec, endpointCustomizationMapping) {
        const modifiedSpec = (0, json_utils_1.deepCopy)(parsedSpec);
        // Loop through the endpoint customization mapping and update the spec
        Object.entries(endpointCustomizationMapping).forEach(([path, methods]) => {
            const endpoint = modifiedSpec.paths[path];
            if (!endpoint) {
                throw new Error(`Endpoint ${path} not found in the spec`);
            }
            if (!methods) {
                throw new Error(`No methods found for the endpoint ${path}`);
            }
            Object.entries(methods).forEach(([method, customization]) => {
                if (!endpoint[method] || !customization) {
                    core_1.ux.stdout(`${(0, text_utils_1.getMarker)('warning')} Method ${method} not found in the endpoint ${path}`);
                    return;
                }
                const security = customization.scopes ? [{ ...customization.scopes }] : undefined;
                endpoint[method] = {
                    ...endpoint[method],
                    operationId: customization.operationId || endpoint[method].operationId,
                    'x-liblab-streaming': customization.streaming || endpoint[method]['x-liblab-streaming'],
                    ...(security && { security }),
                    'x-liblab-pagination': customization.pagination || endpoint[method]['x-liblab-pagination'],
                };
            });
        });
        return modifiedSpec;
    }
}
exports.SpecService = SpecService;
