"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ValidateService = exports.ValidateOption = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const spinnies_1 = tslib_1.__importDefault(require("@trufflesuite/spinnies"));
const fs_extra_1 = require("fs-extra");
const lodash_1 = require("lodash");
const paths_default_1 = require("../constants/paths.default");
const helpers_1 = require("../lib/helpers");
const liblab_1 = require("../lib/liblab");
const cmds_json_1 = require("../logs/cmds.json");
const sleep_1 = require("../utils/sleep");
const text_utils_1 = require("../utils/text.utils");
const format_violation_1 = require("../utils/violation/format-violation");
const wrapper_utils_1 = require("../utils/wrapper.utils");
var ValidateOption;
(function (ValidateOption) {
    ValidateOption["REVIEW"] = "review";
    ValidateOption["SKIP_ALL"] = "skip-all";
    ValidateOption["SKIP_ALL_IGNORE"] = "skip-all-ignore";
})(ValidateOption || (exports.ValidateOption = ValidateOption = {}));
var DiagnosisOption;
(function (DiagnosisOption) {
    DiagnosisOption["CONTINUE"] = "continue";
    DiagnosisOption["CONTINUE_ALL"] = "continue-all";
    DiagnosisOption["IGNORE"] = "ignore";
    DiagnosisOption["IGNORE_ALL"] = "ignore-all";
})(DiagnosisOption || (DiagnosisOption = {}));
const SPINNER_NAME = 'spec-validation';
class ValidateService {
    constructor(apiId, liblabConfig) {
        this.apiId = apiId;
        this.liblabConfig = liblabConfig;
        this.specViolations = [];
        this.specViolationsWithoutRulesToIgnore = [];
        this.continueRules = [];
        this.newRulesToIgnore = [];
        this.spinnies = new spinnies_1.default();
    }
    async addAllDiagnosticsToIgnoreList() {
        const distinctCodes = new Set(this.specViolations.map(({ code }) => code));
        this.newRulesToIgnore = Array.from(distinctCodes).map((code) => ({ code }));
    }
    // TODO: split the logic between exporting and erroring https://liblab.atlassian.net/jira/software/c/projects/SER/boards/2?assignee=63d82715a5d0c82630689a5e&selectedIssue=SER-2175&useStoredSettings=true
    async exportValidation() {
        let allRulesToIgnore = [...this.newRulesToIgnore];
        if (this.liblabConfig.validationsToIgnore) {
            allRulesToIgnore = [...allRulesToIgnore, ...this.liblabConfig.validationsToIgnore];
        }
        this.specViolationsWithoutRulesToIgnore = this.specViolations.filter((diagnostic) => {
            const ruleInIgnoreList = allRulesToIgnore.some((rule) => diagnostic.code === rule.code && rule.path === diagnostic.path);
            return !ruleInIgnoreList;
        });
        if (this.specViolationsWithoutRulesToIgnore.length > 0) {
            await this.saveValidationJson(this.specViolationsWithoutRulesToIgnore);
        }
        if (this.newRulesToIgnore.length > 0) {
            this.updateConfigWithValidationsToIgnore(allRulesToIgnore);
        }
        return this.getSeveritiesCount(this.specViolationsWithoutRulesToIgnore);
    }
    getViolations() {
        return this.specViolations;
    }
    async handleDiagnosisInteractively() {
        if (this.specViolations.length === 0) {
            return;
        }
        const orderedViolations = this.specViolations.reduce((acc, violation) => {
            if (violation.path) {
                acc.push(violation);
            }
            else {
                acc.unshift(violation);
            }
            return acc;
        }, []);
        const linterIssuesCount = orderedViolations.length;
        for (let violationIndex = 0; violationIndex < linterIssuesCount; violationIndex += 1) {
            const violation = orderedViolations[violationIndex];
            // rules are cascading from global to specific
            const continueRulesWithTheSameCode = this.continueRules.find(({ code }) => code === violation.code);
            if (continueRulesWithTheSameCode) {
                continue;
            }
            const ignoreRulesWithTheSameCode = this.newRulesToIgnore.filter(({ code }) => code === violation.code);
            const mostSpecificRule = ignoreRulesWithTheSameCode.reduce(
            // prioritize rules without path
            (acc, rule) => (rule.path && acc && !acc.path ? acc : rule), null);
            // it the rule is global and not specific
            if ((mostSpecificRule === null || mostSpecificRule === void 0 ? void 0 : mostSpecificRule.code) && !mostSpecificRule.path) {
                continue;
            }
            const remainingIssues = linterIssuesCount - violationIndex;
            const selectedOption = (await this.promptViolationSelect(violation, remainingIssues));
            this.handleSelectedValidationOption(selectedOption, violation);
        }
    }
    logViolations() {
        if (this.specViolations.length === 0) {
            return;
        }
        core_1.ux.stdout(`${cmds_json_1.BUILD.SPEC_ISSUES_FOUND.replaceAll('{issueCount}', `${this.specViolations.length}`)}\n`);
        const severityOrder = ['hint', 'info', 'warning', 'error'];
        this.specViolations.sort((a, b) => severityOrder.indexOf(a.severity) - severityOrder.indexOf(b.severity)); // order: hints -> info -> warnings -> errors
        const violationsGroupedByMessage = new Map();
        this.specViolations.forEach((violation) => {
            var _a;
            const { message } = violation;
            const existingViolations = (_a = violationsGroupedByMessage.get(message)) !== null && _a !== void 0 ? _a : [];
            violationsGroupedByMessage.set(message, [...existingViolations, violation]);
        });
        violationsGroupedByMessage.forEach((violations) => {
            core_1.ux.stdout((0, format_violation_1.formatViolationGroup)(violations, this.liblabConfig.specFilePath));
            core_1.ux.stdout();
        });
    }
    async promptValidationAction() {
        core_1.ux.stdout();
        const selectedOption = await wrapper_utils_1.inquirer.select({
            choices: [
                {
                    name: 'Skip all API spec issues',
                    value: ValidateOption.SKIP_ALL,
                },
                {
                    name: 'Skip all API spec issues and add all issues to the ignore list in the config file',
                    value: ValidateOption.SKIP_ALL_IGNORE,
                },
                {
                    name: 'Review all API spec issues',
                    value: ValidateOption.REVIEW,
                },
            ],
            message: 'What would you like to do with the validation issues?',
        });
        return selectedOption;
    }
    async validateSpec() {
        (0, helpers_1.debugLog)('Sending spec validation request', { apiId: this.apiId });
        let specValidationResponse = await liblab_1.client.specValidation.validateSpec({
            apiId: this.apiId,
        });
        this.spinnies.add(SPINNER_NAME, { prefixColor: 'yellow', text: 'Validating spec' });
        while (specValidationResponse.status === 'IN_PROGRESS') {
            (0, sleep_1.sleep)(3000);
            (0, helpers_1.debugLog)('Polling spec validation status', { specValidationId: specValidationResponse.id });
            specValidationResponse = await liblab_1.client.specValidation.getSpecValidation(specValidationResponse.id);
            (0, helpers_1.debugLog)('Obtained spec validation status', specValidationResponse);
        }
        if (specValidationResponse.status === 'FAILURE' || !specValidationResponse.result) {
            (0, helpers_1.debugLog)('Something went wrong while validating the spec', specValidationResponse);
            this.spinnies.fail(SPINNER_NAME, {
                prefixColor: 'red',
                text: 'Something went wrong while validating the spec',
            });
            throw new Error('Something went wrong while validating the spec');
        }
        if (specValidationResponse.result.some(({ severity }) => severity === 'error')) {
            this.spinnies.warn(SPINNER_NAME, {
                prefixColor: 'red',
                text: 'Validation succeeded with errors',
            });
        }
        else if (specValidationResponse.result.some(({ severity }) => severity === 'warning')) {
            this.spinnies.warn(SPINNER_NAME, {
                prefixColor: 'yellow',
                text: 'Validation succeeded with warnings',
            });
        }
        else {
            this.spinnies.succeed(SPINNER_NAME, { prefixColor: 'green', text: 'Validation succeeded' });
        }
        this.specViolations = specValidationResponse.result;
    }
    getSeveritiesCount(specViolationsWithoutRulesToIgnore) {
        return specViolationsWithoutRulesToIgnore.reduce((acc, { severity }) => {
            if (severity === 'error') {
                acc.errors += 1;
            }
            else if (severity === 'warning') {
                acc.warnings += 1;
            }
            return acc;
        }, { errors: 0, warnings: 0 });
    }
    handleSelectedValidationOption(selectedOption, violation) {
        switch (selectedOption) {
            case DiagnosisOption.CONTINUE: {
                break;
            }
            case DiagnosisOption.CONTINUE_ALL: {
                this.continueRules.push({ code: violation.code });
                break;
            }
            case DiagnosisOption.IGNORE: {
                this.newRulesToIgnore.push({ code: violation.code, path: violation.path });
                break;
            }
            case DiagnosisOption.IGNORE_ALL: {
                // remove all similar instances of ignore and put global ignore instead
                const rulesWithoutSpecificSimilarRule = this.newRulesToIgnore.filter(({ code }) => code !== violation.code);
                rulesWithoutSpecificSimilarRule.push({ code: violation.code });
                this.newRulesToIgnore = rulesWithoutSpecificSimilarRule;
                break;
            }
            default: {
                break;
            }
        }
    }
    async promptViolationSelect(violation, remainingIssues) {
        const { code, message, path: violationPath, range } = violation;
        core_1.ux.stdout(`--------------------------------------------------------------`);
        core_1.ux.stdout(`Remaining issues: ${remainingIssues}\n`);
        const startLine = range.start.line + 1;
        const endLine = range.start.line + 1;
        const isSingleLineRuleViolation = startLine === endLine;
        const lineText = isSingleLineRuleViolation
            ? `Line: ${startLine}\n`
            : `Lines: ${startLine} - ${endLine}\n`;
        const formattedPath = violationPath.join(' ~ ');
        core_1.ux.stdout(`Rule "${code}": ${message}`);
        core_1.ux.stdout(`At: ${formattedPath}`);
        core_1.ux.stdout(lineText);
        const selectedOption = await wrapper_utils_1.inquirer.select({
            choices: [
                {
                    description: 'Skip this validation for now, remind me on next build',
                    name: 'Continue without changes for now',
                    value: DiagnosisOption.CONTINUE,
                },
                {
                    description: `Skip all validations with code ${code} for now, remind me on next build`,
                    name: 'Continue without changes for all similar cases',
                    value: DiagnosisOption.CONTINUE_ALL,
                },
                {
                    description: 'Add this particular instance to ignore list',
                    name: 'Add this instance to the ignore list',
                    value: DiagnosisOption.IGNORE,
                },
                {
                    description: `Add all errors with code ${code} to ignore list`,
                    name: 'Add all similar errors to the ignore list',
                    value: DiagnosisOption.IGNORE_ALL,
                },
            ],
            message: 'What would you like to do?',
        });
        (0, helpers_1.debugLog)('Selected option', { diagnostic: violation, selectedOption });
        return selectedOption;
    }
    async saveValidationJson(specViolations) {
        try {
            await (0, fs_extra_1.ensureDir)(paths_default_1.outputDir);
            await (0, fs_extra_1.writeJson)(paths_default_1.apiSchemaValidationPath, specViolations, { spaces: 2 });
            const createdJsonMessage = (0, lodash_1.replace)(cmds_json_1.VALIDATION.VALIDATION_JSON_CREATED_MESSAGE, '{path}', (0, text_utils_1.getColoredText)('blue', paths_default_1.apiSchemaValidationPath));
            core_1.ux.stdout(`\n${createdJsonMessage}\n`);
        }
        catch (error) {
            (0, helpers_1.debugLog)(`commands/build#run.saveSchemaValidationLog`, error);
            throw new Error('Failed to save api-schema-validation locally');
        }
    }
    updateConfigWithValidationsToIgnore(validationsRulesUpdate) {
        (0, fs_extra_1.writeFileSync)(paths_default_1.liblabConfigPath, JSON.stringify({
            ...this.liblabConfig,
            validationsToIgnore: validationsRulesUpdate,
        }, undefined, 2));
    }
}
exports.ValidateService = ValidateService;
