"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getFileName = exports.getParseFileContent = exports.identifyFileContentFormat = exports.urlRegex = void 0;
exports.identifyStringType = identifyStringType;
exports.getFile = getFile;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_extra_1 = require("fs-extra");
const node_os_1 = require("node:os");
const node_path_1 = require("node:path");
const yaml_1 = tslib_1.__importDefault(require("yaml"));
const helpers_1 = require("../lib/helpers");
const internal_json_1 = require("../logs/internal.json");
const is_url_1 = require("./is-url");
const FETCH_MAX_TIME_OUT = 20 * 1000;
exports.urlRegex = /^(https?|ftp):\/\/[^\s#$./?].\S*$/i;
const identifyJson = (specFileContents) => {
    try {
        JSON.parse(specFileContents);
        return 'json';
    }
    catch (error) {
        (0, helpers_1.debugLog)('@identifyJson: failed to parse input as JSON', error);
        return null;
    }
};
const identifyYaml = (specFileContents) => {
    try {
        yaml_1.default.parse(specFileContents);
        return 'yaml';
    }
    catch (error) {
        (0, helpers_1.debugLog)('@identifyYaml: failed to parse input as yaml', error);
        return null;
    }
};
// Used to parse the file content
const parseJson = (specFileContents) => {
    try {
        return JSON.parse(specFileContents);
    }
    catch (error) {
        (0, helpers_1.debugLog)('@parseJson: failed to parse input as JSON', error);
        return null;
    }
};
const parseYaml = (specFileContents) => {
    try {
        return yaml_1.default.parse(specFileContents);
    }
    catch (error) {
        (0, helpers_1.debugLog)('@parseYaml: failed to parse input as yaml', error);
        return null;
    }
};
const identifyFileContentFormat = (specFileContents, error = internal_json_1.UNSUPPORTED_FILE_FORMAT) => {
    const foundFormat = identifyJson(specFileContents) || identifyYaml(specFileContents);
    if (!foundFormat) {
        throw new Error(error);
    }
    return foundFormat;
};
exports.identifyFileContentFormat = identifyFileContentFormat;
const getParseFileContent = (specFileContents) => {
    const fileContent = parseJson(specFileContents) || parseYaml(specFileContents);
    if (!fileContent) {
        throw new Error(internal_json_1.UNSUPPORTED_FILE_FORMAT);
    }
    return fileContent;
};
exports.getParseFileContent = getParseFileContent;
const getFileName = (filepath) => {
    const regex = /(?:.*[/\\])?(.*?)(\?.*)?$/;
    const matches = filepath.match(regex);
    return matches && matches[1] ? matches[1] : filepath;
};
exports.getFileName = getFileName;
function identifyStringType(input) {
    if ((0, is_url_1.isUrl)(input)) {
        return 'URL';
    }
    // Handle different OS path conventions
    const currentPlatform = (0, node_os_1.platform)();
    const isWin = currentPlatform === 'win32';
    // Check for Windows-specific absolute paths
    if (isWin && /^[A-Za-z]:\\/.test(input)) {
        return 'Absolute Path';
    }
    // Check for Unix-specific absolute paths
    if (!isWin && (0, node_path_1.isAbsolute)(input)) {
        return 'Absolute Path';
    }
    if (input.startsWith('~')) {
        return 'Tilde Path';
    }
    return 'Relative Path';
}
async function getFile(specFilePath) {
    const specPathType = identifyStringType(specFilePath);
    switch (specPathType) {
        case 'Relative Path': {
            return (0, fs_extra_1.readFileSync)((0, node_path_1.join)(process.cwd(), specFilePath), 'utf8');
        }
        case 'Tilde Path': {
            const homeDir = (0, node_os_1.homedir)();
            return (0, fs_extra_1.readFileSync)((0, node_path_1.join)(homeDir, specFilePath.slice(1)), 'utf8');
        }
        case 'URL': {
            const { data: specFileData } = await axios_1.default.get(specFilePath, {
                responseType: 'text',
                timeout: FETCH_MAX_TIME_OUT,
                transformResponse: (res) => res,
            });
            return specFileData;
        }
        default: {
            return (0, fs_extra_1.readFileSync)((0, node_path_1.resolve)(specFilePath), 'utf8');
        }
    }
}
