"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Hooks = void 0;
const tslib_1 = require("tslib");
const axios_1 = tslib_1.__importDefault(require("axios"));
const fs_extra_1 = require("fs-extra");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const env_1 = tslib_1.__importDefault(require("../constants/env"));
const paths_default_1 = require("../constants/paths.default");
const fs_1 = require("../lib/fs");
const helpers_1 = require("../lib/helpers");
var TypescriptHookVersions;
(function (TypescriptHookVersions) {
    TypescriptHookVersions["VERSION_1"] = "v1.0.0";
    TypescriptHookVersions["VERSION_2"] = "v2.0.0";
})(TypescriptHookVersions || (TypescriptHookVersions = {}));
const languageVersionToHookVersion = {
    typescript: {
        '1': TypescriptHookVersions.VERSION_1,
        '2': TypescriptHookVersions.VERSION_2,
    },
    python: {},
    java: {},
    go: {},
    csharp: {},
    php: {},
    swift: {},
    terraform: {},
    mcp: {},
};
class Hooks {
    constructor(hooksZipBuffer) {
        this.hooksZipBuffer = hooksZipBuffer;
    }
    static alreadyExist(languageConfigs) {
        return languageConfigs.some((languageConfig) => languageConfig.language === 'terraform'
            ? (0, fs_1.exists)(paths_default_1.customPlanModifiersPath) || (0, fs_1.exists)(node_path_1.default.join(paths_default_1.hooksDir, languageConfig.language))
            : (0, fs_1.exists)(node_path_1.default.join(paths_default_1.hooksDir, languageConfig.language)));
    }
    static async getHooksFromS3() {
        try {
            const { data: hooksBuffer } = await (0, axios_1.default)({
                method: 'GET',
                responseType: 'arraybuffer',
                url: env_1.default.hookURL,
            });
            return new Hooks(hooksBuffer);
        }
        catch (error) {
            (0, helpers_1.debugLog)('Failed to fetch hooks from s3 bucket', error);
            throw new Error('Could not fetch hooks');
        }
    }
    extract(languageConfig) {
        if (languageConfig.language === 'terraform') {
            this.unzipTerraformHooks();
        }
        else {
            this.unzipOpenApiSdkHooks(languageConfig);
        }
    }
    unzipOpenApiSdkHooks(languageWithVersion) {
        var _a;
        if (languageWithVersion.language === 'typescript') {
            (0, fs_extra_1.emptyDirSync)(`${paths_default_1.hooksDir}/${languageWithVersion.language}`);
            const hookVersion = (_a = languageVersionToHookVersion.typescript) === null || _a === void 0 ? void 0 : _a[languageWithVersion.version];
            if (!hookVersion) {
                (0, helpers_1.debugLog)(`Invalid hook version for langauge. Language: ${languageWithVersion.language}. Version: ${languageWithVersion.version}`);
                throw new Error(`Could not fetch ${languageWithVersion.language} hooks for this version ${languageWithVersion.version}`);
            }
            (0, fs_1.unzipEntry)(this.hooksZipBuffer, `typescript_/${hookVersion}/`, `${paths_default_1.hooksDir}/tmp`);
            // NOTE: This step is necessary because the current version of adm-zip (v0.5.14) cannot copy subdirectories when maintainEntryPath is set to false.
            // Therefore we need to unzip into a tmp directory, copy it to the correct location, and then delete the tmp directory.
            (0, fs_1.copyAndRemove)(`${paths_default_1.hooksDir}/tmp/typescript_/${hookVersion}`, `${paths_default_1.hooksDir}/typescript`, `${paths_default_1.hooksDir}/tmp/`);
        }
        else {
            (0, fs_1.unzipEntry)(this.hooksZipBuffer, `${languageWithVersion.language}/`, paths_default_1.hooksDir);
        }
    }
    unzipTerraformHooks() {
        // terraform uses a go sdk under the hood and we can add go hooks to it
        (0, fs_1.unzipEntry)(this.hooksZipBuffer, `terraform/hooks/`, node_path_1.default.join(paths_default_1.hooksDir, 'terraform', 'hooks'), false);
        (0, fs_1.unzipEntry)(this.hooksZipBuffer, `terraform/customPlanModifiers/resources/`, node_path_1.default.join(paths_default_1.customPlanModifiersPath, 'resources'), false);
    }
}
exports.Hooks = Hooks;
