"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLocalToRemoteManifestDiff = getLocalToRemoteManifestDiff;
exports.findMissingSdks = findMissingSdks;
exports.toSdkName = toSdkName;
const tslib_1 = require("tslib");
const axios_1 = require("axios");
const promises_1 = require("node:fs/promises");
const node_path_1 = tslib_1.__importDefault(require("node:path"));
const messages_1 = tslib_1.__importDefault(require("../constants/messages"));
const paths_default_1 = require("../constants/paths.default");
const fs_1 = require("../lib/fs");
const helpers_1 = require("../lib/helpers");
const spec_format_service_1 = require("../services/spec-format.service");
const rethrown_error_1 = require("./error/rethrown-error");
const file_utils_1 = require("./file.utils");
const getSpecFile = async (specFilePath) => {
    try {
        return await (0, file_utils_1.getFile)(specFilePath);
    }
    catch (specFileReadError) {
        (0, helpers_1.debugLog)('commands/validate#run.readFile: ', { specFileReadError });
        if (specFileReadError instanceof axios_1.AxiosError) {
            throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_URL_INVALID, specFileReadError);
        }
        throw new rethrown_error_1.RethrownError(messages_1.default.ERROR.BUILD_FAILED_SPEC_PATH_INVALID, specFileReadError);
    }
};
function parseManifest(manifestJson, language) {
    try {
        return JSON.parse(manifestJson);
    }
    catch {
        throw new Error(`Failed to parse manifest for language: ${language}`);
    }
}
async function getLocalToRemoteManifestDiff(params) {
    if (!(0, fs_1.exists)(params.localManifestPath)) {
        throw new Error(`Local manifest file not found at: ${params.localManifestPath}`);
    }
    if (!(0, fs_1.exists)(params.remoteManifestPath)) {
        return [];
    }
    try {
        const localManifestJson = await (0, promises_1.readFile)(params.localManifestPath, 'utf8');
        const remoteManifestJson = await (0, promises_1.readFile)(params.remoteManifestPath, 'utf8');
        const localManifest = parseManifest(localManifestJson, params.language);
        const remoteManifest = parseManifest(remoteManifestJson, params.language);
        return remoteManifest.files.filter((filePath) => !localManifest.files.includes(filePath));
    }
    catch (error) {
        throw new rethrown_error_1.RethrownError('Failed to get local to remote manifest diff', error);
    }
}
/**
 * Returns a list of sdks that are missing from the output directory structure.
 * @param expectedLanguages List of languages that are expected to exist in the output directory.
 */
function findMissingSdks(expectedLanguages) {
    return expectedLanguages.reduce((missingSdks, language) => {
        const langSdkPath = node_path_1.default.join(paths_default_1.outputDir, language);
        if (!(0, fs_1.exists)(langSdkPath) || (0, fs_1.isEmpty)(langSdkPath)) {
            missingSdks.push(language);
        }
        return missingSdks;
    }, []);
}
async function toSdkName(liblabConfig) {
    if (liblabConfig.sdkName) {
        return liblabConfig.sdkName;
    }
    if (liblabConfig.apiName) {
        return liblabConfig.apiName;
    }
    // retrieve name from the spec file if not in the liblab config
    const { specFilePath } = liblabConfig;
    const specFileContents = await getSpecFile(specFilePath);
    const fileFormat = (0, file_utils_1.identifyFileContentFormat)(specFileContents);
    const identifier = new spec_format_service_1.SpecFormatService(specFileContents, fileFormat);
    await identifier.getSpecFormat();
    const { apiName } = identifier.getSpecTitleAndVersion(liblabConfig);
    return apiName;
}
