"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LiblabConfigValidator = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@oclif/core");
const zod_1 = require("zod");
const messages_1 = tslib_1.__importStar(require("../../constants/messages"));
const docs_schema_1 = require("../../schemas/docs.schema");
const liblab_config_1 = require("../../schemas/liblab.config");
const minimal_liblab_config_1 = require("../../schemas/minimal-liblab-config");
const rethrown_error_1 = require("../../utils/error/rethrown-error");
const normalize_sdk_names_1 = require("../../utils/normalize-sdk-names");
const text_utils_1 = require("../../utils/text.utils");
const liblab_config_warnings_1 = require("./liblab-config-warnings");
const MINTLIFY_NATIVE_LANGUAGES = ['bash', 'go', 'java', 'javascript', 'php', 'python'];
const MINTLIFY_AND_LIBLAB_SUPPORTED_LANGUAGES = new Set([
    ...MINTLIFY_NATIVE_LANGUAGES,
    ...Object.entries(normalize_sdk_names_1.sdkNameToUiNameMapper).flat(),
]);
class LiblabConfigValidator {
    constructor() {
        this.CONFIG_DOCS_URL = 'https://developers.liblab.com/cli/config-file-overview-root/';
    }
    /**
     * Get validation errors and warnings for the liblab config.
     * @param configData liblab config data
     * @returns validation issues (lists of errors and warnings and a formatted message)
     */
    getValidationIssues(configData) {
        try {
            const validatedConfig = liblab_config_1.liblabSchema.parse(configData);
            const warnings = this.getWarnings(validatedConfig);
            const result = warnings.length > 0 ? 'warning' : 'success';
            return {
                errors: [],
                formattedIssuesMessage: this.formatIssuesMessage(warnings, 'warning'),
                result,
                warnings,
            };
        }
        catch (error) {
            if (!(error instanceof zod_1.ZodError)) {
                throw this.getGenericError(error);
            }
            const errors = this.getErrors(error.errors);
            const warnings = this.getWarnings(configData);
            const formattedIssuesMessage = this.getFormattedIssuesMessage(errors, warnings);
            return {
                errors,
                formattedIssuesMessage,
                result: 'error',
                warnings,
            };
        }
    }
    /**
     * Validate liblab config against the schema.
     * If the config has warnings, it will log them.
     * @throws {RethrownError} if the config has invalid properties
     * @param configData liblab config data
     */
    validate(configData) {
        try {
            const validatedConfig = liblab_config_1.liblabSchema.parse(configData);
            this.logWarnings(validatedConfig);
            return validatedConfig;
        }
        catch (error) {
            if (!(error instanceof zod_1.ZodError)) {
                throw this.getGenericError(error);
            }
            const errors = this.getErrors(error.errors);
            const warnings = this.getWarnings(configData);
            const formattedIssuesMessage = this.getFormattedIssuesMessage(errors, warnings);
            const errorMessage = messages_1.default.ERROR.LIBLAB_CONFIG_ISSUES_DETECTED(formattedIssuesMessage, errors.length, warnings.length);
            throw new rethrown_error_1.RethrownError((0, messages_1.withDocsLink)(errorMessage, this.CONFIG_DOCS_URL), error);
        }
    }
    /**
     * Validate config against the minimal liblab validate config schema.
     * Config needs to satisfy the minimal schema in order for validate command to continue validating the spec file.
     * @throws {RethrownError} if the config has invalid properties
     * @param configData liblab config data
     */
    validateMinimalValidateConfig(configData) {
        try {
            return minimal_liblab_config_1.minimalLiblabValidateConfigSchema.parse(configData);
        }
        catch (error) {
            if (!(error instanceof zod_1.ZodError)) {
                throw this.getGenericError(error);
            }
            const errors = this.getErrors(error.errors);
            const formattedIssuesMessage = this.formatIssuesMessage(errors, 'error');
            const errorMessage = messages_1.default.ERROR.LIBLAB_CONFIG_ISSUES_DETECTED(formattedIssuesMessage, errors.length, 0);
            throw new rethrown_error_1.RethrownError((0, messages_1.withDocsLink)(errorMessage, this.CONFIG_DOCS_URL), error);
        }
    }
    formatIssuesMessage(issues, severity) {
        return issues
            .map(({ message, property }) => `${(0, text_utils_1.getMarker)(severity)} ${property ? `Property "${property}": ` : ''}${message}`)
            .join('\n');
    }
    getErrors(errors) {
        return errors.map(({ message, path }) => ({
            message,
            property: path.length > 0 ? path.join('.') : '',
        }));
    }
    getFormattedIssuesMessage(errors, warnings) {
        const emptyLine = warnings.length > 0 ? '\n' : '';
        return `${this.formatIssuesMessage(errors, 'error')}${emptyLine}${this.formatIssuesMessage(warnings, 'warning')}`;
    }
    getGenericError(error) {
        return new rethrown_error_1.RethrownError((0, messages_1.withContactSupport)(messages_1.default.ERROR.LIBLAB_CONFIG_INVALID), error);
    }
    getUnsupportedLanguagesForMintlifyIntegrationWarnings(liblabConfig) {
        var _a, _b, _c, _d, _e, _f;
        if (!((_c = (_b = (_a = liblabConfig.integrations) === null || _a === void 0 ? void 0 : _a.enhancedApiSpec) === null || _b === void 0 ? void 0 : _b.mintlify) === null || _c === void 0 ? void 0 : _c.codeSamplesLanguages)) {
            return null;
        }
        const mintlifyIntegrationLanguagesWarnings = (_f = (_e = (_d = liblabConfig.integrations) === null || _d === void 0 ? void 0 : _d.enhancedApiSpec) === null || _e === void 0 ? void 0 : _e.mintlify) === null || _f === void 0 ? void 0 : _f.codeSamplesLanguages.filter((language) => !MINTLIFY_AND_LIBLAB_SUPPORTED_LANGUAGES.has(language));
        if (mintlifyIntegrationLanguagesWarnings.length === 0) {
            return null;
        }
        return mintlifyIntegrationLanguagesWarnings.map((language, i) => ({
            message: `liblab config language without liblab snippets support nor native Mintlify code snippets: ${language}`,
            property: `liblabConfig.integrations?.enhancedApiSpec.mintlify.codeSamplesLanguages[${i}]`,
        }));
    }
    getWarningForMissingIntegrationOptions(liblabConfig) {
        var _a, _b;
        const warnings = [];
        if (!((_a = liblabConfig.publishing) === null || _a === void 0 ? void 0 : _a.githubOrg) && ((_b = liblabConfig.integrations) === null || _b === void 0 ? void 0 : _b.enhancedApiSpec)) {
            warnings.push({
                message: `liblab config contains an integration object for provider: ${liblabConfig.integrations.enhancedApiSpec.provider} but has invalid github repo, liblab will not be able to integrate without a valid github repo name.`,
                property: 'publishing.githubOrg',
            });
        }
        const unsupportedLanguagesForMintlifyIntegrationWarnings = this.getUnsupportedLanguagesForMintlifyIntegrationWarnings(liblabConfig);
        if (unsupportedLanguagesForMintlifyIntegrationWarnings) {
            unsupportedLanguagesForMintlifyIntegrationWarnings.forEach((warning) => warnings.push(warning));
        }
        return warnings;
    }
    getWarnings(configData) {
        var _a, _b;
        const warnings = [];
        const createDocsDeprecationWarning = (0, liblab_config_warnings_1.getCreateDocsDeprecationWarning)(configData.createDocs);
        if (createDocsDeprecationWarning) {
            warnings.push(createDocsDeprecationWarning);
        }
        const languagesWithoutDocSupportWarning = (0, liblab_config_warnings_1.getLanguagesWithoutDocSupportWarning)(configData);
        if (languagesWithoutDocSupportWarning) {
            warnings.push(languagesWithoutDocSupportWarning);
        }
        if ((_a = configData.docs) === null || _a === void 0 ? void 0 : _a.includes(docs_schema_1.DocsOptions.EnhancedApiSpec)) {
            (_b = (0, liblab_config_warnings_1.getLanguagesWithoutEnhancedApiSpecSupportWarning)(configData)) === null || _b === void 0 ? void 0 : _b.forEach((warning) => warnings.push(warning));
        }
        const integrationWarning = this.getWarningForMissingIntegrationOptions(configData);
        return [...warnings, ...integrationWarning];
    }
    logWarnings(validatedConfig) {
        const warnings = this.getWarnings(validatedConfig);
        if (warnings.length === 0) {
            core_1.ux.stdout(`${(0, text_utils_1.getMarker)('success')} ${messages_1.default.INFO.LIBLAB_CONFIG_VALID}\n`);
        }
        else {
            const message = this.formatIssuesMessage(warnings, 'warning');
            core_1.ux.stdout(`${messages_1.default.ERROR.LIBLAB_CONFIG_ISSUES_DETECTED(message, 0, warnings.length)}\n`);
        }
    }
}
exports.LiblabConfigValidator = LiblabConfigValidator;
